@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CheatCodeDialog.cxx,v 1.9 2006/03/23 16:16:32 stephena Exp $
+// $Id: CheatCodeDialog.cxx,v 1.17 2008/02/06 13:45:19 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,29 +21,19 @@
 
 #include <sstream>
 
-#include "OSystem.hxx"
-#include "Props.hxx"
-#include "Widget.hxx"
-#include "Dialog.hxx"
-#include "DialogContainer.hxx"
-#include "CheatCodeDialog.hxx"
-#include "GuiUtils.hxx"
-#include "CheckListWidget.hxx"
-#include "CheatManager.hxx"
-#include "InputTextDialog.hxx"
-#include "StringList.hxx"
-
 #include "bspf.hxx"
 
-enum {
-  kAddCheatCmd       = 'CHTa',
-  kEditCheatCmd      = 'CHTe',
-  kAddOneShotCmd     = 'CHTo',
-  kCheatAdded        = 'CHad',
-  kCheatEdited       = 'CHed',
-  kOneShotCheatAdded = 'CHoa',
-  kRemCheatCmd       = 'CHTr'
-};
+#include "CheatManager.hxx"
+#include "CheckListWidget.hxx"
+#include "DialogContainer.hxx"
+#include "Dialog.hxx"
+#include "InputTextDialog.hxx"
+#include "OSystem.hxx"
+#include "Props.hxx"
+#include "StringList.hxx"
+#include "Widget.hxx"
+
+#include "CheatCodeDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CheatCodeDialog::CheatCodeDialog(OSystem* osystem, DialogContainer* parent,
@@ -51,6 +41,8 @@ CheatCodeDialog::CheatCodeDialog(OSystem* osystem, DialogContainer* parent,
   : Dialog(osystem, parent, x, y, w, h)
 {
   int xpos, ypos;
+  WidgetArray wid;
+  ButtonWidget* b;
 
   // List of cheats, with checkboxes to enable/disable
   xpos = 10;  ypos = 10;
@@ -58,32 +50,45 @@ CheatCodeDialog::CheatCodeDialog(OSystem* osystem, DialogContainer* parent,
                                     _w - 25 - kButtonWidth, _h - 50);
   myCheatList->setStyle(kXFill);
   myCheatList->setEditable(false);
-  myCheatList->setFlags(WIDGET_NODRAW_FOCUS);
-  addFocusWidget(myCheatList);
+  wid.push_back(myCheatList);
 
   xpos += myCheatList->getWidth() + 15;  ypos = 15;
-  addButton(font, xpos, ypos, "Add", kAddCheatCmd, 0);
-  myEditButton = addButton(font, xpos, ypos+=20, "Edit", kEditCheatCmd, 0);
-  myRemoveButton = addButton(font, xpos, ypos+=20, "Remove", kRemCheatCmd, 0);
-  addButton(font, xpos, ypos+=30, "One shot", kAddOneShotCmd, 0);
+  b = addButton(font, xpos, ypos, "Add", kAddCheatCmd);
+  wid.push_back(b);
+  myEditButton = addButton(font, xpos, ypos+=20, "Edit", kEditCheatCmd);
+  wid.push_back(myEditButton);
+  myRemoveButton = addButton(font, xpos, ypos+=20, "Remove", kRemCheatCmd);
+  wid.push_back(myRemoveButton);
+  b = addButton(font, xpos, ypos+=30, "One shot", kAddOneShotCmd);
+  wid.push_back(b);
 
   // Inputbox which will pop up when adding/editing a cheat
   StringList labels;
   labels.push_back("Name: ");
   labels.push_back("Code: ");
-  myCheatInput = new InputTextDialog(this, font, labels, _x+20, _y+20);
+  myCheatInput = new InputTextDialog(this, font, labels, 0, 0);
   myCheatInput->setTarget(this);
 
-  // Add OK and Cancel buttons  **** FIXME - coordinates
+  // Add OK and Cancel buttons
 #ifndef MAC_OSX
-  addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "OK", kOKCmd, 0);
+  b = addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "OK", kOKCmd);
+  wid.push_back(b);
+  addOKWidget(b);
   myCancelButton = addButton(font, _w - (kButtonWidth + 10), _h - 24,
-                             "Cancel", kCloseCmd, 0);
+                             "Cancel", kCloseCmd);
+  wid.push_back(myCancelButton);
+  addCancelWidget(myCancelButton);
 #else
   myCancelButton = addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24,
-                             "Cancel", kCloseCmd, 0);
-  addButton(font, _w - (kButtonWidth + 10), _h - 24, "OK", kOKCmd, 0);
+                             "Cancel", kCloseCmd);
+  wid.push_back(myCancelButton);
+  addCancelWidget(myCancelButton);
+  b = addButton(font, _w - (kButtonWidth + 10), _h - 24, "OK", kOKCmd);
+  wid.push_back(b);
+  addOKWidget(b);
 #endif
+
+  addToFocusList(wid);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -133,12 +138,14 @@ void CheatCodeDialog::saveConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheatCodeDialog::addCheat()
 {
+  // We have to add the dialog first, so it can be centered
+  // The methods after this depend on the dialog having the correct dimensions
+  parent()->addDialog(myCheatInput);
   myCheatInput->setEditString("", 0);
   myCheatInput->setEditString("", 1);
   myCheatInput->setTitle("");
   myCheatInput->setFocus(0);
   myCheatInput->setEmitSignal(kCheatAdded);
-  parent()->addDialog(myCheatInput);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -152,11 +159,14 @@ void CheatCodeDialog::editCheat()
   const string& name = list[idx]->name();
   const string& code = list[idx]->code();
 
+  // We have to add the dialog first, so it can be centered
+  // The methods after this depend on the dialog having the correct dimensions
+  parent()->addDialog(myCheatInput);
   myCheatInput->setEditString(name, 0);
   myCheatInput->setEditString(code, 1);
+  myCheatInput->setTitle("");
   myCheatInput->setFocus(1);
   myCheatInput->setEmitSignal(kCheatEdited);
-  parent()->addDialog(myCheatInput);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -169,12 +179,14 @@ void CheatCodeDialog::removeCheat()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheatCodeDialog::addOneShotCheat()
 {
+  // We have to add the dialog first, so it can be centered
+  // The methods after this depend on the dialog having the correct dimensions
+  parent()->addDialog(myCheatInput);
   myCheatInput->setEditString("One-shot cheat", 0);
   myCheatInput->setEditString("", 1);
   myCheatInput->setTitle("");
   myCheatInput->setFocus(1);
   myCheatInput->setEmitSignal(kOneShotCheatAdded);
-  parent()->addDialog(myCheatInput);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
