@@ -8,18 +8,20 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CheatManager.cxx,v 1.10 2006/03/22 15:12:02 stephena Exp $
+// $Id: CheatManager.cxx,v 1.16 2008/02/06 13:45:19 stephena Exp $
 //============================================================================
 
 #include <sstream>
 
 #include "OSystem.hxx"
+#include "Console.hxx"
 #include "Cheat.hxx"
+#include "Settings.hxx"
 #include "CheetahCheat.hxx"
 #include "BankRomCheat.hxx"
 #include "RamCheat.hxx"
@@ -230,7 +232,7 @@ void CheatManager::enable(const string& code, bool enable)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheatManager::loadCheatDatabase()
 {
-  string cheatfile = myOSystem->baseDir() + BSPF_PATH_SEPARATOR + "stella.cht";
+  const string& cheatfile = myOSystem->cheatFile();
   ifstream in(cheatfile.c_str(), ios::in);
   if(!in)
     return;
@@ -271,7 +273,7 @@ void CheatManager::saveCheatDatabase()
   if(!myListIsDirty)
     return;
 
-  string cheatfile = myOSystem->baseDir() + BSPF_PATH_SEPARATOR + "stella.cht";
+  const string& cheatfile = myOSystem->cheatFile();
   ofstream out(cheatfile.c_str(), ios::out);
   if(!out)
     return;
@@ -339,6 +341,7 @@ void CheatManager::saveCheats(const string& md5sum)
 
   // Update the dirty flag
   myListIsDirty = myListIsDirty || changed;
+  clear();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
