@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SoundSDL.cxx,v 1.29 2006/03/06 02:26:16 stephena Exp $
+// $Id: SoundSDL.cxx,v 1.44 2008/05/21 14:01:29 stephena Exp $
 //============================================================================
 
 #ifdef SOUND_SUPPORT
@@ -31,6 +31,12 @@
 #include "System.hxx"
 #include "OSystem.hxx"
 
+#include "Console.hxx"
+#include "AtariVox.hxx"
+#ifdef SPEAKJET_EMULATION
+  #include "SpeakJet.hxx"
+#endif
+
 #include "SoundSDL.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -39,7 +45,7 @@ SoundSDL::SoundSDL(OSystem* osystem)
     myIsEnabled(osystem->settings().getBool("sound")),
     myIsInitializedFlag(false),
     myLastRegisterSetCycle(0),
-    myDisplayFrameRate(60),
+    myDisplayFrameRate(60.0),
     myNumChannels(1),
     myFragmentSizeLogBase2(0),
     myIsMuted(false),
@@ -96,13 +102,12 @@ void SoundSDL::initialize()
       Int32 tiafreq   = myOSystem->settings().getInt("tiafreq");
 
       SDL_AudioSpec desired;
-#ifndef PSP
       desired.freq   = frequency;
+    #ifndef GP2X
       desired.format = AUDIO_U8;
-#else
-      desired.freq   = 44100;
+    #else
       desired.format = AUDIO_U16;
-#endif
+    #endif
       desired.channels = myNumChannels;
       desired.samples  = fragsize;
       desired.callback = callback;
@@ -131,14 +136,14 @@ void SoundSDL::initialize()
       myIsMuted = false;
       myFragmentSizeLogBase2 = log((double)myHardwareSpec.samples) / log(2.0);
 
-/*
+		/*
         cerr << "Freq: " << (int)myHardwareSpec.freq << endl;
         cerr << "Format: " << (int)myHardwareSpec.format << endl;
         cerr << "Channels: " << (int)myHardwareSpec.channels << endl;
         cerr << "Silence: " << (int)myHardwareSpec.silence << endl;
         cerr << "Samples: " << (int)myHardwareSpec.samples << endl;
         cerr << "Size: " << (int)myHardwareSpec.size << endl;
-*/
+		  */
 
       // Now initialize the TIASound object which will actually generate sound
       myTIASound.outputFrequency(myHardwareSpec.freq);
@@ -158,6 +163,7 @@ void SoundSDL::initialize()
              << "  Volume     : " << myVolume << endl
              << "  Frag size  : " << fragsize << endl
              << "  Frequency  : " << myHardwareSpec.freq << endl
+             << "  Format     : " << myHardwareSpec.format << endl
              << "  TIA Freq.  : " << tiafreq << endl
              << "  Channels   : " << myNumChannels << endl
              << "  Clip volume: " << (int)clipvol << endl << endl;
@@ -213,6 +219,7 @@ void SoundSDL::reset()
     SDL_PauseAudio(1);
     myIsMuted = false;
     myLastRegisterSetCycle = 0;
+    myTIASound.reset();
     myRegWriteQueue.clear();
     SDL_PauseAudio(0);
   }
@@ -270,18 +277,14 @@ void SoundSDL::adjustCycleCounter(Int32 amount)
 void SoundSDL::setChannels(uInt32 channels)
 {
   if(channels == 1 || channels == 2)
-  {
     myNumChannels = channels;
-    myOSystem->settings().setInt("channels", myNumChannels);
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void SoundSDL::setFrameRate(uInt32 framerate)
+void SoundSDL::setFrameRate(float framerate)
 {
   // FIXME, we should clear out the queue or adjust the values in it
   myDisplayFrameRate = framerate;
-  myLastRegisterSetCycle = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -298,7 +301,8 @@ void SoundSDL::set(uInt16 addr, uInt8 value, Int32 cycle)
   // the sound to "scale" correctly, we have to know the games real frame 
   // rate (e.g., 50 or 60) and the currently emulated frame rate. We use these
   // values to "scale" the time before the register change occurs.
-  delta = delta * (myDisplayFrameRate / (double)myOSystem->frameRate());
+// FIXME - this always results in 1.0, so we don't really need it
+//  delta = delta * (myDisplayFrameRate / myOSystem->frameRate());
   RegWrite info;
   info.addr = addr;
   info.value = value;
@@ -407,6 +411,27 @@ void SoundSDL::callback(void* udata, uInt8* stream, int len)
 {
   SoundSDL* sound = (SoundSDL*)udata;
   sound->processFragment(stream, (Int32)len);
+
+#ifdef SPEAKJET_EMULATION
+//  cerr << "SoundSDL::callback(): len==" << len << endl;
+
+  // See if we need sound from the AtariVox
+  AtariVox *vox = sound->myOSystem->console().atariVox();
+  if(vox)
+  {
+    // If so, mix 'em together (this is a crappy way to mix audio streams...)
+    uInt8 *s = stream;
+    for(int i=0; i<len/OUTPUT_BUFFER_SIZE; i++)
+    {
+      int count;
+      uInt8 *voxSamples = vox->getSpeakJet()->getSamples(&count);
+      if(!count)
+        break;
+      SDL_MixAudio(s, voxSamples, OUTPUT_BUFFER_SIZE, SDL_MIX_MAXVOLUME);
+      s += OUTPUT_BUFFER_SIZE;
+    }
+  }
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -420,12 +445,12 @@ bool SoundSDL::load(Deserializer& in)
       return false;
 
     uInt8 reg1 = 0, reg2 = 0, reg3 = 0, reg4 = 0, reg5 = 0, reg6 = 0;
-    reg1 = (uInt8) in.getInt();
-    reg2 = (uInt8) in.getInt();
-    reg3 = (uInt8) in.getInt();
-    reg4 = (uInt8) in.getInt();
-    reg5 = (uInt8) in.getInt();
-    reg6 = (uInt8) in.getInt();
+    reg1 = (uInt8) in.getByte();
+    reg2 = (uInt8) in.getByte();
+    reg3 = (uInt8) in.getByte();
+    reg4 = (uInt8) in.getByte();
+    reg5 = (uInt8) in.getByte();
+    reg6 = (uInt8) in.getByte();
 
     myLastRegisterSetCycle = (Int32) in.getInt();
 
@@ -480,12 +505,12 @@ bool SoundSDL::save(Serializer& out)
       reg6 = myTIASound.get(0x1a);
     }
 
-    out.putInt(reg1);
-    out.putInt(reg2);
-    out.putInt(reg3);
-    out.putInt(reg4);
-    out.putInt(reg5);
-    out.putInt(reg6);
+    out.putByte((char)reg1);
+    out.putByte((char)reg2);
+    out.putByte((char)reg3);
+    out.putByte((char)reg4);
+    out.putByte((char)reg5);
+    out.putByte((char)reg6);
 
     out.putInt(myLastRegisterSetCycle);
   }
