@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Debugger.cxx,v 1.104 2005/10/27 19:15:14 stephena Exp $
+// $Id: Debugger.cxx,v 1.127 2008/05/18 20:04:30 stephena Exp $
 //============================================================================
 
 #include "bspf.hxx"
@@ -26,8 +26,10 @@
 #include "Version.hxx"
 #include "OSystem.hxx"
 #include "FrameBuffer.hxx"
+#include "Settings.hxx"
 #include "DebuggerDialog.hxx"
 #include "DebuggerParser.hxx"
+#include "StateManager.hxx"
 
 #include "Console.hxx"
 #include "System.hxx"
@@ -36,6 +38,7 @@
 #include "EquateList.hxx"
 #include "CpuDebug.hxx"
 #include "RamDebug.hxx"
+#include "RiotDebug.hxx"
 #include "TIADebug.hxx"
 
 #include "TiaInfoWidget.hxx"
@@ -93,22 +96,33 @@ Debugger::Debugger(OSystem* osystem)
     myParser(NULL),
     myCpuDebug(NULL),
     myRamDebug(NULL),
+    myRiotDebug(NULL),
     myTiaDebug(NULL),
     myTiaInfo(NULL),
     myTiaOutput(NULL),
     myTiaZoom(NULL),
     myRom(NULL),
-    equateList(NULL),
-    breakPoints(NULL),
-    readTraps(NULL),
-    writeTraps(NULL)
+    myEquateList(NULL),
+    myBreakPoints(NULL),
+    myReadTraps(NULL),
+    myWriteTraps(NULL),
+    myWidth(1030),
+    myHeight(690)
 {
+  // Get the dialog size
+  int w, h;
+  myOSystem->settings().getSize("debuggerres", w, h);
+  myWidth = BSPF_max(w, 0);
+  myHeight = BSPF_max(h, 0);
+  myWidth = BSPF_max(myWidth, 1030u);
+  myHeight = BSPF_max(myHeight, 690u);
+  myOSystem->settings().setSize("debuggerres", myWidth, myHeight);
+
   // Init parser
   myParser = new DebuggerParser(this);
-  equateList = new EquateList();
-  breakPoints = new PackedBitArray(0x10000);
-  readTraps = new PackedBitArray(0x10000);
-  writeTraps = new PackedBitArray(0x10000);
+  myBreakPoints = new PackedBitArray(0x10000);
+  myReadTraps = new PackedBitArray(0x10000);
+  myWriteTraps = new PackedBitArray(0x10000);
 
   // Allow access to this object from any class
   // Technically this violates pure OO programming, but since I know
@@ -124,12 +138,13 @@ Debugger::~Debugger()
 
   delete myCpuDebug;
   delete myRamDebug;
+  delete myRiotDebug;
   delete myTiaDebug;
 
-  delete equateList;
-  delete breakPoints;
-  delete readTraps;
-  delete writeTraps;
+  delete myEquateList;
+  delete myBreakPoints;
+  delete myReadTraps;
+  delete myWriteTraps;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -156,7 +171,7 @@ void Debugger::initializeVideo()
   GUI::Rect r = getDialogBounds();
 
   string title = string("Stella ") + STELLA_VERSION + ": Debugger mode";
-  myOSystem->frameBuffer().initialize(title, r.width(), r.height(), false);
+  myOSystem->frameBuffer().initialize(title, r.width(), r.height());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -170,13 +185,22 @@ void Debugger::setConsole(Console* console)
 
   // Create debugger subsystems
   delete myCpuDebug;
-  myCpuDebug = new CpuDebug(this, myConsole);
+  myCpuDebug = new CpuDebug(*this, *myConsole);
 
   delete myRamDebug;
-  myRamDebug = new RamDebug(this, myConsole);
+  myRamDebug = new RamDebug(*this, *myConsole);
+
+  delete myRiotDebug;
+  myRiotDebug = new RiotDebug(*this, *myConsole);
 
   delete myTiaDebug;
-  myTiaDebug = new TIADebug(this, myConsole);
+  myTiaDebug = new TIADebug(*this, *myConsole);
+
+  // Initialize equates and breakpoints to known state
+  delete myEquateList;
+  myEquateList = new EquateList();
+  clearAllBreakPoints();
+  clearAllTraps();
 
   autoLoadSymbols(myOSystem->romFile());
   loadListFile();
@@ -208,92 +232,96 @@ void Debugger::quit()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::autoLoadSymbols(string fileName) {
-	string file = fileName;
+void Debugger::autoLoadSymbols(string fileName)
+{
+  string file = fileName;
 
-	string::size_type pos;
-	if( (pos = file.find_last_of('.')) != string::npos ) {
-		file.replace(pos, file.size(), ".sym");
-	} else {
-		file += ".sym";
-	}
-	string ret = equateList->loadFile(file);
-	//	cerr << "loading syms from file " << file << ": " << ret << endl;
+  string::size_type pos;
+  if( (pos = file.find_last_of('.')) != string::npos )
+    file.replace(pos, file.size(), ".sym");
+  else
+    file += ".sym";
+
+  string ret = myEquateList->loadFile(file);
+  //  cerr << "loading syms from file " << file << ": " << ret << endl;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::loadListFile(string f) {
-	char buffer[255];
+string Debugger::loadListFile(string f)
+{
+  char buffer[255];
 
-	if(f == "") {
-		f = myOSystem->romFile();
+  if(f == "")
+  {
+    f = myOSystem->romFile();
 
-		string::size_type pos;
-		if( (pos = f.find_last_of('.')) != string::npos ) {
-			f.replace(pos, f.size(), ".lst");
-		} else {
-			f += ".lst";
-		}
-	}
+    string::size_type pos;
+    if( (pos = f.find_last_of('.')) != string::npos )
+      f.replace(pos, f.size(), ".lst");
+    else
+      f += ".lst";
+  }
 
-	ifstream in(f.c_str());
-	if(!in.is_open())
-		return "Unable to read listing from " + f;
+  ifstream in(f.c_str());
+  if(!in.is_open())
+    return "Unable to read listing from " + f;
 
-	sourceLines.clear();
-	int count = 0;
-	while( !in.eof() ) {
-		if(!in.getline(buffer, 255))
-			break;
+  sourceLines.clear();
+  int count = 0;
+  while( !in.eof() )
+  {
+    if(!in.getline(buffer, 255))
+      break;
 
-		if(	strlen(buffer) >= 14 &&
-				buffer[0] == ' '     &&
-				buffer[7] == ' '     &&
-				buffer[8] == ' '     &&
-				isxdigit(buffer[9])  &&
-				isxdigit(buffer[12]) &&
-				IS_BLANK(buffer[13]))
-		{
-			count++;
-			char addr[5];
-			for(int i=0; i<4; i++)
-				addr[i] = buffer[9+i];
+    if(strlen(buffer) >= 14 &&
+       buffer[0] == ' '     &&
+       buffer[7] == ' '     &&
+       buffer[8] == ' '     &&
+       isxdigit(buffer[9])  &&
+       isxdigit(buffer[12]) &&
+       BSPF_isblank(buffer[13]))
+    {
+      count++;
+      char addr[5];
+      for(int i=0; i<4; i++)
+        addr[i] = buffer[9+i];
 
-			for(char *c = buffer; *c != '\0'; c++)
-				if(*c == '\t') *c = ' ';
+      for(char *c = buffer; *c != '\0'; c++)
+        if(*c == '\t') *c = ' ';
 
-			addr[4] = '\0';
-			string a = addr;
-			string b = buffer;
-			sourceLines.insert(make_pair(a, b));
-		}
-	}
+      addr[4] = '\0';
+      string a = addr;
+      string b = buffer;
+      sourceLines.insert(make_pair(a, b));
+    }
+  }
+  in.close();
 
-	in.close();
-
-	return valueToString(count) + " lines loaded from " + f;
+  return valueToString(count) + " lines loaded from " + f;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::getSourceLines(int addr) {
-	if(sourceLines.size() == 0)
-		return "no list file loaded (try \"loadlst file.lst\")";
+string Debugger::getSourceLines(int addr) const
+{
+  if(sourceLines.size() == 0)
+    return "";
 
-	string ret;
-	string want = to_hex_16(addr);
+  string ret;
+  string want = to_hex_16(addr);
 
-	bool found = false;
-	pair<ListIter, ListIter> lines = sourceLines.equal_range(want);
-	for(ListIter i = lines.first; i != lines.second; i++) {
-		found = true;
-		ret += i->second;
-		ret += "\n";
-	}
+  bool found = false;
+  pair<ListIter, ListIter> lines = sourceLines.equal_range(want);
+  for(ListIter i = lines.first; i != lines.second; i++)
+  {
+    found = true;
+    ret += i->second;
+    ret += "\n";
+  }
 
-	if(found)
-		return ret;
-	else
-		return "";
+  if(found)
+    return ret;
+  else
+    return "";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -375,200 +403,31 @@ const string Debugger::valueToString(int value, BaseFormat outputBase)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::invIfChanged(int reg, int oldReg) {
-	string ret;
-
-	bool changed = !(reg == oldReg);
-	if(changed) ret += "\177";
-	ret += valueToString(reg);
-	if(changed) ret += "\177";
-
-	return ret;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::cpuState()
+const string Debugger::invIfChanged(int reg, int oldReg)
 {
-  string result;
-  char buf[255];
+  string ret;
 
-  CpuState state    = (CpuState&) myCpuDebug->getState();
-  CpuState oldstate = (CpuState&) myCpuDebug->getOldState();
+  bool changed = reg != oldReg;
+  if(changed) ret += "\177";
+  ret += valueToString(reg);
+  if(changed) ret += "\177";
 
-  result += "\nPC=";
-  result += invIfChanged(state.PC, oldstate.PC);
-  result += " A=";
-  result += invIfChanged(state.A, oldstate.A);
-  result += " X=";
-  result += invIfChanged(state.X, oldstate.X);
-  result += " Y=";
-  result += invIfChanged(state.Y, oldstate.Y);
-  result += " S=";
-  result += invIfChanged(state.SP, oldstate.SP);
-  result += " P=";
-  result += invIfChanged(state.PS, oldstate.PS);
-  result += "/";
-  formatFlags(state.PSbits, buf);
-  result += buf;
-  result += "\n  FrameCyc:";
-  sprintf(buf, "%d", mySystem->cycles());
-  result += buf;
-  result += " Frame:";
-  sprintf(buf, "%d", myTiaDebug->frameCount());
-  result += buf;
-  result += " ScanLine:";
-  sprintf(buf, "%d", myTiaDebug->scanlines());
-  result += buf;
-  result += " Clk/Pix/Cyc:";
-  int clk = myTiaDebug->clocksThisLine();
-  sprintf(buf, "%d/%d/%d", clk, clk-68, clk/3);
-  result += buf;
-  result += " 6502Ins:";
-  sprintf(buf, "%d", mySystem->m6502().totalInstructionCount());
-  result += buf;
-  result += "\n  ";
-
-  result += disassemble(state.PC, 1);
-  return result;
+  return ret;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-/* The timers, joysticks, and switches can be read via peeks, so
-   I didn't write a separate RIOTDebug class. */
-const string Debugger::riotState() {
-	string ret;
-
-	// TODO: inverse video for changed regs. Core needs to track this.
-	// TODO: keyboard controllers?
-
-	for(int i=0x280; i<0x284; i++) {
-		ret += valueToString(i);
-		ret += "/";
-		ret += equates()->getFormatted(i, 2);
-		ret += "=";
-		ret += valueToString(mySystem->peek(i));
-		ret += " ";
-	}
-	ret += "\n";
-
-	// These are squirrely: some symbol files will define these as
-	// 0x284-0x287. Doesn't actually matter, these registers repeat
-   // every 16 bytes.
-	ret += valueToString(0x294);
-	ret += "/TIM1T=";
-	ret += valueToString(mySystem->peek(0x294));
-	ret += " ";
-
-	ret += valueToString(0x295);
-	ret += "/TIM8T=";
-	ret += valueToString(mySystem->peek(0x295));
-	ret += " ";
-
-	ret += valueToString(0x296);
-	ret += "/TIM64T=";
-	ret += valueToString(mySystem->peek(0x296));
-	ret += " ";
-
-	ret += valueToString(0x297);
-	ret += "/TIM1024T=";
-	ret += valueToString(mySystem->peek(0x297));
-	ret += "\n";
-
-	ret += "Left/P0diff: ";
-	ret += (mySystem->peek(0x282) & 0x40) ? "hard/A" : "easy/B";
-	ret += "   ";
-
-	ret += "Right/P1diff: ";
-	ret += (mySystem->peek(0x282) & 0x80) ? "hard/A" : "easy/B";
-	ret += "\n";
-
-	ret += "TVType: ";
-	ret += (mySystem->peek(0x282) & 0x8) ? "Color" : "B&W";
-	ret += "   Switches: ";
-	ret += (mySystem->peek(0x282) & 0x2) ? "-" : "+";
-	ret += "select  ";
-	ret += (mySystem->peek(0x282) & 0x1) ? "-" : "+";
-	ret += "reset";
-	ret += "\n";
-
-	// Yes, the fire buttons are in the TIA, but we might as well
-	// show them here for convenience.
-	ret += "Left/P0 stick:  ";
-	ret += (mySystem->peek(0x280) & 0x80) ? "" : "right ";
-	ret += (mySystem->peek(0x280) & 0x40) ? "" : "left ";
-	ret += (mySystem->peek(0x280) & 0x20) ? "" : "down ";
-	ret += (mySystem->peek(0x280) & 0x10) ? "" : "up ";
-	ret += ((mySystem->peek(0x280) & 0xf0) == 0xf0) ? "(no directions) " : "";
-	ret += (mySystem->peek(0x03c) & 0x80) ? "" : "(button) ";
-	ret += "\n";
-	ret += "Right/P1 stick: ";
-	ret += (mySystem->peek(0x280) & 0x08) ? "" : "right ";
-	ret += (mySystem->peek(0x280) & 0x04) ? "" : "left ";
-	ret += (mySystem->peek(0x280) & 0x02) ? "" : "down ";
-	ret += (mySystem->peek(0x280) & 0x01) ? "" : "up ";
-	ret += ((mySystem->peek(0x280) & 0x0f) == 0x0f) ? "(no directions) " : "";
-	ret += (mySystem->peek(0x03d) & 0x80) ? "" : "(button) ";
-
-	//ret += "\n"; // caller will add
-
-	return ret;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::reset() {
-	int pc = myCpuDebug->dPeek(0xfffc);
-	myCpuDebug->setPC(pc);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::formatFlags(BoolArray& b, char *out) {
-	// NV-BDIZC
-
-	if(myCpuDebug->n())
-		out[0] = 'N';
-	else
-		out[0] = 'n';
-
-	if(myCpuDebug->v())
-		out[1] = 'V';
-	else
-		out[1] = 'v';
-
-	out[2] = '-';
-
-	if(myCpuDebug->b())
-		out[3] = 'B';
-	else
-		out[3] = 'b';
-
-	if(myCpuDebug->d())
-		out[4] = 'D';
-	else
-		out[4] = 'd';
-
-	if(myCpuDebug->i())
-		out[5] = 'I';
-	else
-		out[5] = 'i';
-
-	if(myCpuDebug->z())
-		out[6] = 'Z';
-	else
-		out[6] = 'z';
-
-	if(myCpuDebug->c())
-		out[7] = 'C';
-	else
-		out[7] = 'c';
-
-	out[8] = '\0';
+void Debugger::reset()
+{
+  int pc = myCpuDebug->dPeek(0xfffc);
+  myCpuDebug->setPC(pc);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 /* Element 0 of args is the address. The remaining elements are the data
    to poke, starting at the given address.
 */
-const string Debugger::setRAM(IntArray& args) {
+const string Debugger::setRAM(IntArray& args)
+{
   char buf[10];
 
   int count = args.size();
@@ -585,83 +444,16 @@ const string Debugger::setRAM(IntArray& args) {
   return ret;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-/* Warning: this method really is for dumping *RAM*, not ROM or I/O! */
-const string Debugger::dumpRAM()
-{
-  string result;
-  char buf[128];
-  int bytesPerLine;
-  int start = kRamStart, len = kRamSize;
-
-  switch(myParser->base())
-  {
-    case kBASE_16:
-    case kBASE_10:
-      bytesPerLine = 0x10;
-      break;
-
-    case kBASE_2:
-      bytesPerLine = 0x04;
-      break;
-
-    case kBASE_DEFAULT:
-    default:
-      return DebuggerParser::red("invalid base, this is a BUG");
-  }
-
-  RamState state    = (RamState&) myRamDebug->getState();
-  RamState oldstate = (RamState&) myRamDebug->getOldState();
-  for (uInt8 i = 0x00; i < len; i += bytesPerLine)
-  {
-    sprintf(buf, "%.2x: ", start+i);
-    result += buf;
-
-    for (uInt8 j = 0; j < bytesPerLine; j++)
-    {
-      result += invIfChanged(state.ram[i+j], oldstate.ram[i+j]);
-      result += " ";
-
-      if(j == 0x07) result += " ";
-    }
-    result += "\n";
-  }
-
-  return result;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::dumpTIA()
-{
-  string result;
-  char buf[128];
-
-  sprintf(buf, "%.2x: ", 0);
-  result += buf;
-  for (uInt8 j = 0; j < 0x010; j++)
-  {
-    sprintf(buf, "%.2x ", mySystem->peek(j));
-    result += buf;
-
-    if(j == 0x07) result += "- ";
-  }
-
-  result += "\n";
-  result += myTiaDebug->state();
-
-  return result;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::saveState(int state)
 {
-  myOSystem->eventHandler().saveState(state);
+  myOSystem->state().saveState(state);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::loadState(int state)
 {
-  myOSystem->eventHandler().loadState(state);
+  myOSystem->state().loadState(state);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -670,11 +462,10 @@ int Debugger::step()
   saveOldState();
 
   int cyc = mySystem->cycles();
-  //	mySystem->unlockDataBus();
+
   unlockState();
   myOSystem->console().mediaSource().updateScanlineByStep();
-  //	mySystem->lockDataBus();
-  unlockState();
+  lockState();
 
   return mySystem->cycles() - cyc;
 }
@@ -693,140 +484,136 @@ int Debugger::step()
 int Debugger::trace()
 {
   // 32 is the 6502 JSR instruction:
-  if(mySystem->peek(myCpuDebug->pc()) == 32) {
+  if(mySystem->peek(myCpuDebug->pc()) == 32)
+  {
     saveOldState();
 
     int cyc = mySystem->cycles();
     int targetPC = myCpuDebug->pc() + 3; // return address
 
-    //	mySystem->unlockDataBus();
-	 unlockState();
+    unlockState();
     myOSystem->console().mediaSource().updateScanlineByTrace(targetPC);
-    //	mySystem->lockDataBus();
-	 lockState();
+    lockState();
 
     return mySystem->cycles() - cyc;
-  } else {
-    return step();
   }
+  else
+    return step();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-EquateList *Debugger::equates() {
-  return equateList;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::toggleBreakPoint(int bp) {
-  mySystem->m6502().setBreakPoints(breakPoints);
+void Debugger::toggleBreakPoint(int bp)
+{
+  mySystem->m6502().setBreakPoints(myBreakPoints);
   if(bp < 0) bp = myCpuDebug->pc();
-  breakPoints->toggle(bp);
+  myBreakPoints->toggle(bp);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Debugger::setBreakPoint(int bp, bool set)
 {
-  mySystem->m6502().setBreakPoints(breakPoints);
+  mySystem->m6502().setBreakPoints(myBreakPoints);
   if(bp < 0) bp = myCpuDebug->pc();
   if(set)
-    breakPoints->set(bp);
+    myBreakPoints->set(bp);
   else
-    breakPoints->clear(bp);
+    myBreakPoints->clear(bp);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::breakPoint(int bp) {
+bool Debugger::breakPoint(int bp)
+{
   if(bp < 0) bp = myCpuDebug->pc();
-  return breakPoints->isSet(bp) != 0;
+  return myBreakPoints->isSet(bp) != 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::toggleReadTrap(int t) {
-  mySystem->m6502().setTraps(readTraps, writeTraps);
-  readTraps->toggle(t);
+void Debugger::toggleReadTrap(int t)
+{
+  mySystem->m6502().setTraps(myReadTraps, myWriteTraps);
+  myReadTraps->toggle(t);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::toggleWriteTrap(int t) {
-  mySystem->m6502().setTraps(readTraps, writeTraps);
-  writeTraps->toggle(t);
+void Debugger::toggleWriteTrap(int t)
+{
+  mySystem->m6502().setTraps(myReadTraps, myWriteTraps);
+  myWriteTraps->toggle(t);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::toggleTrap(int t) {
+void Debugger::toggleTrap(int t)
+{
   toggleReadTrap(t);
   toggleWriteTrap(t);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::readTrap(int t) {
-  return readTraps->isSet(t) != 0;
+bool Debugger::readTrap(int t)
+{
+  return myReadTraps->isSet(t) != 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::writeTrap(int t) {
-  return writeTraps->isSet(t) != 0;
+bool Debugger::writeTrap(int t)
+{
+  return myWriteTraps->isSet(t) != 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::cycles() {
+int Debugger::cycles()
+{
   return mySystem->cycles();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string& Debugger::disassemble(int start, int lines) {
-  char buf[255], bbuf[255];
+const string& Debugger::disassemble(int start, int lines)
+{
   static string result;
+  ostringstream buffer;
+  string cpubuf;
 
-  result = "";
   do {
-    const char *label = equateList->getFormatted(start, 4);
+    buffer << myEquateList->getLabel(start, true, 4) << ": ";
 
-    result += label;
-    result += ": ";
+    int count = myCpuDebug->disassemble(start, cpubuf, *myEquateList);
+    for(int i = 0; i < count; i++)
+      buffer << hex << setw(2) << setfill('0') << peek(start++) << dec;
 
-    int count = myCpuDebug->disassemble(start, buf, equateList);
+    if(count < 3) buffer << "   ";
+    if(count < 2) buffer << "   ";
 
-    for(int i=0; i<count; i++) {
-      sprintf(bbuf, "%02x ", peek(start++));
-      result += bbuf;
-    }
-
-    if(count < 3) result += "   ";
-    if(count < 2) result += "   ";
-
-    result += " ";
-    result += buf;
-    result += "\n";
+    buffer << " " << cpubuf << "\n";
   } while(--lines > 0 && start <= 0xffff);
 
+  result = buffer.str();
   return result;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::disassemble(IntArray& addr, StringList& addrLabel, 
+void Debugger::disassemble(IntArray& addr, StringList& addrLabel,
                            StringList& bytes, StringList& data,
                            int start, int lines)
 {
-  char buf[255], bbuf[255];
-  string tmp;
+  string cpubuf, tmp;
+  char buf[255];
 
   do
   {
-    tmp = equateList->getFormatted(start, 4);
-    addrLabel.push_back(tmp + ":");
+    addrLabel.push_back(myEquateList->getLabel(start, true, 4) + ":");
     addr.push_back(start);
 
-    int count = myCpuDebug->disassemble(start, buf, equateList);
+    cpubuf = "";
+    int count = myCpuDebug->disassemble(start, cpubuf, *myEquateList);
 
     tmp = "";
     for(int i=0; i<count; i++) {
-      sprintf(bbuf, "%02x ", peek(start++));
-      tmp += bbuf;
+      sprintf(buf, "%02x ", peek(start++));
+      tmp += buf;
     }
     bytes.push_back(tmp);
 
-    data.push_back(buf);
+    data.push_back(cpubuf);
   }
   while(--lines > 0 && start <= 0xffff);
 }
@@ -835,10 +622,9 @@ void Debugger::disassemble(IntArray& addr, StringList& addrLabel,
 void Debugger::nextScanline(int lines)
 {
   saveOldState();
-  //	mySystem->unlockDataBus();
+
   unlockState();
   myTiaOutput->advanceScanline(lines);
-  //	mySystem->lockDataBus();
   lockState();
 }
 
@@ -846,73 +632,65 @@ void Debugger::nextScanline(int lines)
 void Debugger::nextFrame(int frames)
 {
   saveOldState();
-  //	mySystem->unlockDataBus();
+
   unlockState();
   myTiaOutput->advance(frames);
-  //	mySystem->lockDataBus();
   lockState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::clearAllBreakPoints() {
-  delete breakPoints;
-  breakPoints = new PackedBitArray(0x10000);
+void Debugger::clearAllBreakPoints()
+{
+  delete myBreakPoints;
+  myBreakPoints = new PackedBitArray(0x10000);
   mySystem->m6502().setBreakPoints(NULL);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::clearAllTraps() {
-  delete readTraps;
-  delete writeTraps;
-  readTraps = new PackedBitArray(0x10000);
-  writeTraps = new PackedBitArray(0x10000);
+void Debugger::clearAllTraps() 
+{
+  delete myReadTraps;
+  delete myWriteTraps;
+  myReadTraps = new PackedBitArray(0x10000);
+  myWriteTraps = new PackedBitArray(0x10000);
   mySystem->m6502().setTraps(NULL, NULL);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::peek(int addr) {
+int Debugger::peek(int addr)
+{
   return mySystem->peek(addr);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::dpeek(int addr) {
+int Debugger::dpeek(int addr)
+{
   return mySystem->peek(addr) | (mySystem->peek(addr+1) << 8);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::setHeight(int height)
+string Debugger::showWatches()
 {
-  if(height < kDebuggerLines)
-    height = kDebuggerLines;
-
-  myOSystem->settings().setInt("debugheight", height);
-
-  // Inform the debugger dialog about the new size
-  quit();
-  resizeDialog();
-  start();
-
-  return height;
+  return myParser->showWatches();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::showWatches() {
-	return myParser->showWatches();
+void Debugger::addLabel(string label, int address)
+{
+  myEquateList->addEquate(label, address);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::addLabel(string label, int address) {
-	equateList->addEquate(label, address);
+void Debugger::reloadROM()
+{
+  myOSystem->createConsole();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::reloadROM() {
-  myOSystem->createConsole( myOSystem->romFile() );
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::setBank(int bank) {
-  if(myConsole->cartridge().bankCount() > 1) {
+bool Debugger::setBank(int bank)
+{
+  if(myConsole->cartridge().bankCount() > 1)
+  {
     myConsole->cartridge().unlockBank();
     myConsole->cartridge().bank(bank);
     myConsole->cartridge().lockBank();
@@ -922,22 +700,26 @@ bool Debugger::setBank(int bank) {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::getBank() {
+int Debugger::getBank()
+{
   return myConsole->cartridge().bank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Debugger::bankCount() {
+int Debugger::bankCount()
+{
   return myConsole->cartridge().bankCount();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char *Debugger::getCartType() {
+string Debugger::getCartType()
+{
   return myConsole->cartridge().name();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::patchROM(int addr, int value) {
+bool Debugger::patchROM(int addr, int value)
+{
   return myConsole->cartridge().patch(addr, value);
 }
 
@@ -946,6 +728,7 @@ void Debugger::saveOldState()
 {
   myCpuDebug->saveOldState();
   myRamDebug->saveOldState();
+  myRiotDebug->saveOldState();
   myTiaDebug->saveOldState();
 }
 
@@ -953,7 +736,6 @@ void Debugger::saveOldState()
 void Debugger::setStartState()
 {
   // Lock the bus each time the debugger is entered, so we don't disturb anything
-  //	mySystem->lockDataBus();
   lockState();
 }
 
@@ -961,44 +743,27 @@ void Debugger::setStartState()
 void Debugger::setQuitState()
 {
   // Bus must be unlocked for normal operation when leaving debugger mode
-  //	mySystem->unlockDataBus();
   unlockState();
 
   // execute one instruction on quit. If we're
   // sitting at a breakpoint/trap, this will get us past it.
   // Somehow this feels like a hack to me, but I don't know why
-  //	if(breakPoints->isSet(myCpuDebug->pc()))
+  //	if(myBreakPoints->isSet(myCpuDebug->pc()))
   mySystem->m6502().execute(1);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 GUI::Rect Debugger::getDialogBounds() const
 {
-  // FIXME - This whole method is due for an overall
-  //         We need to decide if Stella GUI size will be pixel based
-  //         or font based, and update the GUI code everywhere
-  GUI::Rect tia = getTiaBounds();
-
-  int userHeight = myOSystem->settings().getInt("debugheight");
-  if(userHeight < kDebuggerLines)
-  {
-    userHeight = kDebuggerLines;
-    myOSystem->settings().setInt("debugheight", userHeight);
-  }
-  userHeight = (userHeight + 3) * kDebuggerLineHeight - 8;
-
-  // Make sure window is always at least 'kDebuggerHeight' high
-  // We need this to make positioning of widget easier
-  if(userHeight + tia.height() < kDebuggerHeight)
-    userHeight = kDebuggerHeight;
-
-  GUI::Rect r(0, 0, kDebuggerWidth, userHeight + tia.height());
+  // The dialog bounds are the actual size of the entire dialog container
+  GUI::Rect r(0, 0, myWidth, myHeight);
   return r;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 GUI::Rect Debugger::getTiaBounds() const
 {
+  // The area showing the TIA image (NTSC and PAL supported, up to 260 lines)
   GUI::Rect r(0, 0, 320, 260);
   return r;
 }
@@ -1008,8 +773,9 @@ GUI::Rect Debugger::getRomBounds() const
 {
   // The ROM area is the full area to the right of the tabs
   GUI::Rect dialog = getDialogBounds();
+  GUI::Rect status = getStatusBounds();
 
-  int x1 = dialog.right - myOSystem->consoleFont().getMaxCharWidth() * 60;
+  int x1 = status.right + 1;
   int y1 = 0;
   int x2 = dialog.right;
   int y2 = dialog.bottom;
@@ -1022,13 +788,15 @@ GUI::Rect Debugger::getRomBounds() const
 GUI::Rect Debugger::getStatusBounds() const
 {
   // The status area is the full area to the right of the TIA image
-  // and left of the ROM area
+  // extending as far as necessary
+  // 30% of any space above 1030 pixels will be allocated to this area
+  GUI::Rect dlg = getDialogBounds();
   GUI::Rect tia = getTiaBounds();
-  GUI::Rect rom = getRomBounds();
 
   int x1 = tia.right + 1;
   int y1 = 0;
-  int x2 = rom.left - 1;
+  int x2 = tia.right + 225 + (dlg.width() > 1030 ?
+           (int) (0.3 * (dlg.width() - 1030)) : 0);
   int y2 = tia.bottom;
   GUI::Rect r(x1, y1, x2, y2);
 
@@ -1041,11 +809,11 @@ GUI::Rect Debugger::getTabBounds() const
   // The tab area is the full area below the TIA image
   GUI::Rect dialog = getDialogBounds();
   GUI::Rect tia    = getTiaBounds();
-  GUI::Rect rom    = getRomBounds();
+  GUI::Rect status = getStatusBounds();
 
   int x1 = 0;
   int y1 = tia.bottom + 1;
-  int x2 = rom.left - 1;
+  int x2 = status.right + 1;
   int y2 = dialog.bottom;
   GUI::Rect r(x1, y1, x2, y2);
 
@@ -1053,61 +821,64 @@ GUI::Rect Debugger::getTabBounds() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::resizeDialog()
+void Debugger::addFunction(const string& name, const string& definition,
+                           Expression* exp, bool builtin)
 {
-//	cerr << "Debugger::resizeDialog()\n";
+  functions.insert(make_pair(name, exp));
+  if(!builtin)
+    functionDefs.insert(make_pair(name, definition));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::addFunction(string name, string definition, Expression *exp, bool builtin) {
-	functions.insert(make_pair(name, exp));
-	if(!builtin) functionDefs.insert(make_pair(name, definition));
+void Debugger::delFunction(const string& name)
+{
+  FunctionMap::iterator iter = functions.find(name);
+  if(iter == functions.end())
+    return;
+
+  functions.erase(name);
+  delete iter->second;
+
+  FunctionDefMap::iterator def_iter = functionDefs.find(name);
+  if(def_iter == functionDefs.end())
+    return;
+
+  functionDefs.erase(name);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::delFunction(string name) {
-	FunctionMap::iterator iter = functions.find(name);
-	if(iter == functions.end())
-		return;
-
-	functions.erase(name);
-	delete iter->second;
-
-	FunctionDefMap::iterator def_iter = functionDefs.find(name);
-	if(def_iter == functionDefs.end())
-		return;
-
-	functionDefs.erase(name);
+Expression* Debugger::getFunction(const string& name)
+{
+  FunctionMap::iterator iter = functions.find(name);
+  if(iter == functions.end())
+    return 0;
+  else
+    return iter->second;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Expression *Debugger::getFunction(string name) {
-	FunctionMap::iterator iter = functions.find(name);
-	if(iter == functions.end())
-		return 0;
-	else
-		return iter->second;
+string Debugger::getFunctionDef(const string& name)
+{
+  FunctionDefMap::iterator iter = functionDefs.find(name);
+  if(iter == functionDefs.end())
+    return "";
+  else
+    return iter->second;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string Debugger::getFunctionDef(string name) {
-	FunctionDefMap::iterator iter = functionDefs.find(name);
-	if(iter == functionDefs.end())
-		return "";
-	else
-		return iter->second;
+const FunctionDefMap Debugger::getFunctionDefMap() const
+{
+  return functionDefs;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const FunctionDefMap Debugger::getFunctionDefMap() {
-	return functionDefs;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const string Debugger::builtinHelp() {
+const string Debugger::builtinHelp() const
+{
   string result;
 
-  for(int i=0; builtin_functions[i][0] != ""; i++) {
+  for(int i=0; builtin_functions[i][0] != ""; i++)
+  {
     result += builtin_functions[i][0];
     result += " {";
     result += builtin_functions[i][1];
@@ -1120,7 +891,7 @@ const string Debugger::builtinHelp() {
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Debugger::saveROM(string filename)
+bool Debugger::saveROM(const string& filename) const
 {
   // TODO: error checking
   ofstream *out = new ofstream(filename.c_str(), ios::out | ios::binary);
@@ -1130,13 +901,15 @@ bool Debugger::saveROM(string filename)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::lockState() {
+void Debugger::lockState()
+{
   mySystem->lockDataBus();
   myConsole->cartridge().lockBank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Debugger::unlockState() {
+void Debugger::unlockState()
+{
   mySystem->unlockDataBus();
   myConsole->cartridge().unlockBank();
 }
