@@ -8,24 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: AudioWidget.cxx,v 1.2 2006/02/22 17:38:04 stephena Exp $
+// $Id: AudioWidget.cxx,v 1.7 2008/02/06 13:45:20 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
+#include "DataGridWidget.hxx"
 #include "FrameBuffer.hxx"
-#include "GuiUtils.hxx"
 #include "GuiObject.hxx"
+#include "OSystem.hxx"
 #include "TIADebug.hxx"
 #include "Widget.hxx"
-#include "DataGridWidget.hxx"
+
 #include "AudioWidget.hxx"
 
 // ID's for the various widgets
@@ -42,6 +42,8 @@ AudioWidget::AudioWidget(GuiObject* boss, const GUI::Font& font,
   : Widget(boss, font, x, y, w, h),
     CommandSender(boss)
 {
+  _type = kAudioWidget;
+
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight();
