@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CpuWidget.cxx,v 1.4 2006/02/22 17:38:04 stephena Exp $
+// $Id: CpuWidget.cxx,v 1.13 2008/05/15 18:59:56 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -22,7 +22,6 @@
 #include <sstream>
 
 #include "OSystem.hxx"
-#include "GuiUtils.hxx"
 #include "GuiObject.hxx"
 #include "Debugger.hxx"
 #include "CpuDebug.hxx"
@@ -63,6 +62,8 @@ CpuWidget::CpuWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   : Widget(boss, font, x, y, 16, 16),
     CommandSender(boss)
 {
+  _type = kCpuWidget;
+
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight();
@@ -242,8 +243,8 @@ void CpuWidget::fillGrid()
   // address in the callback (handleCommand)
   Debugger& dbg = instance()->debugger();
   CpuDebug& cpu = dbg.cpuDebug();
-  CpuState state    = (CpuState&) cpu.getState();
-  CpuState oldstate = (CpuState&) cpu.getOldState();
+  const CpuState& state    = (CpuState&) cpu.getState();
+  const CpuState& oldstate = (CpuState&) cpu.getOldState();
 
   // Add PC to its own DataGridWidget
   alist.push_back(kPCRegAddr);
@@ -280,5 +281,5 @@ void CpuWidget::fillGrid()
     changed.push_back(state.PSbits[i] != oldstate.PSbits[i]);
 
   myPSRegister->setState(state.PSbits, changed);
-  myPCLabel->setEditString(dbg.equates()->getLabel(state.PC));
+  myPCLabel->setEditString(dbg.equates().getLabel(state.PC, true));
 }
