@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DataGridWidget.cxx,v 1.7 2006/03/25 00:34:17 stephena Exp $
+// $Id: DataGridWidget.cxx,v 1.13 2008/02/06 13:45:20 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -45,7 +45,8 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
     _selectedItem(0),
     _opsWidget(NULL)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
+           WIDGET_WANTS_RAWDATA;
   _type = kDataGridWidget;
   _editMode = false;
 
@@ -169,8 +170,8 @@ void DataGridWidget::setSelectedValue(int value)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::setRange(int lower, int upper)
 {
-  _lowerBound = MAX(0, lower);
-  _upperBound = MIN(1 << _bits, upper);
+  _lowerBound = BSPF_max(0, lower);
+  _upperBound = BSPF_min(1 << _bits, upper);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -523,15 +524,15 @@ void DataGridWidget::drawWidget(bool hilite)
         int color = kTextColor;
         if(_changedList[pos])
         {
-          fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kTextColorEm);
+          fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kDbgChangedColor);
 
           if(_hiliteList[pos])
-            color = kHiliteColor;
+            color = kDbgColorHi;
           else
-            color = kTextColorHi;
+            color = kDbgChangedTextColor;
         }
         else if(_hiliteList[pos])
-          color = kHiliteColor;
+          color = kDbgColorHi;
 
         fb.drawString(_font, buffer, x, y, _colWidth, color);
       }
