@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DebuggerDialog.cxx,v 1.13 2006/03/23 16:16:32 stephena Exp $
+// $Id: DebuggerDialog.cxx,v 1.24 2008/04/29 15:13:15 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -29,6 +29,7 @@
 #include "PromptWidget.hxx"
 #include "CpuWidget.hxx"
 #include "RamWidget.hxx"
+#include "RiotWidget.hxx"
 #include "RomWidget.hxx"
 #include "TiaWidget.hxx"
 #include "DataGridOpsWidget.hxx"
@@ -84,8 +85,8 @@ void DebuggerDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::handleKeyDown(int ascii, int keycode, int modifiers)
 {
-  // Doing this means we disallow 'Alt xxx' events to any widget in the tabset
-  if(instance()->eventHandler().kbdAlt(modifiers))
+  bool handled = instance()->eventHandler().kbdAlt(modifiers);
+  if(handled)
   {
     switch(ascii)
     {
@@ -101,9 +102,12 @@ void DebuggerDialog::handleKeyDown(int ascii, int keycode, int modifiers)
       case 'l':
         doScanlineAdvance();
         break;
+      default:
+        handled = false;
+        break;
     }
   }
-  else
+  if(!handled)
     Dialog::handleKeyDown(ascii, keycode, modifiers);
 }
 
@@ -179,6 +183,13 @@ void DebuggerDialog::addTabArea()
   myTab->setParentWidget(tabID, tia);
   addToFocusList(tia->getFocusList(), tabID);
 
+  // The input/output tab (includes RIOT and INPTx from TIA)
+  tabID = myTab->addTab("I/O");
+  RiotWidget* riot = new RiotWidget(myTab, instance()->consoleFont(),
+                                    2, 2, widWidth, widHeight);
+  myTab->setParentWidget(tabID, riot);
+  addToFocusList(riot->getFocusList(), tabID);
+
   // The Audio tab
   tabID = myTab->addTab("Audio");
   AudioWidget* aud = new AudioWidget(myTab, instance()->consoleFont(),
@@ -186,9 +197,6 @@ void DebuggerDialog::addTabArea()
   myTab->setParentWidget(tabID, aud);
   addToFocusList(aud->getFocusList(), tabID);
 
-  // The input/output tab (part of RIOT)
-//  tabID = myTab->addTab("I/O");
-
   myTab->setActiveTab(0);
 }
 
@@ -196,6 +204,7 @@ void DebuggerDialog::addTabArea()
 void DebuggerDialog::addStatusArea()
 {
   const GUI::Font& font = instance()->consoleFont();
+  const int lineHeight = font.getLineHeight();
   GUI::Rect r = instance()->debugger().getStatusBounds();
   int xpos, ypos;
 
@@ -203,15 +212,17 @@ void DebuggerDialog::addStatusArea()
   myTiaInfo = new TiaInfoWidget(this, instance()->consoleFont(), xpos, ypos);
 
   ypos += myTiaInfo->getHeight() + 10;
-  myTiaZoom = new TiaZoomWidget(this, instance()->consoleFont(), xpos+10, ypos);
+  myTiaZoom = new TiaZoomWidget(this, instance()->consoleFont(), xpos+10, ypos,
+                                r.width()-10, r.height()-lineHeight-ypos-10);
   addToFocusList(myTiaZoom->getFocusList());
 
-  xpos += 10;  ypos += myTiaZoom->getHeight() + 20;
+  xpos += 10;  ypos += myTiaZoom->getHeight() + 10;
   myMessageBox = new EditTextWidget(this, instance()->consoleFont(),
                                     xpos, ypos, myTiaZoom->getWidth(),
                                     font.getLineHeight(), "");
   myMessageBox->setEditable(false);
-  myMessageBox->setColor(kTextColorEm);
+  myMessageBox->clearFlags(WIDGET_RETAIN_FOCUS);
+  myMessageBox->setTextColor(kTextColorEm);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -228,7 +239,7 @@ void DebuggerDialog::addRomArea()
   myRam = new RamWidget(this, instance()->consoleFont(), xpos, ypos);
   addToFocusList(myRam->getFocusList());
 
-  xpos = r.left + 10 + myCpu->getWidth() + 20;
+  xpos = r.left + 10 + myCpu->getWidth() + 5;
   DataGridOpsWidget* ops = new DataGridOpsWidget(this, instance()->consoleFont(),
                                                  xpos, 20);
 
@@ -265,29 +276,29 @@ void DebuggerDialog::addRomArea()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doStep()
 {
-  instance()->debugger().parser()->run("step");
+  instance()->debugger().parser().run("step");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doTrace()
 {
-  instance()->debugger().parser()->run("trace");
+  instance()->debugger().parser().run("trace");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doAdvance()
 {
-  instance()->debugger().parser()->run("frame #1");
+  instance()->debugger().parser().run("frame #1");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doScanlineAdvance()
 {
-  instance()->debugger().parser()->run("scanline #1");
+  instance()->debugger().parser().run("scanline #1");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DebuggerDialog::doExit()
 {
-  instance()->debugger().quit();
+  instance()->debugger().parser().run("run");
 }
