@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PromptWidget.cxx,v 1.8 2006/03/25 00:34:17 stephena Exp $
+// $Id: PromptWidget.cxx,v 1.24 2008/05/01 23:08:24 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -49,11 +49,14 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
   : Widget(boss, font, x, y, w - kScrollBarWidth, h),
     CommandSender(boss),
     _makeDirty(false),
-    _firstTime(true)
+    _firstTime(true),
+    _exitedEarly(false)
 {
   _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
-           WIDGET_WANTS_TAB;
+           WIDGET_WANTS_TAB | WIDGET_WANTS_RAWDATA;
   _type = kPromptWidget;
+  _textcolor = kTextColor;
+  _bgcolor = kWidColor;
 
   _kConsoleCharWidth  = font.getMaxCharWidth();
   _kConsoleCharHeight = font.getFontHeight();
@@ -75,10 +78,6 @@ PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
   _scrollBar->setTarget(this);
 
   // Init colors
-  defaultTextColor = kTextColor;
-  defaultBGColor = kBGColor;
-  textColor = defaultTextColor;
-  bgColor = defaultBGColor;
   _inverse = false;
 
   // Init History
@@ -117,12 +116,11 @@ void PromptWidget::drawWidget(bool hilite)
       int c = buffer((start + line) * _lineWidth + column);
 
       if(c & (1 << 17)) { // inverse video flag
-        fgcolor = bgColor;
+        fgcolor = _bgcolor;
         bgcolor = (c & 0x1ffff) >> 8;
         fb.fillRect(x, y, _kConsoleCharWidth, _kConsoleCharHeight, bgcolor);
       } else {
         fgcolor = c >> 8;
-        bgcolor = bgColor;
       }
       fb.drawChar(&instance()->consoleFont(), c & 0x7f, x, y, fgcolor);
       x += _kConsoleCharWidth;
@@ -166,7 +164,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   int i;
   bool handled = true;
   bool dirty = false;
-	
+  
   switch(ascii)
   {
     case '\n': // enter/return
@@ -179,23 +177,28 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
 
       if (len > 0)
       {
-        // We have to allocate the string buffer with new, since VC++ sadly does not
-        // comply to the C++ standard, so we can't use a dynamic sized stack array.
-        char *str = new char[len + 1];
-
-        // Copy the user input to str
+        // Copy the user input to command
+        string command;
         for (i = 0; i < len; i++)
-          str[i] = buffer(_promptStartPos + i) & 0x7f;
-        str[len] = '\0';
+          command += buffer(_promptStartPos + i) & 0x7f;
 
         // Add the input to the history
-        addToHistory(str);
+        addToHistory(command.c_str());
 
         // Pass the command to the debugger, and print the result
-        print( instance()->debugger().run(str) + "\n");
+        string result = instance()->debugger().run(command);
 
-        // Get rid of the string buffer
-        delete [] str;
+        // This is a bit of a hack
+        // Certain commands remove the debugger dialog from underneath us,
+        // so we shouldn't print any messages
+        // Those commands will return '_EXIT_DEBUGGER' as their result
+        if(result == "_EXIT_DEBUGGER")
+        {
+          _exitedEarly = true;
+          return true;
+        }
+        else
+          print(result + "\n");
       }
 
       printPrompt();
@@ -209,7 +212,7 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       // both at once.
 
       if(_currentPos <= _promptStartPos)
-    	break;
+        break;
 
       scrollToCurrent();
       int len = _promptEndPos - _promptStartPos;
@@ -218,12 +221,14 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       char delimiter = '\0';
 
       char *str = new char[len + 1];
-      for (i = 0; i < len; i++) {
-    	str[i] = buffer(_promptStartPos + i) & 0x7f;
-    	if(strchr("*@<> ", str[i]) != NULL ) {
-    	  lastDelimPos = i;
-    	  delimiter = str[i];
-    	}
+      for (i = 0; i < len; i++)
+      {
+        str[i] = buffer(_promptStartPos + i) & 0x7f;
+        if(strchr("*@<> ", str[i]) != NULL )
+        {
+          lastDelimPos = i;
+          delimiter = str[i];
+        }
       }
       str[len] = '\0';
 
@@ -231,60 +236,67 @@ bool PromptWidget::handleKeyDown(int ascii, int keycode, int modifiers)
       const char *prefix;
       int possibilities;
 
-      if(lastDelimPos < 0) {
-    	// no delimiters, do command completion:
-    	DebuggerParser *parser = instance()->debugger().parser();
-    	possibilities = parser->countCompletions(str);
+      if(lastDelimPos < 0)
+      {
+        // no delimiters, do command completion:
+        DebuggerParser& parser = instance()->debugger().parser();
+        possibilities = parser.countCompletions(str);
 
-    	if(possibilities < 1) {
-    	  delete[] str;
-    	  break;
-    	}
+        if(possibilities < 1) {
+          delete[] str;
+          break;
+        }
 
-    	completionList = parser->getCompletions();
-    	prefix = parser->getCompletionPrefix();
-	    } else {
-    	// we got a delimiter, so this must be a label:
-    	EquateList *equates = instance()->debugger().equates();
-    	possibilities = equates->countCompletions(str + lastDelimPos + 1);
+        completionList = parser.getCompletions();
+        prefix = parser.getCompletionPrefix();
+      }
+      else
+      {
+        // we got a delimiter, so this must be a label:
+        EquateList& equates = instance()->debugger().equates();
+        possibilities = equates.countCompletions(str + lastDelimPos + 1);
 
-    	if(possibilities < 1) {
-    	  delete[] str;
-    	  break;
-    	}
+        if(possibilities < 1) {
+          delete[] str;
+          break;
+        }
 
-	    	completionList = equates->getCompletions();
-	    	prefix = equates->getCompletionPrefix();
-	    }
+        // TODO - perhaps use strings instead of char pointers
+        completionList = equates.getCompletions().c_str();
+        prefix = equates.getCompletionPrefix().c_str();
+      }
 
-      if(possibilities == 1) {
-    	// add to buffer as though user typed it (plus a space)
-    	_currentPos = _promptStartPos + lastDelimPos + 1;
-    	while(*completionList != '\0') {
-    	  putcharIntern(*completionList++);
-    	}
-    	putcharIntern(' ');
-    	_promptEndPos = _currentPos;
-      } else {
-    	nextLine();
-    	// add to buffer as-is, then add PROMPT plus whatever we have so far
-    	_currentPos = _promptStartPos + lastDelimPos + 1;
+      if(possibilities == 1)
+      {
+        // add to buffer as though user typed it (plus a space)
+        _currentPos = _promptStartPos + lastDelimPos + 1;
+        while(*completionList != '\0')
+          putcharIntern(*completionList++);
 
-    	print("\n");
-    	print(completionList);
-    	print("\n");
-    	print(PROMPT);
+        putcharIntern(' ');
+        _promptEndPos = _currentPos;
+      }
+      else
+      {
+        nextLine();
+        // add to buffer as-is, then add PROMPT plus whatever we have so far
+        _currentPos = _promptStartPos + lastDelimPos + 1;
 
-    	_promptStartPos = _currentPos;
+        print("\n");
+        print(completionList);
+        print("\n");
+        print(PROMPT);
 
-    	for(i=0; i<lastDelimPos; i++)
-    	  putcharIntern(str[i]);
+        _promptStartPos = _currentPos;
 
-    	if(lastDelimPos > 0)
-    	  putcharIntern(delimiter);
+        for(i=0; i<lastDelimPos; i++)
+          putcharIntern(str[i]);
 
-    	print(prefix);
-    	_promptEndPos = _currentPos;
+        if(lastDelimPos > 0)
+          putcharIntern(delimiter);
+
+        print(prefix);
+        _promptEndPos = _currentPos;
       }
       delete[] str;
       dirty = true;
@@ -511,10 +523,16 @@ void PromptWidget::loadConfig()
     _promptStartPos = _promptEndPos = _currentPos;
 
     _firstTime = false;
+    _exitedEarly = false;
 
     // Take care of one-time debugger stuff
     instance()->debugger().autoExec();
   }
+  else if(_exitedEarly)
+  {
+    printPrompt();
+    _exitedEarly = false;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -528,27 +546,22 @@ void PromptWidget::specialKeys(int keycode)
       _currentPos = _promptStartPos;
       handled = true;
       break;
-
     case 'd':
       killChar(+1);
       handled = true;
       break;
-
     case 'e':
       _currentPos = _promptEndPos;
       handled = true;
       break;
-
     case 'k':
       killLine(+1);
       handled = true;
       break;
-
     case 'u':
       killLine(-1);
       handled = true;
       break;
-
     case 'w':
       killLastWord();
       handled = true;
@@ -651,7 +664,7 @@ void PromptWidget::addToHistory(const char *str)
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int PromptWidget::compareHistory(const char *histLine) {
-	return 1;
+  return 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -715,9 +728,8 @@ void PromptWidget::historyScroll(int direction)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::nextLine()
 {
-  // reset colors every line, so I don't have to remember to do it myself
-  textColor = defaultTextColor;
-  bgColor = defaultBGColor;
+  // Reset colors every line, so I don't have to remember to do it myself
+  _textcolor = kTextColor;
   _inverse = false;
 
   int line = _currentPos / _lineWidth;
@@ -734,7 +746,7 @@ void PromptWidget::nextLine()
 // Call this (at least) when the current line changes or when a new line is added
 void PromptWidget::updateScrollBuffer()
 {
-  int lastchar = MAX(_promptEndPos, _currentPos);
+  int lastchar = BSPF_max(_promptEndPos, _currentPos);
   int line = lastchar / _lineWidth;
   int numlines = (line < _linesInBuffer) ? line + 1 : _linesInBuffer;
   int firstline = line - numlines + 1;
@@ -769,7 +781,7 @@ int PromptWidget::printf(const char *format, ...)
 int PromptWidget::vprintf(const char *format, va_list argptr)
 {
   char buf[2048];
-  int count = VSNPRINTF(buf, sizeof(buf), format, argptr);
+  int count = BSPF_vsnprintf(buf, sizeof(buf), format, argptr);
 
   print(buf);
   return count;
@@ -792,17 +804,17 @@ void PromptWidget::putcharIntern(int c)
                       // don't print or advance cursor
                       // there are only 128 TIA colors, but
                       // OverlayColor contains 256 of them
-    textColor = (c & 0x7f) << 1;
+    _textcolor = (c & 0x7f) << 1;
   }
   else if(c < ' ') { // More colors (the regular GUI ones)
-    textColor = c + 0x100;
+    _textcolor = c + 0x100;
   }
   else if(c == 0x7f) { // toggle inverse video (DEL char)
     _inverse = !_inverse;
   }
   else
   {
-    buffer(_currentPos) = c | (textColor << 8) | (_inverse << 17);
+    buffer(_currentPos) = c | (_textcolor << 8) | (_inverse << 17);
     _currentPos++;
     if ((_scrollLine + 1) * _lineWidth == _currentPos)
     {
