@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RamWidget.cxx,v 1.5 2006/03/29 13:53:00 stephena Exp $
+// $Id: RamWidget.cxx,v 1.17 2008/05/04 17:16:39 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,26 +21,16 @@
 
 #include <sstream>
 
-#include "OSystem.hxx"
+#include "DataGridWidget.hxx"
+#include "EditTextWidget.hxx"
 #include "FrameBuffer.hxx"
-#include "GuiUtils.hxx"
 #include "GuiObject.hxx"
 #include "InputTextDialog.hxx"
-#include "Widget.hxx"
-#include "EditTextWidget.hxx"
-#include "DataGridWidget.hxx"
+#include "OSystem.hxx"
 #include "RamDebug.hxx"
-#include "RamWidget.hxx"
+#include "Widget.hxx"
 
-enum {
-  kUndoCmd     = 'RWud',
-  kRevertCmd   = 'RWrv',
-  kSearchCmd   = 'RWse',
-  kCmpCmd      = 'RWcp',
-  kRestartCmd  = 'RWrs',
-  kSValEntered = 'RWsv',
-  kCValEntered = 'RWcv'
-};
+#include "RamWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 RamWidget::RamWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
@@ -49,6 +39,8 @@ RamWidget::RamWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
     myUndoAddress(-1),
     myUndoValue(-1)
 {
+  _type = kRamWidget;
+
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight(),
@@ -66,27 +58,27 @@ RamWidget::RamWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   // Create actions buttons to the left of the RAM grid
   xpos += lwidth + myRamGrid->getWidth() + 4;
   myUndoButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                  "Undo", kUndoCmd, 0);
+                                  "Undo", kUndoCmd);
   myUndoButton->setTarget(this);
 
   ypos += bheight + 4;
   myRevertButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                    "Rev", kRevertCmd, 0);
+                                    "Rev", kRevertCmd);
   myRevertButton->setTarget(this);
 
   ypos += 2 * bheight + 2;
   mySearchButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                    "Srch", kSearchCmd, 0);
+                                    "Srch", kSearchCmd);
   mySearchButton->setTarget(this);
 
   ypos += bheight + 4;
   myCompareButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                     "Cmp", kCmpCmd, 0);
+                                     "Cmp", kCmpCmd);
   myCompareButton->setTarget(this);
 
   ypos += bheight + 4;
   myRestartButton = new ButtonWidget(boss, font, xpos, ypos, bwidth, bheight,
-                                     "Rset", kRestartCmd, 0);
+                                     "Rset", kRestartCmd);
   myRestartButton->setTarget(this);
 
   // Labels for RAM grid
@@ -142,6 +134,7 @@ RamWidget::RamWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   myInputBox = new InputTextDialog(boss, font, label,
                                    x + lwidth + 20, y + 2*lineHeight - 5);
   myInputBox->setTarget(this);
+  myInputBox->setCenter(false);
 
   // Start with these buttons disabled
   myCompareButton->clearFlags(WIDGET_ENABLED);
@@ -160,7 +153,6 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
   // It will then send the 'kDGItemDataChangedCmd' signal to change the actual
   // memory location
   int addr, value;
-  const char* buf;
 
   RamDebug& dbg = instance()->debugger().ramDebug();
   switch(cmd)
@@ -180,16 +172,16 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
 
     case kDGSelectionChangedCmd:
+    {
       addr  = myRamGrid->getSelectedAddr();
       value = myRamGrid->getSelectedValue();
 
-      buf = instance()->debugger().equates()->getLabel(addr+kRamStart).c_str();
-      if(*buf) myLabel->setEditString(buf);
-      else    myLabel->setEditString("");
-
+      myLabel->setEditString(
+        instance()->debugger().equates().getLabel(addr+kRamStart, true));
       myDecValue->setEditString(instance()->debugger().valueToString(value, kBASE_10));
       myBinValue->setEditString(instance()->debugger().valueToString(value, kBASE_2));
       break;
+    }
 
     case kRevertCmd:
       for(unsigned int i = 0; i < kRamSize; i++)
@@ -204,17 +196,17 @@ void RamWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
 
     case kSearchCmd:
+      parent()->addDialog(myInputBox);
       myInputBox->setEditString("");
       myInputBox->setTitle("");
       myInputBox->setEmitSignal(kSValEntered);
-      parent()->addDialog(myInputBox);
       break;
 
     case kCmpCmd:
+      parent()->addDialog(myInputBox);
       myInputBox->setEditString("");
       myInputBox->setTitle("");
       myInputBox->setEmitSignal(kCValEntered);
-      parent()->addDialog(myInputBox);
       break;
 
     case kRestartCmd:
