@@ -8,17 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RomListWidget.cxx,v 1.5 2006/02/22 17:38:04 stephena Exp $
+// $Id: RomListWidget.cxx,v 1.12 2008/05/11 21:18:34 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
+#include "bspf.hxx"
 #include "ContextMenu.hxx"
 #include "RomListWidget.hxx"
 
@@ -29,6 +30,8 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& font,
     myMenu(NULL),
     myHighlightedItem(-1)
 {
+  _type = kRomListWidget;
+
   myMenu = new ContextMenu(this, font);
 
   StringList l;
@@ -38,8 +41,12 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& font,
 
   myMenu->setList(l);
 
-  myLabelWidth  = font.getMaxCharWidth() * 16;
-  myBytesWidth  = font.getMaxCharWidth() * 12;
+  // Take advantage of a wide debugger window when possible
+  const int fontWidth = font.getMaxCharWidth(),
+            numchars = w / fontWidth;
+
+  myLabelWidth = BSPF_max(20, int(0.35 * (numchars - 12))) * fontWidth;
+  myBytesWidth = 12 * fontWidth;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -71,6 +78,12 @@ void RomListWidget::handleMouseDown(int x, int y, int button, int clickCount)
   ListWidget::handleMouseDown(x, y, button, clickCount);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool RomListWidget::handleEvent(Event::Type e)
+{
+  return ListWidget::handleEvent(e); // override CheckListWidget::handleEvent()
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::drawWidget(bool hilite)
 {
@@ -104,8 +117,7 @@ void RomListWidget::drawWidget(bool hilite)
     if (_highlightedItem == pos)
     {
       fb.frameRect(_x + l.left - 3, _y + 1 + _fontHeight * i,
-                   _w - l.left, _fontHeight,
-                   kHiliteColor);
+                   _w - l.left, _fontHeight, kDbgColorHi);
     }
 
     // Draw the selected item inverted, on a highlighted background.
