@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RomWidget.cxx,v 1.16 2006/02/22 17:38:04 stephena Exp $
+// $Id: RomWidget.cxx,v 1.24 2008/03/23 17:43:22 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -45,8 +45,9 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
     mySourceAvailable(false),
     myCurrentBank(-1)
 {
-  int w = 58 * font.getMaxCharWidth(),
-      h = 0, xpos, ypos;
+  _type = kRomWidget;
+
+  int xpos, ypos;
   StaticTextWidget* t;
 
   // Create bank editable area
@@ -58,7 +59,7 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
 
   xpos += t->getWidth() + 10;
   myBank = new DataGridWidget(boss, font, xpos, ypos-2,
-                              1, 1, 1, 2, kBASE_16_4);
+                              1, 1, 3, 8, kBASE_10);
   myBank->setTarget(this);
   myBank->setRange(0, instance()->debugger().bankCount());
   if(instance()->debugger().bankCount() <= 1)
@@ -74,16 +75,13 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
 
   xpos += t->getWidth() + 10;
   myBankCount = new EditTextWidget(boss, font, xpos, ypos-2,
-                                   20, font.getLineHeight(), "");
+                                   30, font.getLineHeight(), "");
   myBankCount->setEditable(false);
 
   // Create rom listing
   xpos = x;  ypos += myBank->getHeight() + 4;
-
-  // Update height of widget to use all remaining vertical space
   GUI::Rect dialog = instance()->debugger().getDialogBounds();
-  int rows = ((dialog.height() - ypos) / font.getLineHeight()) - 1;
-  h = rows * font.getLineHeight();
+  int w = dialog.width() - x - 5, h = dialog.height() - ypos - 3;
 
   myRomList = new RomListWidget(boss, font, xpos, ypos, w, h);
   myRomList->setTarget(this);
@@ -100,6 +98,7 @@ RomWidget::RomWidget(GuiObject* boss, const GUI::Font& font, int x, int y)
   label.push_back("Filename: ");
   mySaveRom = new InputTextDialog(boss, font, label, _x + 50, _y + 80);
   mySaveRom->setTarget(this);
+  mySaveRom->setCenter(false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -211,15 +210,14 @@ void RomWidget::loadConfig()
   myBank->setList(alist, vlist, changed);
 
   // Indicate total number of banks
-  int bankCount = dbg.bankCount();
-  myBankCount->setEditString(dbg.valueToString(bankCount));
+  myBankCount->setEditString(dbg.valueToString(dbg.bankCount(), kBASE_10));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomWidget::initialUpdate()
 {
   Debugger& dbg = instance()->debugger();
-  PackedBitArray* bp = dbg.breakpoints();
+  PackedBitArray& bp = dbg.breakpoints();
 
   // Reading from ROM might trigger a bankswitch, so save the current bank
   myCurrentBank = dbg.getBank();
@@ -240,7 +238,7 @@ void RomWidget::initialUpdate()
     dbg.disassemble(myAddrList, label, data, disasm, 0xf000, 4096);
     for(unsigned int i = 0; i < data.size(); ++i)
     {
-      if(bp && bp->isSet(myAddrList[i]))
+      if(bp.isSet(myAddrList[i]))
         state.push_back(true);
       else
         state.push_back(false);
@@ -287,14 +285,14 @@ void RomWidget::patchROM(int data, const string& bytes)
   // Temporarily set to base 16, since that's the format the disassembled
   // byte string is in.  This eliminates the need to prefix each byte with
   // a '$' character
-  BaseFormat oldbase = instance()->debugger().parser()->base();
-  instance()->debugger().parser()->setBase(kBASE_16);
+  BaseFormat oldbase = instance()->debugger().parser().base();
+  instance()->debugger().parser().setBase(kBASE_16);
 
   command << "rom #" << myAddrList[data] << " " << bytes;
   instance()->debugger().run(command.str());
 
   // Restore previous base
-  instance()->debugger().parser()->setBase(oldbase);
+  instance()->debugger().parser().setBase(oldbase);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
