@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaOutputWidget.cxx,v 1.10 2006/03/05 01:18:41 stephena Exp $
+// $Id: TiaOutputWidget.cxx,v 1.15 2008/03/23 17:43:22 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -41,6 +41,8 @@ TiaOutputWidget::TiaOutputWidget(GuiObject* boss, const GUI::Font& font,
     myMenu(NULL),
     myZoom(NULL)
 {
+  _type = kTiaOutputWidget;
+
   // Create context menu for commands
   myMenu = new ContextMenu(this, font);
 
@@ -116,7 +118,7 @@ void TiaOutputWidget::handleCommand(CommandSender* sender, int cmd, int data, in
           if(lines > 0)
           {
             command << "scanline #" << lines;
-            instance()->debugger().parser()->run(command.str());
+            instance()->debugger().parser().run(command.str());
           }
           break;
         }
@@ -126,7 +128,7 @@ void TiaOutputWidget::handleCommand(CommandSender* sender, int cmd, int data, in
           ostringstream command;
           int scanline = myClickY + ystart;
           command << "breakif _scan==#" << scanline;
-          instance()->debugger().parser()->run(command.str());
+          instance()->debugger().parser().run(command.str());
           break;
         }
 
