@@ -8,84 +8,38 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaWidget.cxx,v 1.4 2006/04/05 12:28:37 stephena Exp $
+// $Id: TiaWidget.cxx,v 1.13 2008/05/15 15:07:29 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "FrameBuffer.hxx"
-#include "GuiUtils.hxx"
-#include "GuiObject.hxx"
-#include "TIADebug.hxx"
-#include "Widget.hxx"
-#include "EditTextWidget.hxx"
-#include "DataGridWidget.hxx"
 #include "ColorWidget.hxx"
+#include "DataGridWidget.hxx"
+#include "EditTextWidget.hxx"
+#include "FrameBuffer.hxx"
+#include "GuiObject.hxx"
+#include "OSystem.hxx"
+#include "TIADebug.hxx"
 #include "ToggleBitWidget.hxx"
 #include "TogglePixelWidget.hxx"
+#include "Widget.hxx"
+
 #include "TiaWidget.hxx"
 
-// ID's for the various widgets
-// We need ID's, since there are more than one of several types of widgets
-enum {
-  kP0_PFID,   kP0_BLID,   kP0_M1ID,   kP0_M0ID,   kP0_P1ID,
-  kP1_PFID,   kP1_BLID,   kP1_M1ID,   kP1_M0ID,
-  kM0_PFID,   kM0_BLID,   kM0_M1ID,
-  kM1_PFID,   kM1_BLID,
-  kBL_PFID,   // Make these first, since we want them to start from 0
-
-  kRamID,
-  kColorRegsID,
-  kGRP0ID,    kGRP1ID,
-  kPosP0ID,   kPosP1ID,
-  kPosM0ID,   kPosM1ID,   kPosBLID,
-  kHMP0ID,    kHMP1ID,
-  kHMM0ID,    kHMM1ID,    kHMBLID,
-  kRefP0ID,   kRefP1ID,
-  kDelP0ID,   kDelP1ID,   kDelBLID,
-  kNusizP0ID, kNusizP1ID,
-  kNusizM0ID, kNusizM1ID, kSizeBLID,
-  kEnaM0ID,   kEnaM1ID,   kEnaBLID,
-  kResMP0ID,  kResMP1ID,
-  kPF0ID,     kPF1ID,     kPF2ID,
-  kRefPFID,   kScorePFID, kPriorityPFID
-};
-
-// Strobe button commands
-enum {
-  kWsyncCmd = 'Swsy',
-  kRsyncCmd = 'Srsy',
-  kResP0Cmd = 'Srp0',
-  kResP1Cmd = 'Srp1',
-  kResM0Cmd = 'Srm0',
-  kResM1Cmd = 'Srm1',
-  kResBLCmd = 'Srbl',
-  kHmoveCmd = 'Shmv',
-  kHmclrCmd = 'Shmc',
-  kCxclrCmd = 'Scxl'
-};
-
-// Color registers
-enum {
-  kCOLUP0Addr,
-  kCOLUP1Addr,
-  kCOLUPFAddr,
-  kCOLUBKAddr
-};
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
                      int x, int y, int w, int h)
   : Widget(boss, font, x, y, w, h),
     CommandSender(boss)
 {
+  _type = kTiaWidget;
+
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight();
@@ -270,17 +224,6 @@ TiaWidget::TiaWidget(GuiObject* boss, const GUI::Font& font,
                        "CxClr", kCxclrCmd);
   b->setTarget(this);
 
-  // Set the strings to be used in the grPx registers
-  // We only do this once because it's the state that changes, not the strings
-  const char* offstr[] = { "0", "0", "0", "0", "0", "0", "0", "0" };
-  const char* onstr[]  = { "1", "1", "1", "1", "1", "1", "1", "1" };
-  StringList off, on;
-  for(int i = 0; i < 8; ++i)
-  {
-    off.push_back(offstr[i]);
-    on.push_back(onstr[i]);
-  }
-
   ////////////////////////////
   // P0 register info
   ////////////////////////////
@@ -798,7 +741,9 @@ void TiaWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           addr  = myRamGrid->getSelectedAddr();
           value = myRamGrid->getSelectedValue();
 
-          myLabel->setEditString(dbg.equates()->getLabel(addr));
+          // We're using the read-addresses here
+          // Should we also add write-addresses, or remove this entirely?
+          myLabel->setEditString(dbg.equates().getLabel(addr, true));
 
           myDecValue->setEditString(dbg.valueToString(value, kBASE_10));
           myBinValue->setEditString(dbg.valueToString(value, kBASE_2));
@@ -929,8 +874,8 @@ void TiaWidget::fillGrid()
 
   Debugger& dbg = instance()->debugger();
   TIADebug& tia = dbg.tiaDebug();
-  TiaState state    = (TiaState&) tia.getState();
-  TiaState oldstate = (TiaState&) tia.getOldState();
+  TiaState& state    = (TiaState&) tia.getState();
+  TiaState& oldstate = (TiaState&) tia.getOldState();
 
   // TIA RAM
   alist.clear();  vlist.clear();  changed.clear();
