@@ -8,18 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: TiaZoomWidget.cxx,v 1.7 2006/03/25 00:34:17 stephena Exp $
+// $Id: TiaZoomWidget.cxx,v 1.15 2008/02/06 13:45:20 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
 #include "OSystem.hxx"
+#include "Console.hxx"
 #include "FrameBuffer.hxx"
 #include "Widget.hxx"
 #include "GuiObject.hxx"
@@ -29,15 +30,19 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 TiaZoomWidget::TiaZoomWidget(GuiObject* boss, const GUI::Font& font,
-                             int x, int y)
+                             int x, int y, int w, int h)
   : Widget(boss, font, x, y, 16, 16),
     CommandSender(boss),
     myMenu(NULL)
 {
   _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _type = kTiaZoomWidget;
+  _bgcolor = _bgcolorhi = kDlgColor;
 
-  _w = 210;
-  _h = 120;
+  // Use all available space, up to the maximum bounds of the TIA image
+  // Width myst 
+  _w = BSPF_min(w, 320);
+  _h = BSPF_min(h, 260);
 
   addFocusWidget(this);
 
@@ -142,30 +147,46 @@ void TiaZoomWidget::handleMouseDown(int x, int y, int button, int clickCount)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool TiaZoomWidget::handleKeyDown(int ascii, int keycode, int modifiers)
+bool TiaZoomWidget::handleEvent(Event::Type event)
 {
-  bool handled = false;
+  bool handled = true;
 
-  switch(ascii)
+  switch(event)
   {
-    case 256+17:  // up arrow
+    case Event::UIUp:
       myYCenter -= 4;
-      handled = true;
       break;
 
-    case 256+18:  // down arrow
+    case Event::UIDown:
       myYCenter += 4;
-      handled = true;
       break;
 
-    case 256+20:  // left arrow
+    case Event::UILeft:
       myXCenter -= 2;
-      handled = true;
       break;
 
-    case 256+19:  // right arrow
+    case Event::UIRight:
       myXCenter += 2;
-      handled = true;
+      break;
+
+    case Event::UIPgUp:
+      myYCenter = 0;
+      break;
+
+    case Event::UIPgDown:
+      myYCenter = 260;
+      break;
+
+    case Event::UIHome:
+      myXCenter = 0;
+      break;
+
+    case Event::UIEnd:
+      myXCenter = 320;
+      break;
+
+    default:
+      handled = false;
       break;
   }
 
