@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ToggleBitWidget.cxx,v 1.3 2006/02/22 17:38:04 stephena Exp $
+// $Id: ToggleBitWidget.cxx,v 1.8 2008/02/06 13:45:20 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -32,6 +32,8 @@ ToggleBitWidget::ToggleBitWidget(GuiObject* boss, const GUI::Font& font,
                                  int x, int y, int cols, int rows, int colchars)
   : ToggleWidget(boss, font, x, y, cols, rows)
 {
+  _type = kToggleBitWidget;
+
   _rowHeight = font.getLineHeight();
   _colWidth  = colchars * font.getMaxCharWidth() + 8;
 
@@ -105,8 +107,8 @@ void ToggleBitWidget::drawWidget(bool hilite)
       // Highlight changes
       if(_changedList[pos])
       {
-        fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kTextColorEm);
-        fb.drawString(_font, buffer, x, y, _colWidth, kTextColorHi);
+        fb.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kDbgChangedColor);
+        fb.drawString(_font, buffer, x, y, _colWidth, kDbgChangedTextColor);
       }
       else
         fb.drawString(_font, buffer, x, y, _colWidth, kTextColor);
