@@ -8,26 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart3E.cxx,v 1.9 2005/12/17 01:23:07 stephena Exp $
+// $Id: Cart3E.cxx,v 1.15 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "Cart3E.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
 #include "TIA.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "Cart3E.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge3E::Cartridge3E(const uInt8* image, uInt32 size)
-    : mySize(size)
+  : mySize(size)
 {
   // Allocate array for the ROM image
   myImage = new uInt8[mySize];
@@ -52,12 +50,6 @@ Cartridge3E::~Cartridge3E()
   delete[] myImage;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* Cartridge3E::name() const
-{
-  return "Cartridge3E";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3E::reset()
 {
@@ -141,28 +133,10 @@ void Cartridge3E::poke(uInt16 address, uInt8 value)
   mySystem->tia().poke(address, value);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge3E::patch(uInt16 address, uInt8 value)
-{
-  address = address & 0x0FFF;
-  if(address < 0x0800)
-  {
-    if(myCurrentBank < 256)
-      myImage[(address & 0x07FF) + myCurrentBank * 2048] = value;
-    else
-      myRam[(address & 0x03FF) + (myCurrentBank - 256) * 1024] = value;
-  }
-  else
-  {
-    myImage[(address & 0x07FF) + mySize - 2048] = value;
-  }
-  return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge3E::bank(uInt16 bank)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   if(bank < 256)
   {
@@ -227,18 +201,44 @@ void Cartridge3E::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge3E::bank() {
+int Cartridge3E::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int Cartridge3E::bankCount() {
-  return mySize/2048;
+int Cartridge3E::bankCount()
+{
+  return mySize / 2048;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge3E::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  if(address < 0x0800)
+  {
+    if(myCurrentBank < 256)
+      myImage[(address & 0x07FF) + myCurrentBank * 2048] = value;
+    else
+      myRam[(address & 0x03FF) + (myCurrentBank - 256) * 1024] = value;
+  }
+  else
+  {
+    myImage[(address & 0x07FF) + mySize - 2048] = value;
+  }
+  return true;
+} 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge3E::save(Serializer& out)
+uInt8* Cartridge3E::getImage(int& size)
+{
+  size = mySize;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge3E::save(Serializer& out) const
 {
   string cart = name();
 
@@ -250,9 +250,9 @@ bool Cartridge3E::save(Serializer& out)
     // Output RAM
     out.putInt(32768);
     for(uInt32 addr = 0; addr < 32768; ++addr)
-      out.putInt(myRam[addr]);
+      out.putByte((char)myRam[addr]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -281,9 +281,9 @@ bool Cartridge3E::load(Deserializer& in)
     // Input RAM
     uInt32 limit = (uInt32) in.getInt();
     for(uInt32 addr = 0; addr < limit; ++addr)
-      myRam[addr] = (uInt8) in.getInt();
+      myRam[addr] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -299,9 +299,3 @@ bool Cartridge3E::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge3E::getImage(int& size) {
-  size = mySize;
-  return &myImage[0];
-}
