@@ -8,20 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Cart4K.cxx,v 1.7 2005/07/30 16:25:48 urchlay Exp $
+// $Id: Cart4K.cxx,v 1.13 2008/02/19 12:33:03 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "Cart4K.hxx"
+#include <cassert>
+
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "Cart4K.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge4K::Cartridge4K(const uInt8* image)
@@ -38,12 +36,6 @@ Cartridge4K::~Cartridge4K()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* Cartridge4K::name() const
-{
-  return "Cartridge4K";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge4K::reset()
 {
@@ -83,15 +75,41 @@ void Cartridge4K::poke(uInt16, uInt8)
   // This is ROM so poking has no effect :-)
 } 
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Cartridge4K::bank(uInt16)
+{
+  // Doesn't support bankswitching
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int Cartridge4K::bank()
+{
+  // Doesn't support bankswitching
+  return 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int Cartridge4K::bankCount()
+{
+  return 1;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Cartridge4K::patch(uInt16 address, uInt8 value)
 {
-	myImage[address & 0x0FFF] = value;
-	return true;
+  myImage[address & 0x0FFF] = value;
+  return true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Cartridge4K::save(Serializer& out)
+uInt8* Cartridge4K::getImage(int& size)
+{
+  size = 4096;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge4K::save(Serializer& out) const
 {
   string cart = name();
 
@@ -99,7 +117,7 @@ bool Cartridge4K::save(Serializer& out)
   {
     out.putString(cart);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -123,7 +141,7 @@ bool Cartridge4K::load(Deserializer& in)
     if(in.getString() != cart)
       return false;
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -136,9 +154,3 @@ bool Cartridge4K::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* Cartridge4K::getImage(int& size) {
-  size = 4096;
-  return &myImage[0];
-}
