@@ -8,27 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartAR.cxx,v 1.14 2005/12/23 20:48:50 stephena Exp $
+// $Id: CartAR.cxx,v 1.21 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include <string.h>
-#include "CartAR.hxx"
+#include <cassert>
+
 #include "M6502Hi.hxx"
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartAR.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeAR::CartridgeAR(const uInt8* image, uInt32 size)
-    : my6502(0)
+CartridgeAR::CartridgeAR(const uInt8* image, uInt32 size, bool fastbios)
+  : my6502(0)
 {
   uInt32 i;
 
@@ -45,7 +42,7 @@ CartridgeAR::CartridgeAR(const uInt8* image, uInt32 size)
   }
 
   // Initialize SC BIOS ROM
-  initializeROM();
+  initializeROM(fastbios);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -54,12 +51,6 @@ CartridgeAR::~CartridgeAR()
   delete[] myLoadImages;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeAR::name() const
-{
-  return "CartridgeAR";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeAR::reset()
 {
@@ -197,13 +188,6 @@ void CartridgeAR::poke(uInt16 addr, uInt8)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeAR::patch(uInt16 address, uInt8 value)
-{
-  // myImage[address & 0x0FFF] = value;
-  return false;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeAR::bankConfiguration(uInt8 configuration)
 {
@@ -298,28 +282,7 @@ void CartridgeAR::bankConfiguration(uInt8 configuration)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeAR::bank(uInt16 b)
-{
-  if(bankLocked)
-    return;
-
-  bankConfiguration(b);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeAR::bank()
-{
-  return myCurrentBank;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeAR::bankCount()
-{
-  return 32;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeAR::initializeROM()
+void CartridgeAR::initializeROM(bool fastbios)
 {
   static uInt8 dummyROMCode[] = {
     0xa5, 0xfa, 0x85, 0x80, 0x4c, 0x18, 0xf8, 0xff, 
@@ -361,6 +324,11 @@ void CartridgeAR::initializeROM()
     0x4c
   };
 
+  // If fastbios is enabled, set the wait time between vertical bars
+  // to 0 (default is 8), which is stored at address 189 of the bios
+  if(fastbios)
+    dummyROMCode[189] = 0x0;
+
   uInt32 size = sizeof(dummyROMCode);
 
   // Initialize ROM with illegal 6502 opcode that causes a real 6502 to jam
@@ -453,7 +421,41 @@ void CartridgeAR::loadIntoRAM(uInt8 load)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeAR::save(Serializer& out)
+void CartridgeAR::bank(uInt16 bank)
+{
+  if(myBankLocked) return;
+
+  bankConfiguration(bank);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeAR::bank()
+{
+  return myCurrentBank;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeAR::bankCount()
+{
+  return 32;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeAR::patch(uInt16 address, uInt8 value)
+{
+  // myImage[address & 0x0FFF] = value;
+  return false;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeAR::getImage(int& size)
+{
+  size = myNumberOfLoadImages * 8448;
+  return &myLoadImages[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeAR::save(Serializer& out) const
 {
   string cart = name();
 
@@ -471,12 +473,12 @@ bool CartridgeAR::save(Serializer& out)
     // The 6K of RAM and 2K of ROM contained in the Supercharger
     out.putInt(8192);
     for(i = 0; i < 8192; ++i)
-      out.putInt(myImage[i]);
+      out.putByte((char)myImage[i]);
 
     // The 256 byte header for the current 8448 byte load
     out.putInt(256);
     for(i = 0; i < 256; ++i)
-      out.putInt(myHeader[i]);
+      out.putByte((char)myHeader[i]);
 
     // All of the 8448 byte loads associated with the game 
     // Note that the size of this array is myNumberOfLoadImages * 8448
@@ -485,7 +487,7 @@ bool CartridgeAR::save(Serializer& out)
       out.putInt(myLoadImages[i]);
 
     // Indicates how many 8448 loads there are
-    out.putInt(myNumberOfLoadImages);
+    out.putByte((char)myNumberOfLoadImages);
 
     // Indicates if the RAM is write enabled
     out.putBool(myWriteEnabled);
@@ -497,7 +499,7 @@ bool CartridgeAR::save(Serializer& out)
     out.putInt(myPowerRomCycle);
 
     // Data hold register used for writing
-    out.putInt(myDataHoldRegister);
+    out.putByte((char)myDataHoldRegister);
 
     // Indicates number of distinct accesses when data hold register was set
     out.putInt(myNumberOfDistinctAccesses);
@@ -505,7 +507,7 @@ bool CartridgeAR::save(Serializer& out)
     // Indicates if a write is pending or not
     out.putBool(myWritePending);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -539,12 +541,12 @@ bool CartridgeAR::load(Deserializer& in)
     // The 6K of RAM and 2K of ROM contained in the Supercharger
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myImage[i] = (uInt8) in.getInt();
+      myImage[i] = (uInt8) in.getByte();
 
     // The 256 byte header for the current 8448 byte load
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myHeader[i] = (uInt8) in.getInt();
+      myHeader[i] = (uInt8) in.getByte();
 
     // All of the 8448 byte loads associated with the game 
     // Note that the size of this array is myNumberOfLoadImages * 8448
@@ -553,7 +555,7 @@ bool CartridgeAR::load(Deserializer& in)
       myLoadImages[i] = (uInt8) in.getInt();
 
     // Indicates how many 8448 loads there are
-    myNumberOfLoadImages = (uInt8) in.getInt();
+    myNumberOfLoadImages = (uInt8) in.getByte();
 
     // Indicates if the RAM is write enabled
     myWriteEnabled = in.getBool();
@@ -565,7 +567,7 @@ bool CartridgeAR::load(Deserializer& in)
     myPowerRomCycle = (Int32) in.getInt();
 
     // Data hold register used for writing
-    myDataHoldRegister = (uInt8) in.getInt();
+    myDataHoldRegister = (uInt8) in.getByte();
 
     // Indicates number of distinct accesses when data hold register was set
     myNumberOfDistinctAccesses = (uInt32) in.getInt();
@@ -573,7 +575,7 @@ bool CartridgeAR::load(Deserializer& in)
     // Indicates if a write is pending or not
     myWritePending = in.getBool();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -586,10 +588,3 @@ bool CartridgeAR::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeAR::getImage(int& size)
-{
-  size = myNumberOfLoadImages * 8448;
-  return &myLoadImages[0];
-}
