@@ -8,21 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartCV.cxx,v 1.10 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartCV.cxx,v 1.16 2008/02/06 13:45:21 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartCV.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartCV.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCV::CartridgeCV(const uInt8* image, uInt32 size)
@@ -68,12 +66,6 @@ CartridgeCV::~CartridgeCV()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeCV::name() const
-{
-  return "CartridgeCV";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCV::reset()
 {
@@ -131,15 +123,41 @@ void CartridgeCV::poke(uInt16, uInt8)
   // This is ROM so poking has no effect :-)
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void CartridgeCV::bank(uInt16 bank)
+{
+  // Doesn't support bankswitching
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeCV::bank()
+{
+  // Doesn't support bankswitching
+  return 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeCV::bankCount()
+{
+  return 1;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeCV::patch(uInt16 address, uInt8 value)
 {
-	myImage[address & 0x07FF] = value;
-	return true;
+  myImage[address & 0x07FF] = value;
+  return true;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeCV::save(Serializer& out)
+uInt8* CartridgeCV::getImage(int& size)
+{
+  size = 2048;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeCV::save(Serializer& out) const
 {
   string cart = name();
 
@@ -150,9 +168,9 @@ bool CartridgeCV::save(Serializer& out)
     // Output RAM
     out.putInt(1024);
     for(uInt32 addr = 0; addr < 1024; ++addr)
-      out.putInt(myRAM[addr]);
+      out.putByte((char)myRAM[addr]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -179,9 +197,9 @@ bool CartridgeCV::load(Deserializer& in)
     // Input RAM
     uInt32 limit = (uInt32) in.getInt();
     for(uInt32 addr = 0; addr < limit; ++addr)
-      myRAM[addr] = (uInt8) in.getInt();
+      myRAM[addr] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -194,9 +212,3 @@ bool CartridgeCV::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeCV::getImage(int& size) {
-  size = 2048;
-  return &myImage[0];
-}
