@@ -8,20 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartDPC.cxx,v 1.15 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartDPC.cxx,v 1.21 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
+#include <cassert>
 #include <iostream>
-#include "CartDPC.hxx"
+
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
+#include "CartDPC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeDPC::CartridgeDPC(const uInt8* image, uInt32 size)
@@ -67,12 +66,6 @@ CartridgeDPC::~CartridgeDPC()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeDPC::name() const
-{
-  return "CartridgeDPC";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeDPC::reset()
 {
@@ -427,18 +420,10 @@ void CartridgeDPC::poke(uInt16 address, uInt8 value)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeDPC::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myProgramImage[myCurrentBank * 4096 + address] = value;
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeDPC::bank(uInt16 bank)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -461,18 +446,42 @@ void CartridgeDPC::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeDPC::bank() {
+int CartridgeDPC::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeDPC::bankCount() {
+int CartridgeDPC::bankCount()
+{
   return 2; // TODO: support the display ROM somehow
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeDPC::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myProgramImage[myCurrentBank * 4096 + address] = value;
+  return true;
+} 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeDPC::save(Serializer& out)
+uInt8* CartridgeDPC::getImage(int& size)
+{
+  size = 8192 + 2048 + 255;
+
+  int i;
+  for(i = 0; i < 8192; i++)
+    myImageCopy[i] = myProgramImage[i];
+
+  for(i = 0; i < 2048; i++)
+    myImageCopy[i + 8192] = myDisplayImage[i];
+
+  return &myImageCopy[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeDPC::save(Serializer& out) const
 {
   string cart = name();
 
@@ -488,12 +497,12 @@ bool CartridgeDPC::save(Serializer& out)
     // The top registers for the data fetchers
     out.putInt(8);
     for(i = 0; i < 8; ++i)
-      out.putInt(myTops[i]);
+      out.putByte((char)myTops[i]);
 
     // The bottom registers for the data fetchers
     out.putInt(8);
     for(i = 0; i < 8; ++i)
-      out.putInt(myBottoms[i]);
+      out.putByte((char)myBottoms[i]);
 
     // The counter registers for the data fetchers
     out.putInt(8);
@@ -503,7 +512,7 @@ bool CartridgeDPC::save(Serializer& out)
     // The flag registers for the data fetchers
     out.putInt(8);
     for(i = 0; i < 8; ++i)
-      out.putInt(myFlags[i]);
+      out.putByte((char)myFlags[i]);
 
     // The music mode flags for the data fetchers
     out.putInt(3);
@@ -511,12 +520,12 @@ bool CartridgeDPC::save(Serializer& out)
       out.putBool(myMusicMode[i]);
 
     // The random number generator register
-    out.putInt(myRandomNumber);
+    out.putByte((char)myRandomNumber);
 
     out.putInt(mySystemCycles);
     out.putInt((uInt32)(myFractionalClocks * 100000000.0));
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -548,12 +557,12 @@ bool CartridgeDPC::load(Deserializer& in)
     // The top registers for the data fetchers
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myTops[i] = (uInt8) in.getInt();
+      myTops[i] = (uInt8) in.getByte();
 
     // The bottom registers for the data fetchers
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myBottoms[i] = (uInt8) in.getInt();
+      myBottoms[i] = (uInt8) in.getByte();
 
     // The counter registers for the data fetchers
     limit = (uInt32) in.getInt();
@@ -563,7 +572,7 @@ bool CartridgeDPC::load(Deserializer& in)
     // The flag registers for the data fetchers
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myFlags[i] = (uInt8) in.getInt();
+      myFlags[i] = (uInt8) in.getByte();
 
     // The music mode flags for the data fetchers
     limit = (uInt32) in.getInt();
@@ -571,13 +580,13 @@ bool CartridgeDPC::load(Deserializer& in)
       myMusicMode[i] = in.getBool();
 
     // The random number generator register
-    myRandomNumber = (uInt8) in.getInt();
+    myRandomNumber = (uInt8) in.getByte();
 
     // Get system cycles and fractional clocks
     mySystemCycles = in.getInt();
     myFractionalClocks = (double)in.getInt() / 100000000.0;
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -593,18 +602,3 @@ bool CartridgeDPC::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeDPC::getImage(int& size)
-{
-  size = 8192 + 2048 + 255;
-
-  int i;
-  for(i = 0; i < 8192; i++)
-    myImageCopy[i] = myProgramImage[i];
-
-  for(i = 0; i < 2048; i++)
-    myImageCopy[i + 8192] = myDisplayImage[i];
-
-  return &myImageCopy[0];
-}
