@@ -8,20 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE0.cxx,v 1.10 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartE0.cxx,v 1.16 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartE0.hxx"
+#include <cassert>
+
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartE0.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeE0::CartridgeE0(const uInt8* image)
@@ -38,12 +36,6 @@ CartridgeE0::~CartridgeE0()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeE0::name() const
-{
-  return "CartridgeE0";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::reset()
 {
@@ -95,20 +87,18 @@ uInt8 CartridgeE0::peek(uInt16 address)
 {
   address = address & 0x0FFF;
 
-  if(!bankLocked) {
-    // Switch banks if necessary
-    if((address >= 0x0FE0) && (address <= 0x0FE7))
-    {
-      segmentZero(address & 0x0007);
-    }
-    else if((address >= 0x0FE8) && (address <= 0x0FEF))
-    {
-      segmentOne(address & 0x0007);
-    }
-    else if((address >= 0x0FF0) && (address <= 0x0FF7))
-    {
-      segmentTwo(address & 0x0007);
-    }
+  // Switch banks if necessary
+  if((address >= 0x0FE0) && (address <= 0x0FE7))
+  {
+    segmentZero(address & 0x0007);
+  }
+  else if((address >= 0x0FE8) && (address <= 0x0FEF))
+  {
+    segmentOne(address & 0x0007);
+  }
+  else if((address >= 0x0FF0) && (address <= 0x0FF7))
+  {
+    segmentTwo(address & 0x0007);
   }
 
   return myImage[(myCurrentSlice[address >> 10] << 10) + (address & 0x03FF)];
@@ -119,34 +109,26 @@ void CartridgeE0::poke(uInt16 address, uInt8)
 {
   address = address & 0x0FFF;
 
-  if(!bankLocked) {
-    // Switch banks if necessary
-    if((address >= 0x0FE0) && (address <= 0x0FE7))
-    {
-      segmentZero(address & 0x0007);
-    }
-    else if((address >= 0x0FE8) && (address <= 0x0FEF))
-    {
-      segmentOne(address & 0x0007);
-    }
-    else if((address >= 0x0FF0) && (address <= 0x0FF7))
-    {
-      segmentTwo(address & 0x0007);
-    }
+  // Switch banks if necessary
+  if((address >= 0x0FE0) && (address <= 0x0FE7))
+  {
+    segmentZero(address & 0x0007);
+  }
+  else if((address >= 0x0FE8) && (address <= 0x0FEF))
+  {
+    segmentOne(address & 0x0007);
+  }
+  else if((address >= 0x0FF0) && (address <= 0x0FF7))
+  {
+    segmentTwo(address & 0x0007);
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeE0::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myImage[(myCurrentSlice[address >> 10] << 10) + (address & 0x03FF)] = value;
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentZero(uInt16 slice)
 { 
+  if(myBankLocked) return;
+
   // Remember the new slice
   myCurrentSlice[0] = slice;
   uInt16 offset = slice << 10;
@@ -167,6 +149,8 @@ void CartridgeE0::segmentZero(uInt16 slice)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentOne(uInt16 slice)
 { 
+  if(myBankLocked) return;
+
   // Remember the new slice
   myCurrentSlice[1] = slice;
   uInt16 offset = slice << 10;
@@ -187,6 +171,8 @@ void CartridgeE0::segmentOne(uInt16 slice)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE0::segmentTwo(uInt16 slice)
 { 
+  if(myBankLocked) return;
+
   // Remember the new slice
   myCurrentSlice[2] = slice;
   uInt16 offset = slice << 10;
@@ -205,7 +191,42 @@ void CartridgeE0::segmentTwo(uInt16 slice)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeE0::save(Serializer& out)
+void CartridgeE0::bank(uInt16)
+{
+  // Doesn't support bankswitching in the normal sense
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeE0::bank()
+{
+  // Doesn't support bankswitching in the normal sense
+  return 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeE0::bankCount()
+{
+  // Doesn't support bankswitching in the normal sense
+  return 1;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeE0::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myImage[(myCurrentSlice[address >> 10] << 10) + (address & 0x03FF)] = value;
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeE0::getImage(int& size)
+{
+  size = 8192;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeE0::save(Serializer& out) const
 {
   string cart = name();
 
@@ -217,7 +238,7 @@ bool CartridgeE0::save(Serializer& out)
     for(uInt32 i = 0; i < 4; ++i)
       out.putInt(myCurrentSlice[i]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -245,7 +266,7 @@ bool CartridgeE0::load(Deserializer& in)
     for(uInt32 i = 0; i < limit; ++i)
       myCurrentSlice[i] = (uInt16) in.getInt();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -258,10 +279,3 @@ bool CartridgeE0::load(Deserializer& in)
 
   return true;
 }
-
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeE0::getImage(int& size) {
-  size = 8192;
-  return &myImage[0];
-}
