@@ -8,21 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartE7.cxx,v 1.13 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartE7.cxx,v 1.19 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartE7.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartE7.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeE7::CartridgeE7(const uInt8* image)
@@ -46,12 +44,6 @@ CartridgeE7::~CartridgeE7()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeE7::name() const
-{
-  return "CartridgeE7";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::reset()
 {
@@ -138,18 +130,40 @@ void CartridgeE7::poke(uInt16 address, uInt8)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeE7::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)] = value;
-	bank(myCurrentSlice[0]);
-	return true;
-} 
+void CartridgeE7::bankRAM(uInt16 bank)
+{ 
+  // Remember what bank we're in
+  myCurrentRAM = bank;
+  uInt16 offset = bank << 8;
+  uInt16 shift = mySystem->pageShift();
+
+  // Setup the page access methods for the current bank
+  System::PageAccess access;
+  access.device = this;
+
+  // Set the page accessing method for the 256 bytes of RAM writing pages
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  for(uInt32 j = 0x1800; j < 0x1900; j += (1 << shift))
+  {
+    access.directPokeBase = &myRAM[1024 + offset + (j & 0x00FF)];
+    mySystem->setPageAccess(j >> shift, access);
+  }
+
+  // Set the page accessing method for the 256 bytes of RAM reading pages
+  access.directPeekBase = 0;
+  access.directPokeBase = 0;
+  for(uInt32 k = 0x1900; k < 0x1A00; k += (1 << shift))
+  {
+    access.directPeekBase = &myRAM[1024 + offset + (k & 0x00FF)];
+    mySystem->setPageAccess(k >> shift, access);
+  }
+}
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeE7::bank(uInt16 slice)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentSlice[0] = slice;
@@ -196,48 +210,35 @@ void CartridgeE7::bank(uInt16 slice)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE7::bank() {
+int CartridgeE7::bank()
+{
   return myCurrentSlice[0];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeE7::bankCount() {
+int CartridgeE7::bankCount()
+{
   return 8;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeE7::bankRAM(uInt16 bank)
-{ 
-  // Remember what bank we're in
-  myCurrentRAM = bank;
-  uInt16 offset = bank << 8;
-  uInt16 shift = mySystem->pageShift();
+bool CartridgeE7::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myImage[(myCurrentSlice[address >> 11] << 11) + (address & 0x07FF)] = value;
+  bank(myCurrentSlice[0]);
+  return true;
+} 
 
-  // Setup the page access methods for the current bank
-  System::PageAccess access;
-  access.device = this;
-
-  // Set the page accessing method for the 256 bytes of RAM writing pages
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  for(uInt32 j = 0x1800; j < 0x1900; j += (1 << shift))
-  {
-    access.directPokeBase = &myRAM[1024 + offset + (j & 0x00FF)];
-    mySystem->setPageAccess(j >> shift, access);
-  }
-
-  // Set the page accessing method for the 256 bytes of RAM reading pages
-  access.directPeekBase = 0;
-  access.directPokeBase = 0;
-  for(uInt32 k = 0x1900; k < 0x1A00; k += (1 << shift))
-  {
-    access.directPeekBase = &myRAM[1024 + offset + (k & 0x00FF)];
-    mySystem->setPageAccess(k >> shift, access);
-  }
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeE7::getImage(int& size)
+{
+  size = 16384;
+  return &myImage[0];
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeE7::save(Serializer& out)
+bool CartridgeE7::save(Serializer& out) const
 {
   string cart = name();
 
@@ -256,9 +257,9 @@ bool CartridgeE7::save(Serializer& out)
     // The 2048 bytes of RAM
     out.putInt(2048);
     for(i = 0; i < 2048; ++i)
-      out.putInt(myRAM[i]);
+      out.putByte((char)myRAM[i]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -293,9 +294,9 @@ bool CartridgeE7::load(Deserializer& in)
     // The 2048 bytes of RAM
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getInt();
+      myRAM[i] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -312,9 +313,3 @@ bool CartridgeE7::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeE7::getImage(int& size) {
-  size = 16384;
-  return &myImage[0];
-}
