@@ -8,21 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF4.cxx,v 1.7 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartF4.cxx,v 1.14 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include <iostream>
-#include "CartF4.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
+#include "CartF4.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeF4::CartridgeF4(const uInt8* image)
@@ -39,17 +37,11 @@ CartridgeF4::~CartridgeF4()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeF4::name() const
-{
-  return "CartridgeF4";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4::reset()
 {
-  // Upon reset we switch to bank 7
-  bank(7);
+  // Upon reset we switch to bank 0
+  bank(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -72,8 +64,8 @@ void CartridgeF4::install(System& system)
     mySystem->setPageAccess(i >> shift, access);
   }
 
-  // Install pages for bank 7
-  bank(7);
+  // Install pages for bank 0
+  bank(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -102,18 +94,10 @@ void CartridgeF4::poke(uInt16 address, uInt8)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF4::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myImage[myCurrentBank * 4096 + address] = value;
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4::bank(uInt16 bank)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -136,17 +120,34 @@ void CartridgeF4::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF4::bank() {
+int CartridgeF4::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF4::bankCount() {
+int CartridgeF4::bankCount()
+{
   return 8;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF4::save(Serializer& out)
+bool CartridgeF4::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myImage[myCurrentBank * 4096 + address] = value;
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeF4::getImage(int& size)
+{
+  size = 32768;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeF4::save(Serializer& out) const
 {
   string cart = name();
 
@@ -155,7 +156,7 @@ bool CartridgeF4::save(Serializer& out)
     out.putString(cart);
     out.putInt(myCurrentBank);
   }
-  catch(char* msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -183,7 +184,7 @@ bool CartridgeF4::load(Deserializer& in)
 
     myCurrentBank = (uInt16)in.getInt();
   }
-  catch(char* msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -199,10 +200,3 @@ bool CartridgeF4::load(Deserializer& in)
 
   return true;
 }
-
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeF4::getImage(int& size) {
-  size = 32768;
-  return &myImage[0];
-}
