@@ -8,21 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF4SC.cxx,v 1.10 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartF4SC.cxx,v 1.17 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartF4SC.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartF4SC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeF4SC::CartridgeF4SC(const uInt8* image)
@@ -46,17 +44,11 @@ CartridgeF4SC::~CartridgeF4SC()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeF4SC::name() const
-{
-  return "CartridgeF4SC";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4SC::reset()
 {
-  // Upon reset we switch to bank 7
-  bank(7);
+  // Upon reset we switch to bank 0
+  bank(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -97,8 +89,8 @@ void CartridgeF4SC::install(System& system)
     mySystem->setPageAccess(k >> shift, access);
   }
 
-  // Install pages for bank 7
-  bank(7);
+  // Install pages for bank 0
+  bank(0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -132,18 +124,10 @@ void CartridgeF4SC::poke(uInt16 address, uInt8)
   // has been setup
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF4SC::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myImage[myCurrentBank * 4096 + address] = value;
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF4SC::bank(uInt16 bank)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -166,17 +150,34 @@ void CartridgeF4SC::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF4SC::bank() {
+int CartridgeF4SC::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF4SC::bankCount() {
+int CartridgeF4SC::bankCount()
+{
   return 8;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF4SC::save(Serializer& out)
+bool CartridgeF4SC::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myImage[myCurrentBank * 4096 + address] = value;
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeF4SC::getImage(int& size)
+{
+  size = 32768;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeF4SC::save(Serializer& out) const
 {
   string cart = name();
 
@@ -189,9 +190,9 @@ bool CartridgeF4SC::save(Serializer& out)
     // The 128 bytes of RAM
     out.putInt(128);
     for(uInt32 i = 0; i < 128; ++i)
-      out.putInt(myRAM[i]);
+      out.putByte((char)myRAM[i]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -219,9 +220,9 @@ bool CartridgeF4SC::load(Deserializer& in)
 
     uInt32 limit = (uInt32) in.getInt();
     for(uInt32 i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getInt();
+      myRAM[i] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -237,9 +238,3 @@ bool CartridgeF4SC::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeF4SC::getImage(int& size) {
-  size = 32768;
-  return &myImage[0];
-}
