@@ -8,29 +8,31 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF8.cxx,v 1.11 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartF8.cxx,v 1.18 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartF8.hxx"
+#include <cassert>
+
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartF8.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CartridgeF8::CartridgeF8(const uInt8* image)
+CartridgeF8::CartridgeF8(const uInt8* image, bool swapbanks)
 {
   // Copy the ROM image into my buffer
   for(uInt32 addr = 0; addr < 8192; ++addr)
   {
     myImage[addr] = image[addr];
   }
+
+  // Normally bank 1 is the reset bank, unless we're dealing with ROMs
+  // that have been incorrectly created with banks in the opposite order
+  myResetBank = swapbanks ? 0 : 1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -38,17 +40,11 @@ CartridgeF8::~CartridgeF8()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeF8::name() const
-{
-  return "CartridgeF8";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF8::reset()
 {
-  // Upon reset we switch to bank 1
-  bank(1);
+  // Upon reset we switch to the reset bank
+  bank(myResetBank);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -123,19 +119,10 @@ void CartridgeF8::poke(uInt16 address, uInt8)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF8::patch(uInt16 address, uInt8 value)
-{
-	address &= 0xfff;
-	myImage[myCurrentBank * 4096 + address] = value;
-	bank(myCurrentBank);
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF8::bank(uInt16 bank)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -158,17 +145,35 @@ void CartridgeF8::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF8::bank() {
+int CartridgeF8::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF8::bankCount() {
+int CartridgeF8::bankCount()
+{
   return 2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF8::save(Serializer& out)
+bool CartridgeF8::patch(uInt16 address, uInt8 value)
+{
+  address &= 0xfff;
+  myImage[myCurrentBank * 4096 + address] = value;
+  bank(myCurrentBank);
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeF8::getImage(int& size)
+{
+  size = 8192;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeF8::save(Serializer& out) const
 {
   string cart = name();
 
@@ -178,7 +183,7 @@ bool CartridgeF8::save(Serializer& out)
 
     out.putInt(myCurrentBank);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -204,7 +209,7 @@ bool CartridgeF8::load(Deserializer& in)
 
     myCurrentBank = (uInt16) in.getInt();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -220,9 +225,3 @@ bool CartridgeF8::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeF8::getImage(int& size) {
-  size = 8192;
-  return &myImage[0];
-}
