@@ -8,21 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartF8SC.cxx,v 1.9 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartF8SC.cxx,v 1.16 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartF8SC.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartF8SC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeF8SC::CartridgeF8SC(const uInt8* image)
@@ -46,12 +44,6 @@ CartridgeF8SC::~CartridgeF8SC()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeF8SC::name() const
-{
-  return "CartridgeF8SC";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF8SC::reset()
 {
@@ -106,23 +98,21 @@ uInt8 CartridgeF8SC::peek(uInt16 address)
 {
   address = address & 0x0FFF;
 
-  if(!bankLocked) {
-    // Switch banks if necessary
-    switch(address)
-    {
-      case 0x0FF8:
-        // Set the current bank to the lower 4k bank
-        bank(0);
-        break;
+  // Switch banks if necessary
+  switch(address)
+  {
+    case 0x0FF8:
+      // Set the current bank to the lower 4k bank
+      bank(0);
+      break;
   
-      case 0x0FF9:
-        // Set the current bank to the upper 4k bank
-        bank(1);
-        break;
+    case 0x0FF9:
+      // Set the current bank to the upper 4k bank
+      bank(1);
+      break;
   
-      default:
-        break;
-    }
+    default:
+      break;
   }
 
   // NOTE: This does not handle accessing RAM, however, this function
@@ -136,23 +126,21 @@ void CartridgeF8SC::poke(uInt16 address, uInt8)
 {
   address = address & 0x0FFF;
 
-  if(!bankLocked) {
-    // Switch banks if necessary
-    switch(address)
-    {
-      case 0x0FF8:
-        // Set the current bank to the lower 4k bank
-        bank(0);
-        break;
+  // Switch banks if necessary
+  switch(address)
+  {
+    case 0x0FF8:
+      // Set the current bank to the lower 4k bank
+      bank(0);
+      break;
   
-      case 0x0FF9:
-        // Set the current bank to the upper 4k bank
-        bank(1);
-        break;
+    case 0x0FF9:
+      // Set the current bank to the upper 4k bank
+      bank(1);
+      break;
   
-      default:
-        break;
-    }
+    default:
+      break;
   }
 
   // NOTE: This does not handle accessing RAM, however, this function
@@ -160,27 +148,11 @@ void CartridgeF8SC::poke(uInt16 address, uInt8)
   // has been setup
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF8SC::bank() {
-  return myCurrentBank;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeF8SC::bankCount() {
-  return 2;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF8SC::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myImage[myCurrentBank * 4096 + address] = value;
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeF8SC::bank(uInt16 bank)
 { 
+  if(myBankLocked) return;
+
   // Remember what bank we're in
   myCurrentBank = bank;
   uInt16 offset = myCurrentBank << 12;
@@ -202,7 +174,34 @@ void CartridgeF8SC::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeF8SC::save(Serializer& out)
+int CartridgeF8SC::bank()
+{
+  return myCurrentBank;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeF8SC::bankCount()
+{
+  return 2;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeF8SC::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myImage[myCurrentBank * 4096 + address] = value;
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeF8SC::getImage(int& size)
+{
+  size = 8192;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeF8SC::save(Serializer& out) const
 {
   string cart = name();
 
@@ -215,9 +214,9 @@ bool CartridgeF8SC::save(Serializer& out)
     // The 128 bytes of RAM
     out.putInt(128);
     for(uInt32 i = 0; i < 128; ++i)
-      out.putInt(myRAM[i]);
+      out.putByte((char)myRAM[i]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -245,9 +244,9 @@ bool CartridgeF8SC::load(Deserializer& in)
 
     uInt32 limit = (uInt32) in.getInt();
     for(uInt32 i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getInt();
+      myRAM[i] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -263,9 +262,3 @@ bool CartridgeF8SC::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeF8SC::getImage(int& size) {
-  size = 8192;
-  return &myImage[0];
-}
