@@ -8,20 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartMB.cxx,v 1.8 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartMB.cxx,v 1.14 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartMB.hxx"
+#include <cassert>
+
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartMB.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeMB::CartridgeMB(const uInt8* image)
@@ -38,12 +36,6 @@ CartridgeMB::~CartridgeMB()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeMB::name() const
-{
-  return "CartridgeMB";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMB::reset()
 {
@@ -97,18 +89,10 @@ void CartridgeMB::poke(uInt16 address, uInt8)
   if(address == 0x0FF0) incbank();
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeMB::patch(uInt16 address, uInt8 value)
-{
-	address = address & 0x0FFF;
-	myImage[myCurrentBank * 4096 + address] = value;
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMB::incbank()
 {
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentBank ++;
@@ -132,23 +116,43 @@ void CartridgeMB::incbank()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void CartridgeMB::bank(uInt16 b) {
-  myCurrentBank = (b - 1);
+void CartridgeMB::bank(uInt16 bank)
+{
+  if(myBankLocked) return;
+
+  myCurrentBank = (bank - 1);
   incbank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeMB::bank() {
+int CartridgeMB::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeMB::bankCount() {
+int CartridgeMB::bankCount()
+{
   return 16;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeMB::save(Serializer& out)
+bool CartridgeMB::patch(uInt16 address, uInt8 value)
+{
+  address = address & 0x0FFF;
+  myImage[myCurrentBank * 4096 + address] = value;
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeMB::getImage(int& size)
+{
+  size = 65536;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMB::save(Serializer& out) const
 {
   string cart = name();
 
@@ -158,7 +162,7 @@ bool CartridgeMB::save(Serializer& out)
 
     out.putInt(myCurrentBank);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -184,7 +188,7 @@ bool CartridgeMB::load(Deserializer& in)
 
     myCurrentBank = (uInt16) in.getInt();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -201,9 +205,3 @@ bool CartridgeMB::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeMB::getImage(int& size) {
-  size = 65536;
-  return &myImage[0];
-}
