@@ -8,25 +8,23 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartMC.cxx,v 1.9 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartMC.cxx,v 1.14 2008/02/06 13:45:21 stephena Exp $
 //============================================================================
 
-#include <assert.h>
-#include "CartMC.hxx"
+#include <cassert>
+
 #include "Random.hxx"
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
-#include <iostream>
+#include "CartMC.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeMC::CartridgeMC(const uInt8* image, uInt32 size)
-    : mySlot3Locked(false)
+  : mySlot3Locked(false)
 {
   uInt32 i;
 
@@ -66,12 +64,6 @@ CartridgeMC::~CartridgeMC()
   delete[] myImage;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeMC::name() const
-{
-  return "CartridgeMC";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeMC::reset()
 {
@@ -219,15 +211,42 @@ void CartridgeMC::poke(uInt16 address, uInt8 value)
   }  
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void CartridgeMC::bank(uInt16 b)
+{
+  // TODO: add support for debugger
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeMC::bank()
+{
+  // TODO: add support for debugger
+  return 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CartridgeMC::bankCount()
+{
+  // TODO: add support for debugger
+  return 1;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool CartridgeMC::patch(uInt16 address, uInt8 value)
 {
-	// TODO: implement
-	return false;
+  // TODO: implement
+  return false;
 } 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeMC::save(Serializer& out)
+uInt8* CartridgeMC::getImage(int& size)
+{
+  size = 128 * 1024; // FIXME: keep track of original size
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeMC::save(Serializer& out) const
 {
   uInt32 i;
   string cart = name();
@@ -239,14 +258,14 @@ bool CartridgeMC::save(Serializer& out)
     // The currentBlock array
     out.putInt(4);
     for(i = 0; i < 4; ++i)
-      out.putInt(myCurrentBlock[i]);
+      out.putByte((char)myCurrentBlock[i]);
 
     // The 32K of RAM
     out.putInt(32 * 1024);
     for(i = 0; i < 32 * 1024; ++i)
-      out.putInt(myRAM[i]);
+      out.putByte((char)myRAM[i]);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -276,14 +295,14 @@ bool CartridgeMC::load(Deserializer& in)
     // The currentBlock array
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myCurrentBlock[i] = (uInt8) in.getInt();
+      myCurrentBlock[i] = (uInt8) in.getByte();
 
     // The 32K of RAM
     limit = (uInt32) in.getInt();
     for(i = 0; i < limit; ++i)
-      myRAM[i] = (uInt8) in.getInt();
+      myRAM[i] = (uInt8) in.getByte();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -296,9 +315,3 @@ bool CartridgeMC::load(Deserializer& in)
 
   return true;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeMC::getImage(int& size) {
-  size = 128 * 1024; // FIXME: keep track of original size
-  return &myImage[0];
-}
