@@ -8,20 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartUA.cxx,v 1.7 2005/12/17 01:23:07 stephena Exp $
+// $Id: CartUA.cxx,v 1.13 2008/03/28 23:29:13 stephena Exp $
 //============================================================================
 
 #include <cassert>
-#include <iostream>
-#include "CartUA.hxx"
+
 #include "System.hxx"
-#include "Serializer.hxx"
-#include "Deserializer.hxx"
+#include "CartUA.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeUA::CartridgeUA(const uInt8* image)
@@ -38,12 +36,6 @@ CartridgeUA::~CartridgeUA()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* CartridgeUA::name() const
-{
-  return "CartridgeUA";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeUA::reset()
 {
@@ -137,19 +129,10 @@ void CartridgeUA::poke(uInt16 address, uInt8 value)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeUA::patch(uInt16 address, uInt8 value)
-{
-	address &= 0x0fff;
-	myImage[myCurrentBank * 4096] = value;
-	bank(myCurrentBank); // TODO: see if this is really necessary
-	return true;
-} 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeUA::bank(uInt16 bank)
 { 
-  if(bankLocked) return;
+  if(myBankLocked) return;
 
   // Remember what bank we're in
   myCurrentBank = bank;
@@ -171,17 +154,35 @@ void CartridgeUA::bank(uInt16 bank)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeUA::bank() {
+int CartridgeUA::bank()
+{
   return myCurrentBank;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int CartridgeUA::bankCount() {
+int CartridgeUA::bankCount()
+{
   return 2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CartridgeUA::save(Serializer& out)
+bool CartridgeUA::patch(uInt16 address, uInt8 value)
+{
+  address &= 0x0fff;
+  myImage[myCurrentBank * 4096] = value;
+  bank(myCurrentBank); // TODO: see if this is really necessary
+  return true;
+} 
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt8* CartridgeUA::getImage(int& size)
+{
+  size = 8192;
+  return &myImage[0];
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool CartridgeUA::save(Serializer& out) const
 {
   string cart = name();
 
@@ -191,7 +192,7 @@ bool CartridgeUA::save(Serializer& out)
 
     out.putInt(myCurrentBank);
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -217,7 +218,7 @@ bool CartridgeUA::load(Deserializer& in)
 
     myCurrentBank = (uInt16)in.getInt();
   }
-  catch(char *msg)
+  catch(const char* msg)
   {
     cerr << msg << endl;
     return false;
@@ -233,10 +234,3 @@ bool CartridgeUA::load(Deserializer& in)
 
   return true;
 }
-
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8* CartridgeUA::getImage(int& size) {
-  size = 8192;
-  return &myImage[0];
-}
