@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Deserializer.cxx,v 1.10 2005/12/29 21:16:26 stephena Exp $
+// $Id: Deserializer.cxx,v 1.14 2008/02/06 13:45:21 stephena Exp $
 //============================================================================
 
 #include "Deserializer.hxx"
@@ -51,6 +51,18 @@ bool Deserializer::isOpen(void)
   return myStream.is_open();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+char Deserializer::getByte(void)
+{
+  if(myStream.eof())
+    throw "Deserializer: end of file";
+
+  char buf[1];
+  myStream.read(buf, 1);
+
+  return buf[0];
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Deserializer::getInt(void)
 {
@@ -85,10 +97,10 @@ bool Deserializer::getBool(void)
 {
   bool result = false;
 
-  int b = getInt();
-  if(b == (int)TruePattern)
+  char b = getByte();
+  if(b == (char)TruePattern)
     result = true;
-  else if(b == (int)FalsePattern)
+  else if(b == (char)FalsePattern)
     result = false;
   else
     throw "Deserializer: data corruption";
