@@ -8,19 +8,18 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNode.cxx,v 1.5 2006/03/19 22:06:20 stephena Exp $
+// $Id: FSNode.cxx,v 1.12 2008/02/06 13:45:21 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
 #include "bspf.hxx"
-#include "GuiUtils.hxx"
 #include "FSNode.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -36,7 +35,7 @@ void FSList::sort()
         min = j;
     }
     if (min != i)
-      SWAP(_data[min], _data[i]);
+      BSPF_swap(_data[min], _data[i]);
   }
 }
 
