@@ -8,53 +8,46 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FrameBuffer.cxx,v 1.85 2006/03/25 00:34:17 stephena Exp $
+// $Id: FrameBuffer.cxx,v 1.131 2008/05/21 21:01:40 stephena Exp $
 //============================================================================
 
 #include <sstream>
 
 #include "bspf.hxx"
-#include "Console.hxx"
-#include "Event.hxx"
-#include "EventHandler.hxx"
-#include "Settings.hxx"
-#include "MediaSrc.hxx"
-#include "FrameBuffer.hxx"
-#include "Font.hxx"
-#include "GuiUtils.hxx"
-#include "Menu.hxx"
+
 #include "CommandMenu.hxx"
+#include "Console.hxx"
+#include "EventHandler.hxx"
+#include "Event.hxx"
+#include "Font.hxx"
 #include "Launcher.hxx"
+#include "MediaSrc.hxx"
+#include "Menu.hxx"
 #include "OSystem.hxx"
+#include "Settings.hxx"
 
-#ifdef DEVELOPER_SUPPORT
+#include "FrameBuffer.hxx"
+
+#ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
 #endif
 
-#if defined(OS2) // FIXME - make proper OS/2 port
-  #define INCL_WIN
-  #include <os2emx.h>
-#endif
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FrameBuffer::FrameBuffer(OSystem* osystem)
   : myOSystem(osystem),
     myScreen(0),
-    theZoomLevel(2),
-    theMaxZoomLevel(2),
-    theAspectRatio(1.0),
     theRedrawTIAIndicator(true),
     myUsePhosphor(false),
     myPhosphorBlend(77),
-    myFrameRate(0)
+    myInitializedCount(0),
+    myPausedCount(0),
+    myFrameStatsEnabled(false)
 {
-  myBaseDim.x = myBaseDim.y = myBaseDim.w = myBaseDim.h = 0;
-  myImageDim = myScreenDim = myDesktopDim = myBaseDim;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -63,67 +56,41 @@ FrameBuffer::~FrameBuffer(void)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height,
-                             bool useAspect)
+void FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height)
 {
-  bool isAlreadyInitialized = (SDL_WasInit(SDL_INIT_VIDEO) & SDL_INIT_VIDEO) > 0;
+  // Now (re)initialize the SDL video system
+  // These things only have to be done one per FrameBuffer creation
+  if(SDL_WasInit(SDL_INIT_VIDEO) == 0)
+    if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0)
+      return;
 
+  myInitializedCount++;
+
+  myBaseDim.x = myBaseDim.y = 0;
   myBaseDim.w = (uInt16) width;
   myBaseDim.h = (uInt16) height;
-  myFrameRate = myOSystem->frameRate();
-
-  // Now (re)initialize the SDL video system
-  if(!isAlreadyInitialized)
-  {
-    Uint32 initflags = SDL_INIT_VIDEO | SDL_INIT_TIMER;
-
-    if(SDL_Init(initflags) < 0)
-      return;
-  }
-  setWindowIcon();
-  cls();
-
-  // Query the desktop size
-  // This is really the job of SDL
-  int dwidth = 0, dheight = 0;
-#if defined(OS2)  // FIXME - make proper OS/2 port
-  myDesktopDim.w = WinQuerySysValue(HWND_DESKTOP, SV_CXSCREEN);
-  myDesktopDim.h = WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);
-#else
-  myOSystem->getScreenDimensions(dwidth, dheight);
-  myDesktopDim.w = dwidth;  myDesktopDim.h = dheight;
-#endif
 
   // Set fullscreen flag
+#ifdef WINDOWED_SUPPORT
   mySDLFlags = myOSystem->settings().getBool("fullscreen") ? SDL_FULLSCREEN : 0;
+#else
+  mySDLFlags = 0;
+#endif
 
-  // Set window title
+  // Set the available video modes for this framebuffer
+  setAvailableVidModes();
+
+  // Set window title and icon
   setWindowTitle(title);
-
-  // Get the aspect ratio for the display if it's been enabled
-  theAspectRatio = 1.0;
-  if(useAspect)
-    setAspectRatio();
-
-  // Get the maximum size of a window for the current desktop
-  theMaxZoomLevel = maxWindowSizeForScreen();
-
-  // Check to see if window size will fit in the screen
-  if((uInt32)myOSystem->settings().getInt("zoom") > theMaxZoomLevel)
-    theZoomLevel = theMaxZoomLevel;
-  else
-    theZoomLevel = myOSystem->settings().getInt("zoom");
+  if(myInitializedCount == 1) setWindowIcon();
 
   // Initialize video subsystem
-  initSubsystem();
+  VideoMode mode = getSavedVidMode();
+  initSubsystem(mode);
 
   // And refresh the display
   myOSystem->eventHandler().refreshDisplay();
 
-  // Set palette for GUI
-  for(int i = 0; i < kNumColors-256; i++)
-    myDefPalette[i+256] = mapRGB(ourGUIColors[i][0], ourGUIColors[i][1], ourGUIColors[i][2]);
-
   // Enable unicode so we can see translated key events
   // (lowercase vs. uppercase characters)
   SDL_EnableUNICODE(1);
@@ -131,7 +98,10 @@ void FrameBuffer::initialize(const string& title, uInt32 width, uInt32 height,
   // Erase any messages from a previous run
   myMessage.counter = 0;
 
-  myUseDirtyRects = myOSystem->settings().getBool("dirtyrects");
+  // Finally, show some information about the framebuffer,
+  // but only on the first initialization
+  if(myInitializedCount == 1 && myOSystem->settings().getBool("showinfo"))
+    cout << about() << endl;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -147,26 +117,42 @@ void FrameBuffer::update()
   {
     case EventHandler::S_EMULATE:
     {
-      bool mediaSourceChanged = false;
+      // Run the console for one frame
+      myOSystem->console().mediaSource().update();
+      if(myOSystem->eventHandler().frying())
+        myOSystem->console().fry();
 
-      // Draw changes to the mediasource
-      if(!myOSystem->eventHandler().isPaused())
+      // And update the screen
+      drawMediaSource();
+
+      // Show frame statistics
+      if(myFrameStatsEnabled)
       {
-        myOSystem->console().mediaSource().update();
-        if(myOSystem->eventHandler().frying())
-          myOSystem->console().fry();
-          mediaSourceChanged = true;  // mediasource changed, so force an update
+        // FIXME - sizes hardcoded for now; fix during UI refactoring
+        char msg[30];
+        sprintf(msg, "%u LINES  %2.2f FPS",
+                myOSystem->console().mediaSource().scanlines(),
+                myOSystem->console().getFramerate());
+        fillRect(3, 3, 95, 9, kBGColor);
+        drawString(&myOSystem->font(), msg, 3, 3, 95, kBtnTextColor, kTextAlignCenter);
       }
 
+      break;  // S_EMULATE
+    }
+
+    case EventHandler::S_PAUSE:
+    {
       // Only update the screen if it's been invalidated
-      if(mediaSourceChanged || theRedrawTIAIndicator)
+      if(theRedrawTIAIndicator)
         drawMediaSource();
 
-      // Draw any pending messages
-      if(myMessage.counter > 0 && !myOSystem->eventHandler().isPaused())
-        drawMessage();
-
-      break;  // S_EMULATE
+      // Show a pause message every 5 seconds
+      if(myPausedCount++ >= 7*myOSystem->frameRate())
+      {
+        myPausedCount = 0;
+        showMessage("Paused", kMiddleCenter);
+      }
+      break;  // S_PAUSE
     }
 
     case EventHandler::S_MENU:
@@ -186,25 +172,16 @@ void FrameBuffer::update()
         drawMediaSource();
 
       myOSystem->commandMenu().draw();
-
-      // Draw any pending messages
-      if(myMessage.counter > 0 && !myOSystem->eventHandler().isPaused())
-        drawMessage();
       break;  // S_CMDMENU
     }
 
     case EventHandler::S_LAUNCHER:
     {
       myOSystem->launcher().draw();
-
-      // Draw any pending messages
-      if(myMessage.counter > 0)
-        drawMessage();
-
       break;  // S_LAUNCHER
     }
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
     case EventHandler::S_DEBUGGER:
     {
       myOSystem->debugger().draw();
@@ -217,6 +194,10 @@ void FrameBuffer::update()
       break;
   }
 
+  // Draw any pending messages
+  if(myMessage.counter > 0)
+    drawMessage();
+
   // Do any post-frame stuff
   postFrameUpdate();
 
@@ -237,7 +218,7 @@ void FrameBuffer::showMessage(const string& message, MessagePosition position,
 
   // Precompute the message coordinates
   myMessage.text    = message;
-  myMessage.counter = myFrameRate << 1;   // Show message for 2 seconds
+  myMessage.counter = uInt32(myOSystem->frameRate()) << 1; // Show message for 2 seconds
   myMessage.color   = color;
 
   myMessage.w = myOSystem->font().getStringWidth(myMessage.text) + 10;
@@ -293,11 +274,38 @@ void FrameBuffer::showMessage(const string& message, MessagePosition position,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBuffer::hideMessage()
+void FrameBuffer::toggleFrameStats()
 {
-  // Erase old messages on the screen
-  if(myMessage.counter > 0)
-    myOSystem->eventHandler().refreshDisplay();
+  showFrameStats(!myOSystem->settings().getBool("stats"));
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::showFrameStats(bool enable)
+{
+  myOSystem->settings().setBool("stats", enable);
+  myFrameStatsEnabled = enable;
+  myOSystem->eventHandler().refreshDisplay();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::enableMessages(bool enable)
+{
+  if(enable)
+  {
+    // Only re-anable frame stats if they were already enabled before
+    myFrameStatsEnabled = myOSystem->settings().getBool("stats");
+  }
+  else
+  {
+    // Temporarily disable frame stats
+    myFrameStatsEnabled = false;
+
+    // Erase old messages on the screen
+    myMessage.counter = 0;
+
+    myOSystem->eventHandler().refreshDisplay(true);  // Do this twice for
+    myOSystem->eventHandler().refreshDisplay(true);  // double-buffered modes
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -305,43 +313,36 @@ inline void FrameBuffer::drawMessage()
 {
   // Draw the bounded box and text
   fillRect(myMessage.x+1, myMessage.y+2, myMessage.w-2, myMessage.h-4, kBGColor);
-  box(myMessage.x, myMessage.y+1, myMessage.w, myMessage.h-2, kColor, kColor);
+  box(myMessage.x, myMessage.y+1, myMessage.w, myMessage.h-2, kColor, kShadowColor);
   drawString(&myOSystem->font(), myMessage.text, myMessage.x+1, myMessage.y+4,
              myMessage.w, myMessage.color, kTextAlignCenter);
   myMessage.counter--;
 
   // Either erase the entire message (when time is reached),
   // or show again this frame
-  if(myMessage.counter == 0)
-  {
-    // Force an immediate update
+  if(myMessage.counter == 0)  // Force an immediate update
     myOSystem->eventHandler().refreshDisplay(true);
-  }
   else
-	addDirtyRect(myMessage.x, myMessage.y, myMessage.w, myMessage.h);
+    addDirtyRect(myMessage.x, myMessage.y, myMessage.w, myMessage.h);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBuffer::pause(bool status)
+void FrameBuffer::refresh()
 {
-  if(&myOSystem->console())
-  {
-    enablePhosphor(myOSystem->console().properties().get(Display_Phosphor) == "YES");
-    setPalette(myOSystem->console().mediaSource().palette());
-  }
+  theRedrawTIAIndicator = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBuffer::setPalette(const uInt32* palette)
+void FrameBuffer::setTIAPalette(const uInt32* palette)
 {
   int i, j;
 
   // Set palette for normal fill
   for(i = 0; i < 256; ++i)
   {
-    Uint8 r = (Uint8) ((palette[i] & 0x00ff0000) >> 16);
-    Uint8 g = (Uint8) ((palette[i] & 0x0000ff00) >> 8);
-    Uint8 b = (Uint8) (palette[i] & 0x000000ff);
+    Uint8 r = (palette[i] >> 16) & 0xff;
+    Uint8 g = (palette[i] >> 8) & 0xff;
+    Uint8 b = palette[i] & 0xff;
 
     myDefPalette[i] = mapRGB(r, g, b);
   }
@@ -351,12 +352,12 @@ void FrameBuffer::setPalette(const uInt32* palette)
   {
     for(j = 0; j < 256; ++j)
     {
-      uInt8 ri = (uInt8) ((palette[i] & 0x00ff0000) >> 16);
-      uInt8 gi = (uInt8) ((palette[i] & 0x0000ff00) >> 8);
-      uInt8 bi = (uInt8) (palette[i] & 0x000000ff);
-      uInt8 rj = (uInt8) ((palette[j] & 0x00ff0000) >> 16);
-      uInt8 gj = (uInt8) ((palette[j] & 0x0000ff00) >> 8);
-      uInt8 bj = (uInt8) (palette[j] & 0x000000ff);
+      uInt8 ri = (palette[i] >> 16) & 0xff;
+      uInt8 gi = (palette[i] >> 8) & 0xff;
+      uInt8 bi = palette[i] & 0xff;
+      uInt8 rj = (palette[j] >> 16) & 0xff;
+      uInt8 gj = (palette[j] >> 8) & 0xff;
+      uInt8 bj = palette[j] & 0xff;
 
       Uint8 r = (Uint8) getPhosphor(ri, rj);
       Uint8 g = (Uint8) getPhosphor(gi, gj);
@@ -369,6 +370,19 @@ void FrameBuffer::setPalette(const uInt32* palette)
   theRedrawTIAIndicator = true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::setUIPalette(const uInt32* palette)
+{
+  // Set palette for GUI
+  for(int i = 0; i < kNumColors-256; ++i)
+  {
+    Uint8 r = (palette[i] >> 16) & 0xff;
+    Uint8 g = (palette[i] >> 8) & 0xff;
+    Uint8 b = palette[i] & 0xff;
+    myDefPalette[i+256] = mapRGB(r, g, b);
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::toggleFullscreen()
 {
@@ -378,63 +392,80 @@ void FrameBuffer::toggleFullscreen()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::setFullscreen(bool enable)
 {
+#ifdef WINDOWED_SUPPORT
   // Update the settings
   myOSystem->settings().setBool("fullscreen", enable);
-
   if(enable)
     mySDLFlags |= SDL_FULLSCREEN;
   else
     mySDLFlags &= ~SDL_FULLSCREEN;
 
-  if(!createScreen())
-    return;
+  // Do a dummy call to getSavedVidMode to set up the modelists
+  // and have it point to the correct 'current' mode
+  getSavedVidMode();
 
-  myOSystem->eventHandler().refreshDisplay();
-
-  setCursorState();
+  // Do a mode change to the 'current' mode by not passing a '+1' or '-1'
+  // to changeVidMode()
+  changeVidMode(0);
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void FrameBuffer::resize(int size, Int8 zoom)
+bool FrameBuffer::changeVidMode(int direction)
 {
-  switch(size)
+  bool saveModeChange = true;
+
+  VideoMode oldmode = myCurrentModeList->current();
+  if(direction == +1)
+    myCurrentModeList->next();
+  else if(direction == -1)
+    myCurrentModeList->previous();
+  else
+    saveModeChange = false;  // no resolution or zoom level actually changed
+
+  VideoMode newmode = myCurrentModeList->current();
+  if(!setVidMode(newmode))
+    return false;
+
+  myOSystem->eventHandler().handleResizeEvent();
+  myOSystem->eventHandler().refreshDisplay(true);
+  setCursorState();
+  showMessage(newmode.name);
+
+  if(saveModeChange)
   {
-    case -1:   // decrease size
-      if(myOSystem->settings().getBool("fullscreen"))
-        return;
-      if(theZoomLevel == 1)
-        theZoomLevel = theMaxZoomLevel;
-      else
-        theZoomLevel--;
-      break;
+    // Determine which mode we're in, and save to the appropriate setting
+    if(fullScreen())
+    {
+      myOSystem->settings().setSize("fullres", newmode.screen_w, newmode.screen_h);
+    }
+    else
+    {
+      EventHandler::State state = myOSystem->eventHandler().state();
+      bool inTIAMode = (state == EventHandler::S_EMULATE ||
+                        state == EventHandler::S_PAUSE   ||
+                        state == EventHandler::S_MENU    ||
+                        state == EventHandler::S_CMDMENU);
 
-    case +1:       // increase size
-      if(myOSystem->settings().getBool("fullscreen"))
-        return;
-      if(theZoomLevel == theMaxZoomLevel)
-        theZoomLevel = 1;
+      if(inTIAMode)
+        myOSystem->settings().setInt("zoom_tia", newmode.zoom);
       else
-        theZoomLevel++;
-      break;
-
-    case 0:      // use 'zoom' quantity
-      if(zoom < 1)
-        theZoomLevel = 1;
-      else if((uInt32)zoom > theMaxZoomLevel)
-        theZoomLevel = theMaxZoomLevel;
-      else
-        theZoomLevel = zoom;
-      break;
-
-    default:   // should never happen
-      return;
-      break;
+        myOSystem->settings().setInt("zoom_ui", newmode.zoom);
+    }
   }
-
-  if(!createScreen())
-    return;
-
-  myOSystem->eventHandler().refreshDisplay();
+  return true;
+/*
+cerr << "New mode:" << endl
+	<< "    screen w = " << newmode.screen_w << endl
+	<< "    screen h = " << newmode.screen_h << endl
+	<< "    image x  = " << newmode.image_x << endl
+	<< "    image y  = " << newmode.image_y << endl
+	<< "    image w  = " << newmode.image_w << endl
+	<< "    image h  = " << newmode.image_h << endl
+	<< "    zoom     = " << newmode.zoom << endl
+	<< "    name     = " << newmode.name << endl
+	<< endl;
+*/
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -450,9 +481,9 @@ void FrameBuffer::setCursorState()
   switch(myOSystem->eventHandler().state())
   {
     case EventHandler::S_EMULATE:
+    case EventHandler::S_PAUSE:
       showCursor(false);
       break;
-
     default:
       showCursor(true);
   }
@@ -461,55 +492,23 @@ void FrameBuffer::setCursorState()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::showCursor(bool show)
 {
-  if(show)
-    SDL_ShowCursor(SDL_ENABLE);
-  else
-    SDL_ShowCursor(SDL_DISABLE);
+  SDL_ShowCursor(show ? SDL_ENABLE : SDL_DISABLE);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void FrameBuffer::grabMouse(bool grab)
 {
-  if(grab)
-    SDL_WM_GrabInput(SDL_GRAB_ON);
-  else
-    SDL_WM_GrabInput(SDL_GRAB_OFF);
+  SDL_WM_GrabInput(grab ? SDL_GRAB_ON : SDL_GRAB_OFF);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool FrameBuffer::fullScreen()
+bool FrameBuffer::fullScreen() const
 {
-  return myOSystem->settings().getBool("fullscreen");
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 FrameBuffer::maxWindowSizeForScreen()
-{
-  uInt32 sWidth     = myDesktopDim.w;
-  uInt32 sHeight    = myDesktopDim.h;
-  uInt32 multiplier = 10;
-
-  // If screenwidth or height could not be found, use default zoom value
-  if(sWidth == 0 || sHeight == 0)
-    return 4;
-
-  bool found = false;
-  while(!found && (multiplier > 0))
-  {
-    // Figure out the desired size of the window
-    uInt32 width  = (uInt32) (myBaseDim.w * multiplier * theAspectRatio);
-    uInt32 height = myBaseDim.h * multiplier;
-
-    if((width < sWidth) && (height < sHeight))
-      found = true;
-    else
-      multiplier--;
-  }
-
-  if(found)
-    return multiplier;
-  else
-    return 1;
+#ifdef WINDOWED_SUPPORT
+    return myOSystem->settings().getBool("fullscreen");
+#else
+    return true;
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -710,22 +709,144 @@ void FrameBuffer::drawString(const GUI::Font* font, const string& s,
 uInt8 FrameBuffer::getPhosphor(uInt8 c1, uInt8 c2)
 {
   if(c2 > c1)
-    SWAP(c1, c2);
+    BSPF_swap(c1, c2);
 
   return ((c1 - c2) * myPhosphorBlend)/100 + c2;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8 FrameBuffer::ourGUIColors[kNumColors-256][3] = {
-  { 104, 104, 104 },  // kColor
-  {   0,   0,   0 },  // kBGColor
-  {  64,  64,  64 },  // kShadowColor
-  { 200, 200, 255 },  // kHiliteColor
-  {  32, 160,  32 },  // kTextColor
-#if !defined(GP2X)  // Quick GP2X hack to change colours, until an in-game GUI is added
-  {   0, 255,   0 },  // kTextColorHi
-#else
-  {   0,   0, 255 },  // kTextColorHi
+uInt32 FrameBuffer::maxWindowSizeForScreen(uInt32 screenWidth, uInt32 screenHeight)
+{
+  uInt32 multiplier = 1;
+  for(;;)
+  {
+    // Figure out the zoomed size of the window
+    uInt32 width  = myBaseDim.w * multiplier;
+    uInt32 height = myBaseDim.h * multiplier;
+
+    if((width > screenWidth) || (height > screenHeight))
+      break;
+
+    ++multiplier;
+  }
+  return multiplier > 1 ? multiplier - 1 : 1;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void FrameBuffer::setAvailableVidModes()
+{
+  // First we look at windowed modes
+  // These can be sized exactly as required, since there's normally no
+  // restriction on window size (up the maximum size)
+  myWindowedModeList.clear();
+  int max_zoom = maxWindowSizeForScreen(myOSystem->desktopWidth(),
+                                        myOSystem->desktopHeight());
+  for(int i = 1; i <= max_zoom; ++i)
+  {
+    VideoMode m;
+    m.image_x = m.image_y = 0;
+    m.image_w = m.screen_w = myBaseDim.w * i;
+    m.image_h = m.screen_h = myBaseDim.h * i;
+    m.zoom = i;
+    ostringstream buf;
+    buf << "Zoom " << i << "x";
+    m.name = buf.str();
+
+    myWindowedModeList.add(m);
+  }
+
+  // Now consider the fullscreen modes
+  // There are often stricter requirements on these, and they're normally
+  // different depending on the OSystem in use
+  // As well, we usually can't get fullscreen modes in the exact size
+  // we want, so we need to calculate image offsets
+  myFullscreenModeList.clear();
+  const ResolutionList& res = myOSystem->supportedResolutions();
+  for(unsigned int i = 0; i < res.size(); ++i)
+  {
+    VideoMode m;
+    m.screen_w = res[i].width;
+    m.screen_h = res[i].height;
+    m.zoom = maxWindowSizeForScreen(m.screen_w, m.screen_h);
+    m.name = res[i].name;
+
+    // Auto-calculate 'smart' centering; platform-specific framebuffers are
+    // free to ignore or augment it
+    m.image_w = myBaseDim.w * m.zoom;
+    m.image_h = myBaseDim.h * m.zoom;
+    m.image_x = (m.screen_w - m.image_w) / 2;
+    m.image_y = (m.screen_h - m.image_h) / 2;
+
+/*
+cerr << "Fullscreen modes:" << endl
+	<< "  Mode " << i << endl
+	<< "    screen w = " << m.screen_w << endl
+	<< "    screen h = " << m.screen_h << endl
+	<< "    image x  = " << m.image_x << endl
+	<< "    image y  = " << m.image_y << endl
+	<< "    image w  = " << m.image_w << endl
+	<< "    image h  = " << m.image_h << endl
+	<< "    zoom     = " << m.zoom << endl
+	<< endl;
+*/
+    myFullscreenModeList.add(m);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+VideoMode FrameBuffer::getSavedVidMode()
+{
+  EventHandler::State state = myOSystem->eventHandler().state();
+
+  if(myOSystem->settings().getBool("fullscreen"))
+  {
+    // Point the modelist to fullscreen modes, and set the iterator to
+    // the mode closest to the given resolution
+    int w = -1, h = -1;
+    myOSystem->settings().getSize("fullres", w, h);
+    if(w < 0 || h < 0)
+    {
+      w = myOSystem->desktopWidth();
+      h = myOSystem->desktopHeight();
+    }
+
+    // The launcher and debugger modes are different, in that their size is
+    // set at program launch and can't be changed
+    // In these cases, the resolution must accommodate their size
+    if(state == EventHandler::S_LAUNCHER)
+    {
+      int lw, lh;
+      myOSystem->settings().getSize("launcherres", lw, lh);
+      w = BSPF_max(w, lw);
+      h = BSPF_max(h, lh);
+    }
+#ifdef DEBUGGER_SUPPORT
+    else if(state == EventHandler::S_DEBUGGER)
+    {
+      int lw, lh;
+      myOSystem->settings().getSize("debuggerres", lw, lh);
+      w = BSPF_max(w, lw);
+      h = BSPF_max(h, lh);
+    }
 #endif
-  { 200,   0,   0 }   // kTextColorEm
-};
+
+    myCurrentModeList = &myFullscreenModeList;
+    myCurrentModeList->setByResolution(w, h);
+  }
+  else
+  {
+    // Point the modelist to windowed modes, and set the iterator to
+    // the mode closest to the given zoom level
+    bool inTIAMode = (state == EventHandler::S_EMULATE ||
+                      state == EventHandler::S_PAUSE   ||
+                      state == EventHandler::S_MENU    ||
+                      state == EventHandler::S_CMDMENU);
+    int zoom = (inTIAMode ? myOSystem->settings().getInt("zoom_tia") :
+                            myOSystem->settings().getInt("zoom_ui") );
+
+    myCurrentModeList = &myWindowedModeList;
+    myCurrentModeList->setByZoom(zoom);
+  }
+
+  return myCurrentModeList->current();
+}
