@@ -8,18 +8,19 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Props.cxx,v 1.12 2006/03/05 01:18:42 stephena Exp $
+// $Id: Props.cxx,v 1.22 2008/02/06 13:45:22 stephena Exp $
 //============================================================================
 
 #include <cctype>
 #include <algorithm>
+#include <sstream>
 
-#include "GuiUtils.hxx"
+#include "bspf.hxx"
 #include "Props.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -65,6 +66,7 @@ void Properties::set(PropertyType key, const string& value)
       case Console_SwapPorts:
       case Controller_Left:
       case Controller_Right:
+      case Controller_SwapPaddles:
       case Display_Format:
       case Display_Phosphor:
       case Emulation_HmoveBlanks:
@@ -74,6 +76,16 @@ void Properties::set(PropertyType key, const string& value)
         break;
       }
 
+      case Display_PPBlend:
+      {
+        int blend = atoi(myProperties[key].c_str());
+        if(blend < 0 || blend > 100) blend = 77;
+        ostringstream buf;
+        buf << blend;
+        myProperties[key] = buf.str();
+        break;
+      }
+
       default:
         break;
     }
@@ -124,9 +136,10 @@ void Properties::load(istream& in)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Properties::save(ostream& out)
+void Properties::save(ostream& out) const
 {
   // Write out each of the key and value pairs
+  bool changed = false;
   for(int i = 0; i < LastPropType; ++i)
   {
     // Try to save some space by only saving the items that differ from default
@@ -136,13 +149,17 @@ void Properties::save(ostream& out)
       out.put(' ');
       writeQuotedString(out, myProperties[i]);
       out.put('\n');
+      changed = true;
     }
   }
 
-  // Put a trailing null string so we know when to stop reading
-  writeQuotedString(out, "");
-  out.put('\n');
-  out.put('\n');
+  if(changed)
+  {
+    // Put a trailing null string so we know when to stop reading
+    writeQuotedString(out, "");
+    out.put('\n');
+    out.put('\n');
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -232,7 +249,7 @@ void Properties::copy(const Properties& properties)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Properties::print()
+void Properties::print() const
 {
   cout << get(Cartridge_MD5) << "|"
        << get(Cartridge_Name) << "|"
@@ -276,12 +293,12 @@ const char* Properties::ourDefaultProperties[LastPropType] = {
   "NO",          // Console.SwapPorts
   "JOYSTICK",    // Controller.Left
   "JOYSTICK",    // Controller.Right
-  "NTSC",        // Display.Format
-  "0",           // Display.XStart
-  "160",         // Display.Width
+  "NO",          // Controller.SwapPaddles
+  "AUTO-DETECT", // Display.Format
   "34",          // Display.YStart
   "210",         // Display.Height
   "NO",          // Display.Phosphor
+  "77",          // Display.PPBlend
   "YES"          // Emulation.HmoveBlanks
 };
 
@@ -301,11 +318,11 @@ const char* Properties::ourPropertyNames[LastPropType] = {
   "Console.SwapPorts",
   "Controller.Left",
   "Controller.Right",
+  "Controller.SwapPaddles",
   "Display.Format",
-  "Display.XStart",
-  "Display.Width",
   "Display.YStart",
   "Display.Height",
   "Display.Phosphor",
+  "Display.PPBlend",
   "Emulation.HmoveBlanks"
 };
