@@ -8,18 +8,23 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PropsSet.cxx,v 1.21 2006/03/19 00:46:04 stephena Exp $
+// $Id: PropsSet.cxx,v 1.37 2008/02/06 13:45:22 stephena Exp $
 //============================================================================
 
-#include "OSystem.hxx"
-#include "GuiUtils.hxx"
+#include <sstream>
+
+#include "bspf.hxx"
+
 #include "DefProps.hxx"
+#include "OSystem.hxx"
 #include "Props.hxx"
+#include "Settings.hxx"
+
 #include "PropsSet.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -28,6 +33,11 @@ PropertiesSet::PropertiesSet(OSystem* osystem)
     myRoot(NULL),
     mySize(0)
 {
+  const string& props = myOSystem->propertiesFile();
+  load(props, true);    // do save these properties
+
+  if(myOSystem->settings().getBool("showinfo"))
+    cout << "User game properties: \'" << props << "\'\n";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -37,47 +47,47 @@ PropertiesSet::~PropertiesSet()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::getMD5(const string& md5, Properties &properties)
+void PropertiesSet::getMD5(const string& md5, Properties& properties,
+                           bool useDefaults) const
 {
   properties.setDefaults();
   bool found = false;
 
   // First check our dynamic BST for the object
-  if(myRoot != 0)
+  if(!useDefaults && myRoot != 0)
   {
-    bool found = false;
     TreeNode* current = myRoot;
-
     while(current)
     {
-      string currentMd5 = current->props->get(Cartridge_MD5);
-
+      const string& currentMd5 = current->props->get(Cartridge_MD5);
       if(currentMd5 == md5)
       {
-        found = true;
+        // We only report a node as found if it's been marked as valid.
+        // Invalid nodes are those that should be removed, and are
+        // essentially treated as if they're not present.
+        found = current->valid;
         break;
       }
-      else
-      {
-        if(md5 < currentMd5)
-          current = current->left;
-        else 
-          current = current->right;
-      }
+      else if(md5 < currentMd5)
+        current = current->left;
+      else 
+        current = current->right;
     }
 
     if(found)
       properties = *(current->props);
   }
 
-  // Otherwise, search the internal BST array
+  // Otherwise, search the internal database using binary search
   if(!found)
   {
-    int i = 0;
-    while(i < ARRAYSIZE(DefProps))
+    int low = 0, high = DEF_PROPS_SIZE - 1;
+    while(low <= high)
     {
+      int i = (low + high) / 2;
       int cmp = strncmp(md5.c_str(), DefProps[i][Cartridge_MD5], 32);
-      if(cmp == 0)
+
+      if(cmp == 0)  // found it
       {
         for(int p = 0; p < LastPropType; ++p)
           if(DefProps[i][p][0] != 0)
@@ -87,28 +97,46 @@ void PropertiesSet::getMD5(const string& md5, Properties &properties)
         break;
       }
       else if(cmp < 0)
-        i = 2*i + 1;   // left child
+        high = i - 1; // look at lower range
       else
-        i = 2*i + 2;   // right child
+        low = i + 1;  // look at upper range
     }
   }
-
-  // Reset TIA positioning to defaults if option is enabled
-  if(myOSystem->settings().getBool("tiadefaults"))
-  {
-    properties.set(Display_XStart, Properties::ourDefaultProperties[Display_XStart]);
-    properties.set(Display_Width,  Properties::ourDefaultProperties[Display_Width]);
-    properties.set(Display_YStart, Properties::ourDefaultProperties[Display_YStart]);
-    properties.set(Display_Height, Properties::ourDefaultProperties[Display_Height]);
-  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PropertiesSet::insert(const Properties& properties, bool save)
 {
+  // Since the PropSet is keyed by md5, we can't insert without a valid one
+  if(properties.get(Cartridge_MD5) == "")
+    return;
+
   insertNode(myRoot, properties, save);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PropertiesSet::removeMD5(const string& md5)
+{
+  // We only remove from the dynamic BST
+  if(myRoot != 0)
+  {
+    TreeNode* current = myRoot;
+    while(current)
+    {
+      const string& currentMd5 = current->props->get(Cartridge_MD5);
+      if(currentMd5 == md5)
+      {
+        current->valid = false;  // Essentially, this node doesn't exist
+        break;
+      }
+      else if(md5 < currentMd5)
+        current = current->left;
+      else 
+        current = current->right;
+    }
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PropertiesSet::insertNode(TreeNode* &t, const Properties& properties,
                                bool save)
@@ -127,6 +155,7 @@ void PropertiesSet::insertNode(TreeNode* &t, const Properties& properties,
       delete t->props;
       t->props = new Properties(properties);
       t->save = save;
+      t->valid = true;
     }
   }
   else
@@ -136,6 +165,7 @@ void PropertiesSet::insertNode(TreeNode* &t, const Properties& properties,
     t->left = 0;
     t->right = 0;
     t->save = save;
+    t->valid = true;
 
     ++mySize;
   }
@@ -178,24 +208,30 @@ void PropertiesSet::load(const string& filename, bool save)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::save(ostream& out)
+bool PropertiesSet::save(const string& filename) const
 {
+  ofstream out(filename.c_str(), ios::out);
+  if(!out)
+    return false;
+
   saveNode(out, myRoot);
+  out.close();
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::print()
+void PropertiesSet::print() const
 {
   cout << size() << endl;
   printNode(myRoot);  // FIXME - print out internal properties as well
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::saveNode(ostream& out, TreeNode *node)
+void PropertiesSet::saveNode(ostream& out, TreeNode *node) const
 {
   if(node)
   {
-    if(node->save)
+    if(node->valid && node->save)
       node->props->save(out);
     saveNode(out, node->left);
     saveNode(out, node->right);
@@ -203,11 +239,12 @@ void PropertiesSet::saveNode(ostream& out, TreeNode *node)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::printNode(TreeNode *node)
+void PropertiesSet::printNode(TreeNode *node) const
 {
   if(node)
   {
-    node->props->print();
+    if(node->valid && node->save)
+      node->props->print();
     printNode(node->left);
     printNode(node->right);
   }
@@ -218,18 +255,3 @@ uInt32 PropertiesSet::size() const
 {
   return mySize;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool PropertiesSet::merge(const Properties& properties, const string& filename)
-{
-  ofstream out(filename.c_str());
-  if(out.is_open())
-  {
-    insert(properties, true);  // always save merged properties
-    save(out);
-    out.close();
-    return true;
-  }
-  else
-    return false;
-}
