@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Serializer.cxx,v 1.9 2005/12/29 21:16:28 stephena Exp $
+// $Id: Serializer.cxx,v 1.13 2008/02/06 13:45:22 stephena Exp $
 //============================================================================
 
 #include "Serializer.hxx"
@@ -51,6 +51,16 @@ bool Serializer::isOpen(void)
   return myStream.is_open();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Serializer::putByte(char value)
+{
+  char buf[1];
+  buf[0] = value;
+  myStream.write(buf, 1);
+  if(myStream.bad())
+    throw "Serializer: file write failed";
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Serializer::putInt(int value)
 {
@@ -77,5 +87,5 @@ void Serializer::putString(const string& str)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Serializer::putBool(bool b)
 {
-  putInt(b ? TruePattern: FalsePattern);
+  putByte(b ? TruePattern: FalsePattern);
 }
