@@ -8,21 +8,24 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Settings.cxx,v 1.85 2006/04/05 12:28:37 stephena Exp $
+// $Id: Settings.cxx,v 1.147 2008/05/21 16:49:07 stephena Exp $
 //============================================================================
 
 #include <cassert>
 #include <sstream>
 #include <fstream>
+#include <algorithm>
+
+#include "bspf.hxx"
 
 #include "OSystem.hxx"
 #include "Version.hxx"
-#include "bspf.hxx"
+
 #include "Settings.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -34,21 +37,27 @@ Settings::Settings(OSystem* osystem)
 
   // Add options that are common to all versions of Stella
   setInternal("video", "soft");
-  setInternal("dirtyrects", "true");
-  setInternal("ppblend", "77");
 
+  // OpenGL specific options
   setInternal("gl_filter", "nearest");
-  setInternal("gl_aspect", "2.0");
-  setInternal("gl_fsmax", "false");
-  setInternal("gl_lib", "");
+  setInternal("gl_aspect", "100");
+  setInternal("gl_fsmax", "never");
+  setInternal("gl_lib", "libGL.so");
+  setInternal("gl_vsync", "false");
+  setInternal("gl_texrect", "false");
 
-  setInternal("zoom", "2");
+  // Framebuffer-related options
+  setInternal("zoom_ui", "2");
+  setInternal("zoom_tia", "2");
   setInternal("fullscreen", "false");
+  setInternal("fullres", "");
   setInternal("center", "true");
   setInternal("grabmouse", "false");
   setInternal("palette", "standard");
-  setInternal("debugheight", "0");
+  setInternal("colorloss", "false");
+  setInternal("timing", "sleep");
 
+  // Sound options
   setInternal("sound", "true");
   setInternal("fragsize", "512");
   setInternal("freq", "31400");
@@ -56,32 +65,44 @@ Settings::Settings(OSystem* osystem)
   setInternal("volume", "100");
   setInternal("clipvol", "true");
 
+  // Input event options
   setInternal("keymap", "");
   setInternal("joymap", "");
   setInternal("joyaxismap", "");
   setInternal("joyhatmap", "");
-  setInternal("paddle", "0");
+  setInternal("joydeadzone", "0");
+  setInternal("pspeed", "6");
   setInternal("sa1", "left");
   setInternal("sa2", "right");
-  setInternal("joymouse", "false");
-  setInternal("p1speed", "50");
-  setInternal("p2speed", "50");
-  setInternal("p3speed", "50");
-  setInternal("p4speed", "50");
-  setInternal("pthresh", "600");
-
-  setInternal("showinfo", "false");
 
+  // Snapshot options
   setInternal("ssdir", "");
-  setInternal("ssname", "romname");
   setInternal("sssingle", "false");
 
+  // Config files and paths
   setInternal("romdir", "");
-  setInternal("rombrowse", "false");
-  setInternal("lastrom", "");
-  setInternal("modtime", "");  // romdir last modification time
+  setInternal("statedir", "");
+  setInternal("cheatfile", "");
+  setInternal("palettefile", "");
+  setInternal("propsfile", "");
 
-  setInternal("tiadefaults", "false");
+  // ROM browser options
+  setInternal("launcherres", "640x480");
+  setInternal("launcherfont", "small");
+  setInternal("romviewer", "false");
+  setInternal("lastrom", "");
+
+  // UI-related options
+  setInternal("debuggerres", "1030x690");
+  setInternal("uipalette", "0");
+  setInternal("mwheel", "4");
+
+  // Misc options
+  setInternal("autoslot", "false");
+  setInternal("showinfo", "false");
+  setInternal("tiafloat", "true");
+  setInternal("avoxport", "");
+  setInternal("stats", "false");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -137,66 +158,51 @@ void Settings::loadConfig()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Settings::loadCommandLine(int argc, char** argv)
+string Settings::loadCommandLine(int argc, char** argv)
 {
   for(int i = 1; i < argc; ++i)
   {
     // strip off the '-' character
     string key = argv[i];
-    if(key[0] != '-')
-      return true;     // stop processing here, ignore the remaining items
+    if(key[0] == '-')
+    {
+      key = key.substr(1, key.length());
 
-    key = key.substr(1, key.length());
+      // Take care of the arguments which are meant to be executed immediately
+      // (and then Stella should exit)
+      if(key == "help" || key == "listrominfo")
+      {
+        setExternal(key, "true");
+        return "";
+      }
 
-    // Take care of the arguments which are meant to be executed immediately
-    // (and then Stella should exit)
-    if(key == "help")
-    {
-      usage();
-      return false;
-    }
-    else if(key == "listrominfo")
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "debug") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "holdreset") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "holdselect") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
-    else if(key == "holdbutton0") // this doesn't make Stella exit
-    {
-      setExternal(key, "true");
-      return true;
-    }
+      // Take care of arguments without an option
+      if(key == "rominfo" || key == "debug" || key == "holdreset" ||
+         key == "holdselect" || key == "holdbutton0" || key == "takesnapshot")
+      {
+        setExternal(key, "true");
+        continue;
+      }
 
-    if(++i >= argc)
-    {
-      cerr << "Missing argument for '" << key << "'" << endl;
-      return false;
-    }
-    string value = argv[i];
+      if(++i >= argc)
+      {
+        cerr << "Missing argument for '" << key << "'" << endl;
+        return "";
+      }
+      string value = argv[i];
 
-    // Settings read from the commandline must not be saved to 
-    // the rc-file, unless they were previously set
-    if(int idx = getInternalPos(key) != -1)
-      setInternal(key, value, idx);   // don't set initialValue here
+      // Settings read from the commandline must not be saved to 
+      // the rc-file, unless they were previously set
+      if(int idx = getInternalPos(key) != -1)
+        setInternal(key, value, idx);   // don't set initialValue here
+      else
+        setExternal(key, value);
+    }
     else
-      setExternal(key, value);
+      return key;
   }
 
-  return true;
+  return "";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -209,25 +215,25 @@ void Settings::validate()
   if(s != "soft" && s != "gl")
     setInternal("video", "soft");
 
+  s = getString("timing");
+  if(s != "sleep" && s != "busy")
+    setInternal("timing", "sleep");
+
 #ifdef DISPLAY_OPENGL
   s = getString("gl_filter");
   if(s != "linear" && s != "nearest")
     setInternal("gl_filter", "nearest");
 
-  float f = getFloat("gl_aspect");
-  if(f < 1.1 || f > 2.0)
-    setInternal("gl_aspect", "2.0");
+  i = getInt("gl_aspect");
+  if(i < 50 || i > 100)
+    setInternal("gl_aspect", "100");
+
+  s = getString("gl_fsmax");
+  if(s != "never" && s != "ui" && s != "tia" && s != "always")
+    setInternal("gl_fsmax", "never");
 #endif
 
 #ifdef SOUND_SUPPORT
-  i = getInt("fragsize");
-  if(i != 256 && i != 512 && i != 1024 && i != 2048 && i != 4096)
-  #ifdef WIN32
-    setInternal("fragsize", "2048");
-  #else
-    setInternal("fragsize", "512");
-  #endif
-
   i = getInt("volume");
   if(i < 0 || i > 100)
     setInternal("volume", "100");
@@ -239,31 +245,33 @@ void Settings::validate()
     setInternal("tiafreq", "31400");
 #endif
 
-  i = getInt("zoom");
-  if(i < 1 || i > 6)
-    setInternal("zoom", "2");
+  i = getInt("zoom_ui");
+  if(i < 1 || i > 10)
+    setInternal("zoom_ui", "2");
 
-  i = getInt("paddle");
-  if(i < 0 || i > 3)
-    setInternal("paddle", "0");
+  i = getInt("zoom_tia");
+  if(i < 1 || i > 10)
+    setInternal("zoom_tia", "2");
 
-  i = getInt("pthresh");
-  if(i < 400)
-    setInternal("pthresh", "400");
-  else if(i > 800)
-    setInternal("pthresh", "800");
+  i = getInt("joydeadzone");
+  if(i < 0)
+    setInternal("joydeadzone", "0");
+  else if(i > 29)
+    setInternal("joydeadzone", "29");
+
+  i = getInt("pspeed");
+  if(i < 1)
+    setInternal("pspeed", "1");
+  else if(i > 15)
+    setInternal("pspeed", "15");
 
   s = getString("palette");
-  if(s != "standard" && s != "original" && s != "z26")
+  if(s != "standard" && s != "z26" && s != "user")
     setInternal("palette", "standard");
 
-  i = getInt("ppblend");
-  if(i < 0) setInternal("ppblend", "0");
-  if(i > 100) setInternal("ppblend", "100");
-
-  s = getString("ssname");
-  if(s != "romname" && s != "md5sum")
-    setInternal("ssname", "romname");
+  s = getString("launcherfont");
+  if(s != "small" && s != "large")
+    setInternal("launcherfont", "small");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -283,27 +291,32 @@ void Settings::usage()
   #ifdef DISPLAY_OPENGL
     << "                 gl              SDL OpenGL mode\n"
     << endl
+    << "  -gl_lib       <name>         Specify the OpenGL library\n"
     << "  -gl_filter    <type>         Type is one of the following:\n"
     << "                 nearest         Normal scaling (GL_NEAREST)\n"
     << "                 linear          Blurred scaling (GL_LINEAR)\n"
-    << "  -gl_aspect    <number>       Scale the width by the given amount\n"
-    << "  -gl_fsmax     <1|0>          Use the largest available screenmode in fullscreen OpenGL\n"
-    << "  -gl_lib       <filename>     Specify the OpenGL library\n"
+    << "  -gl_aspect    <number>       Scale the width by the given percentage\n"
+    << "  -gl_fsmax     <never|always| Stretch GL image in fullscreen mode\n"
+    << "                 ui|tia\n"
+    << "  -gl_vsync     <1|0>          Enable synchronize to vertical blank interrupt\n"
+    << "  -gl_texrect   <1|0>          Enable GL_TEXTURE_RECTANGLE extension\n"
     << endl
   #endif
-    << "  -zoom         <size>         Makes window be 'size' times normal\n"
+    << "  -zoom_tia     <zoom>         Use the specified zoom level in emulation mode\n"
+    << "  -zoom_ui      <zoom>         Use the specified zoom level in non-emulation mode (ROM browser/debugger)\n"
     << "  -fullscreen   <1|0>          Play the game in fullscreen mode\n"
+    << "  -fullres      <WxH>          The resolution to use in fullscreen mode\n"
     << "  -center       <1|0>          Centers game window (if possible)\n"
     << "  -grabmouse    <1|0>          Keeps the mouse in the game window\n"
-    << "  -palette      <original|     Use the specified color palette\n"
-    << "                 standard|\n"
-    << "                 z26>\n"
-    << "  -framerate    <number>       Display the given number of frames per second\n"
-    << "  -ppblend      <number>       Set blending for phosphor effect, if enabled (0-100)\n"
+    << "  -palette      <standard|     Use the specified color palette\n"
+    << "                 z26|\n"
+    << "                 user>\n"
+    << "  -colorloss    <1|0>          Enable PAL color-loss effect\n"
+    << "  -framerate    <number>       Display the given number of frames per second (0 to auto-calculate)\n"
+    << "  -timing       <sleep|busy>   Use the given type of wait between frames\n"
     << endl
   #ifdef SOUND_SUPPORT
     << "  -sound        <1|0>          Enable sound generation\n"
-    << "  -channels     <1|2>          Enable mono or stereo sound\n"
     << "  -fragsize     <number>       The size of sound fragments (must be a power of two)\n"
     << "  -freq         <number>       Set sound sample output frequency (0 - 48000)\n"
     << "  -tiafreq      <number>       Set sound sample generation frequency (0 - 48000)\n"
@@ -313,41 +326,44 @@ void Settings::usage()
   #endif
     << "  -cheat        <code>         Use the specified cheatcode (see manual for description)\n"
     << "  -showinfo     <1|0>          Shows some game info\n"
-    << "  -paddle       <0|1|2|3>      Indicates which paddle the mouse should emulate\n"
+    << "  -joydeadzone  <number>       Sets 'deadzone' area for analog joysticks (0-29)\n"
+    << "  -pspeed       <number>       Speed of digital emulated paddle movement (1-15)\n"
     << "  -sa1          <left|right>   Stelladaptor 1 emulates specified joystick port\n"
     << "  -sa2          <left|right>   Stelladaptor 2 emulates specified joystick port\n"
-    << "  -joymouse     <1|0>          Enable mouse emulation using joystick in GUI\n"
-    << "  -p1speed      <number>       Speed of emulated mouse movement for paddle 1 (0-100)\n"
-    << "  -p2speed      <number>       Speed of emulated mouse movement for paddle 2 (0-100)\n"
-    << "  -p3speed      <number>       Speed of emulated mouse movement for paddle 3 (0-100)\n"
-    << "  -p4speed      <number>       Speed of emulated mouse movement for paddle 4 (0-100)\n"
-    << "  -pthresh      <number>       Set threshold for eliminating paddle jitter\n"
-    << "  -tiadefaults  <1|0>          Use TIA positioning defaults instead of enhanced values\n"
-  #ifdef UNIX
-    << "  -accurate     <1|0>          Accurate game timing (uses more CPU)\n"
-  #endif
-  #ifdef SNAPSHOT_SUPPORT
+    << "  -romviewer    <1|0>          Show ROM info viewer in ROM launcher\n"
+    << "  -autoslot     <1|0>          Automatically switch to next save slot when state saving\n"
     << "  -ssdir        <path>         The directory to save snapshot files to\n"
-    << "  -ssname       <name>         How to name the snapshot (romname or md5sum)\n"
     << "  -sssingle     <1|0>          Generate single snapshot instead of many\n"
+    << "  -stats        <1|0>          Show scanline and framerate info during emulation\n"
     << endl
-  #endif
     << "  -listrominfo                 Display contents of stella.pro, one line per ROM entry\n"
+    << "  -rominfo      <rom>          Display detailed information for the given ROM\n"
+    << "  -launcherres  <WxH>          The resolution to use in ROM launcher mode\n"
+    << "  -launcherfont <small|large>  Use small or large font in the ROM launcher\n"
+    << "  -uipalette    <1|2>          Used the specified palette for UI elements\n"
+    << "  -mwheel       <lines>        Number of lines the mouse wheel will scroll in UI\n"
+    << "  -statedir     <dir>          Directory in which to save state files\n"
+    << "  -cheatfile    <file>         Full pathname of cheatfile database\n"
+    << "  -palettefile  <file>         Full pathname of user-defined palette file\n"
+    << "  -propsfile    <file>         Full pathname of ROM properties file\n"
+    << "  -tiafloat     <1|0>          Set unused TIA pins floating on a read/peek\n"
+    << "  -avoxport     <name>         The name of the serial port where an AtariVox is connected\n"
     << "  -help                        Show the text you're now reading\n"
-  #ifdef DEVELOPER_SUPPORT
+  #ifdef DEBUGGER_SUPPORT
     << endl
     << " The following options are meant for developers\n"
     << " Arguments are more fully explained in the manual\n"
     << endl
+    << "   -debuggerres  <WxH>         The resolution to use in debugger mode\n"
     << "   -break        <address>     Set a breakpoint at 'address'\n"
-    << "   -debugheight  <number>      Set height of debugger in lines of text (NOT pixels)\n"
     << "   -debug                      Start in debugger mode\n"
     << "   -holdreset                  Start the emulator with the Game Reset switch held down\n"
     << "   -holdselect                 Start the emulator with the Game Select switch held down\n"
     << "   -holdbutton0                Start the emulator with the left joystick button held down\n"
     << endl
-    << "   -pro         <props file>   Use the given properties file instead of stella.pro\n"
-    << "   -type        <arg>          Sets the 'Cartridge.Type' property\n"
+    << "   -bs          <arg>          Sets the 'Cartridge.Type' (bankswitch) property\n"
+    << "   -type        <arg>          Same as using -bs\n"
+    << "   -channels    <arg>          Sets the 'Cartridge.Sound' property\n"
     << "   -ld          <arg>          Sets the 'Console.LeftDifficulty' property\n"
     << "   -rd          <arg>          Sets the 'Console.RightDifficulty' property\n"
     << "   -tv          <arg>          Sets the 'Console.TelevisionType' property\n"
@@ -355,12 +371,12 @@ void Settings::usage()
     << "   -lc          <arg>          Sets the 'Controller.Left' property\n"
     << "   -rc          <arg>          Sets the 'Controller.Right' property\n"
     << "   -bc          <arg>          Same as using both -lc and -rc\n"
+    << "   -cp          <arg>          Sets the 'Controller.SwapPaddles' property\n"
     << "   -format      <arg>          Sets the 'Display.Format' property\n"
-    << "   -xstart      <arg>          Sets the 'Display.XStart' property\n"
     << "   -ystart      <arg>          Sets the 'Display.YStart' property\n"
-    << "   -width       <arg>          Sets the 'Display.Width' property\n"
     << "   -height      <arg>          Sets the 'Display.Height' property\n"
     << "   -pp          <arg>          Sets the 'Display.Phosphor' property\n"
+    << "   -ppblend     <arg>          Sets the 'Display.PPBlend' property\n"
     << "   -hmove       <arg>          Sets the 'Emulation.HmoveBlanks' property\n"
   #endif
     << endl;
@@ -462,6 +478,16 @@ void Settings::setString(const string& key, const string& value)
     setExternal(key, value);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Settings::getSize(const string& key, int& x, int& y) const
+{
+  string size = getString(key);
+  replace(size.begin(), size.end(), 'x', ' ');
+  istringstream buf(size);
+  buf >> x;
+  buf >> y;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Settings::getInt(const string& key) const
 {
@@ -472,7 +498,7 @@ int Settings::getInt(const string& key) const
   else if((idx = getExternalPos(key)) != -1)
     return (int) atoi(myExternalSettings[idx].value.c_str());
   else
-    return 0;
+    return -1;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -530,6 +556,14 @@ const string& Settings::getString(const string& key) const
     return EmptyString;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Settings::setSize(const string& key, const int value1, const int value2)
+{
+  ostringstream buf;
+  buf << value1 << "x" << value2;
+  setString(key, buf.str());
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int Settings::getInternalPos(const string& key) const
 {
