@@ -8,25 +8,37 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-1998 by Bradford W. Mott
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6502.cxx,v 1.2 2001/12/29 17:55:59 bwmott Exp $
+// $Id: M6502.cxx,v 1.23 2008/05/04 17:16:39 stephena Exp $
 //============================================================================
 
 #include "M6502.hxx"
 
+#ifdef DEBUGGER_SUPPORT
+  #include "Expression.hxx"
+#endif
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6502::M6502(uInt32 systemCyclesPerProcessorCycle)
-    : myExecutionStatus(0),
-      mySystem(0),
-      mySystemCyclesPerProcessorCycle(systemCyclesPerProcessorCycle)
+  : myExecutionStatus(0),
+    mySystem(0),
+    mySystemCyclesPerProcessorCycle(systemCyclesPerProcessorCycle),
+    myLastAccessWasRead(true),
+    myTotalInstructionCount(0)
 {
-  uInt16 t;
+#ifdef DEBUGGER_SUPPORT
+  myDebugger    = NULL;
+  myBreakPoints = NULL;
+  myReadTraps   = NULL;
+  myWriteTraps  = NULL;
+#endif
 
   // Compute the BCD lookup table
+  uInt16 t;
   for(t = 0; t < 256; ++t)
   {
     ourBCDTable[0][t] = ((t >> 4) * 10) + (t & 0x0f);
@@ -44,6 +56,10 @@ M6502::M6502(uInt32 systemCyclesPerProcessorCycle)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6502::~M6502()
 {
+#ifdef DEBUGGER_SUPPORT
+  myBreakConds.clear();
+  myBreakCondNames.clear();
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -64,8 +80,13 @@ void M6502::reset()
   SP = 0xff;
   PS(0x20);
 
+  // Reset access flag
+  myLastAccessWasRead = true;
+
   // Load PC from the reset vector
   PC = (uInt16)mySystem->peek(0xfffc) | ((uInt16)mySystem->peek(0xfffd) << 8);
+
+  myTotalInstructionCount = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -86,12 +107,6 @@ void M6502::stop()
   myExecutionStatus |= StopExecutionBit;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-M6502::AddressingMode M6502::addressingMode(uInt8 opcode) const
-{
-  return ourAddressingModeTable[opcode];
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 M6502::PS() const
 {
@@ -120,7 +135,7 @@ void M6502::PS(uInt8 ps)
 {
   N = ps & 0x80;
   V = ps & 0x40;
-  B = ps & 0x10;
+  B = true;        // B = ps & 0x10;  The 6507's B flag always true
   D = ps & 0x08;
   I = ps & 0x04;
   notZ = !(ps & 0x02);
@@ -180,86 +195,169 @@ uInt8 M6502::ourBCDTable[2][256];
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 M6502::AddressingMode M6502::ourAddressingModeTable[256] = {
-    Implied,    IndirectX, Invalid,   IndirectX,    // 0x0?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Implied,    IndirectX, Invalid,   IndirectX,    // 0x0?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0x1?
-    ZeroX,      ZeroX,     ZeroX,     ZeroX,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0x1?
+  ZeroX,      ZeroX,     ZeroX,     ZeroX,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
 
-    Absolute,   IndirectX, Invalid,   IndirectX,    // 0x2?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Absolute,   IndirectX, Invalid,   IndirectX,    // 0x2?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0x3?
-    ZeroX,      ZeroX,     ZeroX,     ZeroX,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0x3?
+  ZeroX,      ZeroX,     ZeroX,     ZeroX,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
 
-    Implied,    IndirectX, Invalid,   IndirectX,    // 0x4?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Implied,    IndirectX, Invalid,   IndirectX,    // 0x4?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0x5?
-    ZeroX,      ZeroX,     ZeroX,     ZeroX,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0x5?
+  ZeroX,      ZeroX,     ZeroX,     ZeroX,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
 
-    Implied,    IndirectX, Invalid,   IndirectX,    // 0x6?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Indirect,   Absolute,  Absolute,  Absolute,
+  Implied,    IndirectX, Invalid,   IndirectX,    // 0x6?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Indirect,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0x7?
-    ZeroX,      ZeroX,     ZeroX,     ZeroX,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0x7?
+  ZeroX,      ZeroX,     ZeroX,     ZeroX,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
 
-    Immediate,  IndirectX, Immediate, IndirectX,    // 0x8?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Immediate,  IndirectX, Immediate, IndirectX,    // 0x8?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0x9?
-    ZeroX,      ZeroX,     ZeroY,     ZeroY,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteY, AbsoluteY,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0x9?
+  ZeroX,      ZeroX,     ZeroY,     ZeroY,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteY, AbsoluteY,
 
-    Immediate,  IndirectX, Immediate, IndirectX,    // 0xA?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Immediate,  IndirectX, Immediate, IndirectX,    // 0xA?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0xB?
-    ZeroX,      ZeroX,     ZeroY,     ZeroY,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteY, AbsoluteY,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0xB?
+  ZeroX,      ZeroX,     ZeroY,     ZeroY,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteY, AbsoluteY,
 
-    Immediate,  IndirectX, Immediate, IndirectX,    // 0xC?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Immediate,  IndirectX, Immediate, IndirectX,    // 0xC?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0xD?
-    ZeroX,      ZeroX,     ZeroX,     ZeroX,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0xD?
+  ZeroX,      ZeroX,     ZeroX,     ZeroX,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX,
 
-    Immediate,  IndirectX, Immediate, IndirectX,    // 0xE?
-    Zero,       Zero,      Zero,      Zero,
-    Implied,    Immediate, Implied,   Immediate,
-    Absolute,   Absolute,  Absolute,  Absolute,
+  Immediate,  IndirectX, Immediate, IndirectX,    // 0xE?
+  Zero,       Zero,      Zero,      Zero,
+  Implied,    Immediate, Implied,   Immediate,
+  Absolute,   Absolute,  Absolute,  Absolute,
 
-    Relative,   IndirectY, Invalid,   IndirectY,    // 0xF?
-    ZeroX,      ZeroX,     ZeroX,     ZeroX,
-    Implied,    AbsoluteY, Implied,   AbsoluteY,
-    AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX
-  };
+  Relative,   IndirectY, Invalid,   IndirectY,    // 0xF?
+  ZeroX,      ZeroX,     ZeroX,     ZeroX,
+  Implied,    AbsoluteY, Implied,   AbsoluteY,
+  AbsoluteX,  AbsoluteX, AbsoluteX, AbsoluteX
+};
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+M6502::AccessMode M6502::ourAccessModeTable[256] = {
+  None,   Read,   None,   Write,    // 0x0?
+  None,   Read,   Write,  Write,
+  None,   Read,   Write,  Read,
+  None,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0x1?
+  None,   Read,   Write,  Write,
+  None,   Read,   None,   Write,
+  None,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0x2?
+  Read,   Read,   Write,  Write,
+  None,   Read,   Write,  Read,
+  Read,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0x3?
+  None,   Read,   Write,  Write,
+  None,   Read,   None,   Write,
+  None,   Read,   Write,  Write,
+  
+  None,   Read,   None,   Write,    // 0x4?
+  None,   Read,   Write,  Write,
+  None,   Read,   Write,  Read,
+  Read,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0x5?
+  None,   Read,   Write,  Write,
+  None,   Read,   None,   Write,
+  None,   Read,   Write,  Write,
+
+  None,   Read,   None,   Write,    // 0x6?
+  None,   Read,   Write,  Write,
+  None,   Read,   Write,  Read,
+  Read,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0x7?
+  None,   Read,   Write,  Write,
+  None,   Read,   None,   Write,
+  None,   Read,   Write,  Write,
+
+  None,   Write,  None,   Write,    // 0x8?
+  Write,  Write,  Write,  Write,
+  None,   None,   None,   Read,
+  Write,  Write,  Write,  Write,
+
+  Read,   Write,  None,   Write,    // 0x9?
+  Write,  Write,  Write,  Write,
+  None,   Write,  None,   Write,
+  Write,  Write,  Write,  Write,
+
+  Read,   Read,   Read,   Read,     // 0xA?
+  Read,   Read,   Read,   Read,
+  None,   Read,   None,   Read,
+  Read,   Read,   Read,   Read,
+
+  Read,   Read,   None,   Read,     // 0xB?
+  Read,   Read,   Read,   Read,
+  None,   Read,   None,   Read,
+  Read,   Read,   Read,   Read,
+
+  Read,   Read,   None,   Write,    // 0xC?
+  Read,   Read,   Write,  Write,
+  None,   Read,   None,   Read,
+  Read,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0xD?
+  None,   Read,   Write,  Write,
+  None,   Read,   None,   Write,
+  None,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0xE?
+  Read,   Read,   Write,  Write,
+  None,   Read,   None,   Read,
+  Read,   Read,   Write,  Write,
+
+  Read,   Read,   None,   Write,    // 0xF?
+  None,   Read,   Write,  Write,
+  None,   Read,   None,   Write,
+  None,   Read,   Write,  Write
+};
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt32 M6502::ourInstructionProcessorCycleTable[256] = {
@@ -333,3 +431,68 @@ const char* M6502::ourInstructionMnemonicTable[256] = {
   "SED",  "SBC",  "nop",  "isb",  "nop",  "SBC",  "INC",  "isb"
 };
 
+#ifdef DEBUGGER_SUPPORT
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6502::attach(Debugger& debugger)
+{
+  // Remember the debugger for this microprocessor
+  myDebugger = &debugger;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+unsigned int M6502::addCondBreak(Expression *e, const string& name)
+{
+  myBreakConds.push_back(e);
+  myBreakCondNames.push_back(name);
+  return myBreakConds.size() - 1;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6502::delCondBreak(unsigned int brk)
+{
+  if(brk < myBreakConds.size())
+  {
+    delete myBreakConds[brk];
+    myBreakConds.remove_at(brk);
+    myBreakCondNames.remove_at(brk);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6502::clearCondBreaks()
+{
+  for(unsigned int i=0; i<myBreakConds.size(); i++)
+    delete myBreakConds[i];
+  myBreakConds.clear();
+  myBreakCondNames.clear();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const StringList& M6502::getCondBreakNames() const
+{
+  return myBreakCondNames;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int M6502::evalCondBreaks()
+{
+  for(unsigned int i=0; i<myBreakConds.size(); i++)
+    if(myBreakConds[i]->evaluate())
+      return i;
+
+  return -1; // no break hit
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6502::setBreakPoints(PackedBitArray *bp)
+{
+  myBreakPoints = bp;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void M6502::setTraps(PackedBitArray *read, PackedBitArray *write)
+{
+  myReadTraps = read;
+  myWriteTraps = write;
+}
+#endif
