@@ -8,19 +8,19 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6502Hi.cxx,v 1.15 2005/12/17 01:23:07 stephena Exp $
+// $Id: M6502Hi.cxx,v 1.22 2008/04/11 01:28:35 stephena Exp $
 //============================================================================
 
 #include "M6502Hi.hxx"
 #include "Serializer.hxx"
 #include "Deserializer.hxx"
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
 #endif
 
@@ -33,7 +33,7 @@ M6502High::M6502High(uInt32 systemCyclesPerProcessorCycle)
   myNumberOfDistinctAccesses = 0;
   myLastAddress = 0;
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   myJustHitTrapFlag = false;
 #endif
 }
@@ -53,7 +53,7 @@ inline uInt8 M6502High::peek(uInt16 address)
   }
   mySystem->incrementCycles(mySystemCyclesPerProcessorCycle);
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   if(myReadTraps != NULL && myReadTraps->isSet(address))
   {
     myJustHitTrapFlag = true;
@@ -62,7 +62,9 @@ inline uInt8 M6502High::peek(uInt16 address)
   }
 #endif
 
-  return mySystem->peek(address);
+  uInt8 result = mySystem->peek(address);
+  myLastAccessWasRead = true;
+  return result;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -75,7 +77,7 @@ inline void M6502High::poke(uInt16 address, uInt8 value)
   }
   mySystem->incrementCycles(mySystemCyclesPerProcessorCycle);
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   if(myWriteTraps != NULL && myWriteTraps->isSet(address))
   {
     myJustHitTrapFlag = true;
@@ -85,6 +87,7 @@ inline void M6502High::poke(uInt16 address, uInt8 value)
 #endif
 
   mySystem->poke(address, value);
+  myLastAccessWasRead = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -101,7 +104,7 @@ bool M6502High::execute(uInt32 number)
       uInt16 operandAddress = 0;
       uInt8 operand = 0;
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
       if(myJustHitTrapFlag)
       {
         if(myDebugger->start(myHitTrapInfo.message, myHitTrapInfo.address))
@@ -236,12 +239,12 @@ bool M6502High::save(Serializer& out)
   {
     out.putString(CPU);
 
-    out.putInt(A);    // Accumulator
-    out.putInt(X);    // X index register
-    out.putInt(Y);    // Y index register
-    out.putInt(SP);   // Stack Pointer
-    out.putInt(IR);   // Instruction register
-    out.putInt(PC);   // Program Counter
+    out.putByte((char)A);   // Accumulator
+    out.putByte((char)X);   // X index register
+    out.putByte((char)Y);   // Y index register
+    out.putByte((char)SP);  // Stack Pointer
+    out.putByte((char)IR);  // Instruction register
+    out.putInt(PC);         // Program Counter
 
     out.putBool(N);     // N flag for processor status register
     out.putBool(V);     // V flag for processor status register
@@ -251,7 +254,7 @@ bool M6502High::save(Serializer& out)
     out.putBool(notZ);  // Z flag complement for processor status register
     out.putBool(C);     // C flag for processor status register
 
-    out.putInt(myExecutionStatus);
+    out.putByte((char)myExecutionStatus);
 
     // Indicates the number of distinct memory accesses
     out.putInt(myNumberOfDistinctAccesses);
@@ -283,11 +286,11 @@ bool M6502High::load(Deserializer& in)
     if(in.getString() != CPU)
       return false;
 
-    A = (uInt8) in.getInt();    // Accumulator
-    X = (uInt8) in.getInt();    // X index register
-    Y = (uInt8) in.getInt();    // Y index register
-    SP = (uInt8) in.getInt();   // Stack Pointer
-    IR = (uInt8) in.getInt();   // Instruction register
+    A = (uInt8) in.getByte();    // Accumulator
+    X = (uInt8) in.getByte();    // X index register
+    Y = (uInt8) in.getByte();    // Y index register
+    SP = (uInt8) in.getByte();   // Stack Pointer
+    IR = (uInt8) in.getByte();   // Instruction register
     PC = (uInt16) in.getInt();  // Program Counter
 
     N = in.getBool();     // N flag for processor status register
@@ -298,7 +301,7 @@ bool M6502High::load(Deserializer& in)
     notZ = in.getBool();  // Z flag complement for processor status register
     C = in.getBool();     // C flag for processor status register
 
-    myExecutionStatus = (uInt8) in.getInt();
+    myExecutionStatus = (uInt8) in.getByte();
 
     // Indicates the number of distinct memory accesses
     myNumberOfDistinctAccesses = (uInt32) in.getInt();
