@@ -8,19 +8,19 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: M6502Low.cxx,v 1.8 2006/02/05 02:49:47 stephena Exp $
+// $Id: M6502Low.cxx,v 1.15 2008/05/04 17:16:39 stephena Exp $
 //============================================================================
 
 #include "M6502Low.hxx"
 #include "Serializer.hxx"
 #include "Deserializer.hxx"
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   #include "Debugger.hxx"
 #endif
 
@@ -30,7 +30,7 @@
 M6502Low::M6502Low(uInt32 systemCyclesPerProcessorCycle)
     : M6502(systemCyclesPerProcessorCycle)
 {
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   myJustHitTrapFlag = false;
 #endif
 }
@@ -43,7 +43,7 @@ M6502Low::~M6502Low()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 inline uInt8 M6502Low::peek(uInt16 address)
 {
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   if(myReadTraps != NULL && myReadTraps->isSet(address))
   {
     myJustHitTrapFlag = true;
@@ -52,13 +52,15 @@ inline uInt8 M6502Low::peek(uInt16 address)
   }
 #endif
 
-  return mySystem->peek(address);
+  uInt8 result = mySystem->peek(address);
+  myLastAccessWasRead = true;
+  return result;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 inline void M6502Low::poke(uInt16 address, uInt8 value)
 {
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
   if(myWriteTraps != NULL && myWriteTraps->isSet(address))
   {
     myJustHitTrapFlag = true;
@@ -68,6 +70,7 @@ inline void M6502Low::poke(uInt16 address, uInt8 value)
 #endif
 
   mySystem->poke(address, value);
+  myLastAccessWasRead = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -84,7 +87,7 @@ bool M6502Low::execute(uInt32 number)
       uInt16 operandAddress = 0;
       uInt8 operand = 0;
 
-#ifdef DEVELOPER_SUPPORT
+#ifdef DEBUGGER_SUPPORT
       if(myJustHitTrapFlag)
       {
         if(myDebugger->start(myHitTrapInfo.message, myHitTrapInfo.address))
@@ -139,6 +142,8 @@ bool M6502Low::execute(uInt32 number)
           cerr << "Illegal Instruction! " << hex << (int) IR << endl;
       }
 
+      myTotalInstructionCount++;
+
 #ifdef DEBUG
       debugStream << hex << setw(4) << operandAddress << " ";
       debugStream << setw(4) << ourInstructionMnemonicTable[IR];
@@ -221,12 +226,12 @@ bool M6502Low::save(Serializer& out)
   {
     out.putString(CPU);
 
-    out.putInt(A);    // Accumulator
-    out.putInt(X);    // X index register
-    out.putInt(Y);    // Y index register
-    out.putInt(SP);   // Stack Pointer
-    out.putInt(IR);   // Instruction register
-    out.putInt(PC);   // Program Counter
+    out.putByte((char)A);  // Accumulator
+    out.putByte((char)X);  // X index register
+    out.putByte((char)Y);  // Y index register
+    out.putByte((char)SP); // Stack Pointer
+    out.putByte((char)IR); // Instruction register
+    out.putInt(PC);        // Program Counter
 
     out.putBool(N);     // N flag for processor status register
     out.putBool(V);     // V flag for processor status register
@@ -236,7 +241,7 @@ bool M6502Low::save(Serializer& out)
     out.putBool(notZ);  // Z flag complement for processor status register
     out.putBool(C);     // C flag for processor status register
 
-    out.putInt(myExecutionStatus);
+    out.putByte((char)myExecutionStatus);
   }
   catch(char *msg)
   {
@@ -262,11 +267,11 @@ bool M6502Low::load(Deserializer& in)
     if(in.getString() != CPU)
       return false;
 
-    A = (uInt8) in.getInt();    // Accumulator
-    X = (uInt8) in.getInt();    // X index register
-    Y = (uInt8) in.getInt();    // Y index register
-    SP = (uInt8) in.getInt();   // Stack Pointer
-    IR = (uInt8) in.getInt();   // Instruction register
+    A = (uInt8) in.getByte();   // Accumulator
+    X = (uInt8) in.getByte();   // X index register
+    Y = (uInt8) in.getByte();   // Y index register
+    SP = (uInt8) in.getByte();  // Stack Pointer
+    IR = (uInt8) in.getByte();  // Instruction register
     PC = (uInt16) in.getInt();  // Program Counter
 
     N = in.getBool();     // N flag for processor status register
@@ -277,7 +282,7 @@ bool M6502Low::load(Deserializer& in)
     notZ = in.getBool();  // Z flag complement for processor status register
     C = in.getBool();     // C flag for processor status register
 
-    myExecutionStatus = (uInt8) in.getInt();
+    myExecutionStatus = (uInt8) in.getByte();
   }
   catch(char *msg)
   {
