@@ -8,12 +8,12 @@
 // MM     MM 66  66 55  55 00  00 22
 // MM     MM  6666   5555   0000  222222
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: NullDev.cxx,v 1.3 2005/06/16 01:11:29 stephena Exp $
+// $Id: NullDev.cxx,v 1.7 2008/02/06 13:45:22 stephena Exp $
 //============================================================================
 
 #include "NullDev.hxx"
@@ -30,12 +30,6 @@ NullDevice::~NullDevice()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* NullDevice::name() const
-{
-  return "NULL";
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void NullDevice::reset()
 {
@@ -61,7 +55,7 @@ void NullDevice::poke(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool NullDevice::save(Serializer& out)
+bool NullDevice::save(Serializer& out) const
 {
   return true;
 }
