@@ -8,27 +8,28 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: BrowserDialog.cxx,v 1.18 2006/03/20 13:23:13 stephena Exp $
+// $Id: BrowserDialog.cxx,v 1.30 2008/03/23 16:22:46 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "Widget.hxx"
-#include "StringListWidget.hxx"
+#include "bspf.hxx"
+
 #include "Dialog.hxx"
 #include "FSNode.hxx"
+#include "GameList.hxx"
 #include "GuiObject.hxx"
-#include "GuiUtils.hxx"
-#include "BrowserDialog.hxx"
+#include "OSystem.hxx"
+#include "StringListWidget.hxx"
+#include "Widget.hxx"
 
-#include "bspf.hxx"
+#include "BrowserDialog.hxx"
 
 /* We want to use this as a general directory selector at some point... possible uses
  * - to select the data dir for a game
@@ -43,12 +44,20 @@ BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
   : Dialog(boss->instance(), boss->parent(), x, y, w, h),
     CommandSender(boss),
     _fileList(NULL),
-    _currentPath(NULL)
+    _currentPath(NULL),
+    _nodeList(NULL),
+    _mode(AbstractFilesystemNode::kListDirectoriesOnly)
 {
-  const int lineHeight = font.getLineHeight(),
-            bwidth     = font.getStringWidth("Cancel") + 20,
-            bheight    = font.getLineHeight() + 4;
+  const int lineHeight   = font.getLineHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 20,
+            buttonHeight = font.getLineHeight() + 4;
   int xpos, ypos;
+  ButtonWidget* b;
+
+  // Set real dimensions
+  // This is one dialog that can take as much space as is available
+//  _w = _DLG_MIN_SWIDTH - 30;
+//  _h = _DLG_MIN_SHEIGHT - 30;
 
   xpos = 10;  ypos = 4;
   _title = new StaticTextWidget(this, font, xpos, ypos,
@@ -59,54 +68,123 @@ BrowserDialog::BrowserDialog(GuiObject* boss, const GUI::Font& font,
   ypos += lineHeight + 4;
   _currentPath = new StaticTextWidget(this, font, xpos, ypos,
                                        _w - 2 * xpos, lineHeight,
-                                      "DUMMY", kTextAlignLeft);
+                                      "", kTextAlignLeft);
 
   // Add file list
   ypos += lineHeight;
   _fileList = new StringListWidget(this, font, xpos, ypos,
-                                   _w - 2 * xpos, _h - bheight - ypos - 15);
+                                   _w - 2 * xpos, _h - buttonHeight - ypos - 20);
   _fileList->setNumberingMode(kListNumberingOff);
   _fileList->setEditable(false);
-  _fileList->setFlags(WIDGET_NODRAW_FOCUS);
-  addFocusWidget(_fileList);
 
   // Buttons
-  xpos = 10;  ypos = _h - bheight - 8;
-  _goUpButton = new ButtonWidget(this, font, xpos, ypos, bwidth, bheight,
-                                 "Go up", kGoUpCmd, 0);
+  _goUpButton = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
+                                 buttonWidth, buttonHeight, "Go up", kGoUpCmd);
+  addFocusWidget(_goUpButton);
+
 #ifndef MAC_OSX
-  xpos = _w - 2 *(bwidth + 10);  
-  new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Choose",
-                   kChooseCmd, 0);
-  xpos += bwidth + 10;
-  new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Cancel",
-                   kCloseCmd, 0);
+  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Choose", kChooseCmd);
+  addFocusWidget(b);
+  addOKWidget(b);
+  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
+  addFocusWidget(b);
+  addCancelWidget(b);
 #else
-  xpos = _w - 2 *(bwidth + 10);  ypos = _h - bheight - 8;
-  new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Cancel",
-                   kCloseCmd, 0);
-  xpos += bwidth + 10;
-  new ButtonWidget(this, font, xpos, ypos, bwidth, bheight, "Choose",
-                   kChooseCmd, 0);
+  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
+  addFocusWidget(b);
+  addCancelWidget(b);
+  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Choose", kChooseCmd);
+  addFocusWidget(b);
+  addOKWidget(b);
 #endif
+
+  addFocusWidget(_fileList);
+
+  // Create a list of directory nodes
+  _nodeList = new GameList();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void BrowserDialog::setStartPath(const string& startpath)
+BrowserDialog::~BrowserDialog()
 {
+  delete _nodeList;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void BrowserDialog::setStartPath(const string& startpath,
+                                 FilesystemNode::ListMode mode)
+{
+  _mode = mode;
+
   // If no node has been set, or the last used one is now invalid,
   // go back to the root/default dir.
-  _choice = FilesystemNode(startpath);
+  _node = FilesystemNode(startpath);
 
-  if (_choice.isValid())
-    _node = _choice;
-  else if (!_node.isValid())
+  if(!_node.isValid())
     _node = FilesystemNode();
 
+  // Generally, we always want a directory listing 
+  if(!_node.isDirectory() && _node.hasParent())
+    _node = _node.getParent();
+
   // Alway refresh file list
   updateListing();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void BrowserDialog::updateListing()
+{
+  if(!_node.isDirectory())
+    return;
+
+  // Start with empty list
+  _nodeList->clear();
+
+  // Update the path display
+  _currentPath->setLabel(_node.path());
+
+  // Read in the data from the file system
+  FSList content = _node.listDir(_mode);
+
+  // Add '[..]' to indicate previous folder
+  if(_node.hasParent())
+  {
+    const string& parent = _node.getParent().path();
+    _nodeList->appendGame(" [..]", parent, "", true);
+  }
+
+  // Now add the directory entries
+  for(unsigned int idx = 0; idx < content.size(); idx++)
+  {
+    string name = content[idx].displayName();
+    bool isDir = content[idx].isDirectory();
+    if(isDir)
+      name = " [" + name + "]";
+
+    _nodeList->appendGame(name, content[idx].path(), "", isDir);
+  }
+  _nodeList->sortByName();
+
+  // Now fill the list widget with the contents of the GameList
+  StringList l;
+  for (int i = 0; i < (int) _nodeList->size(); ++i)
+    l.push_back(_nodeList->name(i));
+
+  _fileList->setList(l);
+  if(_fileList->getList().size() > 0)
+    _fileList->setSelected(0);
+
+  // Only hilite the 'up' button if there's a parent directory
+  _goUpButton->setEnabled(_node.hasParent());
+
+  // Finally, redraw
+  setDirty(); draw();
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void BrowserDialog::handleCommand(CommandSender* sender, int cmd,
                                   int data, int id)
@@ -114,23 +192,11 @@ void BrowserDialog::handleCommand(CommandSender* sender, int cmd,
   switch (cmd)
   {
     case kChooseCmd:
-    {
-      // If nothing is selected in the list widget, choose the current dir.
-      // Else, choose the dir that is selected.
-      int selection = _fileList->getSelected();
-      if (selection >= 0 && selection < (int)_nodeContent.size())
-        _choice = _nodeContent[selection];
-      else
-        _choice = _node;
-
       // Send a signal to the calling class that a selection has been made
       // Since we aren't derived from a widget, we don't have a 'data' or 'id'
-      if(_cmd)
-        sendCommand(_cmd, 0, 0);
-
+      if(_cmd) sendCommand(_cmd, 0, 0);
       close();
       break;
-    }
 
     case kGoUpCmd:
       _node = _node.getParent();
@@ -139,44 +205,18 @@ void BrowserDialog::handleCommand(CommandSender* sender, int cmd,
 
     case kListItemActivatedCmd:
     case kListItemDoubleClickedCmd:
-      _node = _nodeContent[data];
-      updateListing();
+    {
+      int item = _fileList->getSelected();
+      if(item >= 0)
+      {
+        _node = _nodeList->path(item);
+        updateListing();
+      }
       break;
+    }
 
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
       break;
   }
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void BrowserDialog::updateListing()
-{
-  // Update the path display
-  _currentPath->setLabel(_node.path());
-
-  // Read in the data from the file system
-  _nodeContent = _node.listDir();
-  _nodeContent.sort();
-
-  // Populate the ListWidget
-  StringList list;
-  int size = _nodeContent.size();
-  for (int i = 0; i < size; i++)
-  {
-    if(_nodeContent[i].isDirectory())
-      list.push_back(" [" + _nodeContent[i].displayName() + "]");
-    else
-      list.push_back(_nodeContent[i].displayName());
-  }
-
-  _fileList->setList(list);
-  if(size > 0)
-    _fileList->setSelected(0);
-
-  // Only hilite the 'up' button if there's a parent directory
-  _goUpButton->setEnabled(_node.hasParent());
-
-  // Finally, redraw
-  setDirty(); draw();
-}
