@@ -8,19 +8,17 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Dialog.cxx,v 1.43 2006/03/02 13:10:53 stephena Exp $
+// $Id: Dialog.cxx,v 1.60 2008/03/25 13:11:34 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include <SDL.h>
-
 #include "OSystem.hxx"
 #include "FrameBuffer.hxx"
 #include "Menu.hxx"
@@ -43,7 +41,10 @@ Dialog::Dialog(OSystem* instance, DialogContainer* parent,
     _mouseWidget(0),
     _focusedWidget(0),
     _dragWidget(0),
+    _okWidget(0),
+    _cancelWidget(0),
     _visible(true),
+    _center(true),
     _ourTab(NULL),
     _focusID(0)
 {
@@ -57,6 +58,8 @@ Dialog::~Dialog()
 
   delete _firstWidget;
   _firstWidget = NULL;
+
+  _ourButtonGroup.clear();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -85,6 +88,17 @@ void Dialog::close()
   parent()->removeDialog();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Dialog::center()
+{
+  FrameBuffer& fb = instance()->frameBuffer();
+  if(_center && &fb)
+  {
+    _x = (fb.baseWidth() - _w)  / 2;
+    _y = (fb.baseHeight() - _h) / 2;
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::releaseFocus()
 {
@@ -98,6 +112,10 @@ void Dialog::releaseFocus()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::addFocusWidget(Widget* w)
 {
+  // All focusable widgets should retain focus
+  if(w)
+    w->setFlags(WIDGET_RETAIN_FOCUS);
+
   if(_ourFocusList.size() == 0)
   {
 	Focus f;
@@ -111,6 +129,10 @@ void Dialog::addFocusWidget(Widget* w)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::addToFocusList(WidgetArray& list, int id)
 {
+  // All focusable widgets should retain focus
+  for(unsigned int i = 0; i < list.size(); ++i)
+    list[i]->setFlags(WIDGET_RETAIN_FOCUS);
+
   id++;  // Arrays start at 0, not -1.
 
   // Make sure the array is large enough
@@ -125,7 +147,7 @@ void Dialog::addToFocusList(WidgetArray& list, int id)
   if(id == 0 && _ourFocusList.size() > 0)
     _focusList = _ourFocusList[0].focusList;
 
-  if(list.size() > 0 && !(list[0]->getFlags() & WIDGET_NODRAW_FOCUS))
+  if(list.size() > 0)
     _ourFocusList[id].focusedWidget = list[0];
 }
 
@@ -167,6 +189,12 @@ void Dialog::buildFocusWidgetList(int id)
     if(_focusID > 0)
       _focusList.push_back(_ourFocusList[_focusID].focusList);
 
+    // Add button group at end of current focus list
+    // We do it this way for TabWidget, so that buttons are scanned
+    // *after* the widgets in the current tab
+    if(_ourButtonGroup.size() > 0)
+      _focusList.push_back(_ourButtonGroup);
+
     // Only update _focusedWidget if it doesn't belong to the main focus list
     // HACK - figure out how to properly deal with only one focus-able widget
     // in a tab -- TabWidget is the spawn of the devil
@@ -186,12 +214,6 @@ void Dialog::redrawFocus()
     _focusedWidget = Widget::setFocusForChain(this, getFocusList(), _focusedWidget, 0);
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool Dialog::wantsEvents()
-{
-  return _focusedWidget && _focusedWidget->wantsEvents();
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::draw()
 {
@@ -208,7 +230,7 @@ void Dialog::drawDialog()
 //    cerr << "Dialog::drawDialog()\n";
     FrameBuffer& fb = instance()->frameBuffer();
 
-    fb.fillRect(_x+1, _y+1, _w-2, _h-2, kBGColor);
+    fb.fillRect(_x+1, _y+1, _w-2, _h-2, kDlgColor);
     fb.box(_x, _y, _w, _h, kColor, kShadowColor);
 
     // Make all child widget dirty
@@ -228,6 +250,7 @@ void Dialog::drawDialog()
 
     // Tell the framebuffer this area is dirty
     fb.addDirtyRect(_x, _y, _w, _h);
+//cerr << "dirty: x = " << _x << ", y = " << _y << ", w = " << _w << ", h = " << _h << endl;
 
     _dirty = false;
   }
@@ -290,41 +313,43 @@ void Dialog::handleKeyDown(int ascii, int keycode, int modifiers)
   // Shift-left/shift-right cursor selects next tab
   // Tab sets next widget in current tab
   // Shift-Tab sets previous widget in current tab
-  //
-  // Widgets are only cycled if currently focused key hasn't claimed
-  // the TAB key
-  // TODO - figure out workaround for this
-  if(_ourTab && instance()->eventHandler().kbdShift(modifiers))
+  Event::Type e = Event::NoType;
+
+  // Detect selection of previous and next tab headers and objects
+  // For some strange reason, 'tab' needs to be interpreted as keycode,
+  // not ascii??
+  if(instance()->eventHandler().kbdShift(modifiers))
   {
-    // these key-combos are never passed to the child widget
-    if(ascii == 256 + 20)      // left arrow
+    if(ascii == 256+20 && _ourTab)      // left arrow
     {
       _ourTab->cycleTab(-1);
       return;
     }
-    else if(ascii == 256 + 19) // right arrow
+    else if(ascii == 256+19 && _ourTab) // right arrow
     {
       _ourTab->cycleTab(+1);
       return;
     }
+    else if(keycode == 9)     // tab
+      e = Event::UINavPrev;
   }
+  else if(keycode == 9)       // tab
+    e = Event::UINavNext;
 
-  if(keycode == 9)  // tab key
+  // Check the keytable now, since we might get one of the above events,
+  // which must always be processed before any widget sees it.
+  if(e == Event::NoType)
+    e = instance()->eventHandler().eventForKey(keycode, kMenuMode);
+
+  // Unless a widget has claimed all responsibility for data, we assume
+  // that if an event exists for the given data, it should have priority.
+  if(!handleNavEvent(e) && _focusedWidget)
   {
-    if(_focusedWidget && !(_focusedWidget->getFlags() & WIDGET_WANTS_TAB))
-    {
-      if(instance()->eventHandler().kbdShift(modifiers))
-        _focusedWidget = Widget::setFocusForChain(this, getFocusList(),
-                                                  _focusedWidget, -1);
-      else
-        _focusedWidget = Widget::setFocusForChain(this, getFocusList(),
-                                                  _focusedWidget, +1);
-      return;  // this key-combo is never passed to the child widget
-    }
+    if(_focusedWidget->wantsRaw() || e == Event::NoType)
+      _focusedWidget->handleKeyDown(ascii, keycode, modifiers);
+    else
+      _focusedWidget->handleEvent(e);
   }
-
-  if (_focusedWidget)
-    _focusedWidget->handleKeyDown(ascii, keycode, modifiers);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -388,9 +413,18 @@ void Dialog::handleMouseMoved(int x, int y, int button)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::handleJoyDown(int stick, int button)
 {
-  // Focused widget receives joystick events
-  if(_focusedWidget)
-    _focusedWidget->handleJoyDown(stick, button);
+  Event::Type e =
+    instance()->eventHandler().eventForJoyButton(stick, button, kMenuMode);
+
+  // Unless a widget has claimed all responsibility for data, we assume
+  // that if an event exists for the given data, it should have priority.
+  if(!handleNavEvent(e) && _focusedWidget)
+  {
+    if(_focusedWidget->wantsRaw() || e == Event::NoType)
+      _focusedWidget->handleJoyDown(stick, button);
+    else
+      _focusedWidget->handleEvent(e);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -404,16 +438,84 @@ void Dialog::handleJoyUp(int stick, int button)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Dialog::handleJoyAxis(int stick, int axis, int value)
 {
-  // Focused widget receives joystick events
-  if(_focusedWidget)
-    _focusedWidget->handleJoyAxis(stick, axis, value);
+  Event::Type e =
+    instance()->eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode);
+
+  // Unless a widget has claimed all responsibility for data, we assume
+  // that if an event exists for the given data, it should have priority.
+  if(!handleNavEvent(e) && _focusedWidget)
+  {
+    if(_focusedWidget->wantsRaw() || e == Event::NoType)
+      _focusedWidget->handleJoyAxis(stick, axis, value);
+    else if(value != 0)
+      _focusedWidget->handleEvent(e);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool Dialog::handleJoyHat(int stick, int hat, int value)
 {
-  // Focused widget receives joystick events
-  return (_focusedWidget && _focusedWidget->handleJoyHat(stick, hat, value));
+  Event::Type e =
+    instance()->eventHandler().eventForJoyHat(stick, hat, value, kMenuMode);
+
+  // Unless a widget has claimed all responsibility for data, we assume
+  // that if an event exists for the given data, it should have priority.
+  if(!handleNavEvent(e) && _focusedWidget)
+  {
+    if(_focusedWidget->wantsRaw() || e == Event::NoType)
+      return _focusedWidget->handleJoyHat(stick, hat, value);
+    else
+      return _focusedWidget->handleEvent(e);
+  }
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Dialog::handleNavEvent(Event::Type e)
+{
+  switch(e)
+  {
+    case Event::UINavPrev:
+      if(_focusedWidget && !_focusedWidget->wantsTab())
+      {
+        _focusedWidget = Widget::setFocusForChain(this, getFocusList(),
+                                                  _focusedWidget, -1);
+        return true;
+      }
+      break;
+
+    case Event::UINavNext:
+      if(_focusedWidget && !_focusedWidget->wantsTab())
+      {
+        _focusedWidget = Widget::setFocusForChain(this, getFocusList(),
+                                                  _focusedWidget, +1);
+        return true;
+      }
+      break;
+
+    case Event::UIOK:
+      if(_okWidget && _okWidget->isEnabled())
+      {
+        // Receiving 'OK' is the same as getting the 'Select' event
+        _okWidget->handleEvent(Event::UISelect);
+        return true;
+      }
+      break;
+
+    case Event::UICancel:
+      if(_cancelWidget && _cancelWidget->isEnabled())
+      {
+        // Receiving 'Cancel' is the same as getting the 'Select' event
+        _cancelWidget->handleEvent(Event::UISelect);
+        return true;
+      }
+      break;
+
+    default:
+      return false;
+      break;
+  }
+  return false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -442,16 +544,45 @@ Widget* Dialog::findWidget(int x, int y)
   return Widget::findWidgetInChain(_firstWidget, x, y);
 }
 
+ // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Dialog::addOKCancelBGroup(WidgetArray& wid, const GUI::Font& font)
+{
+  int buttonWidth  = font.getStringWidth("Cancel") + 15;
+  int buttonHeight = font.getLineHeight() + 4;
+  ButtonWidget* b;
+#ifndef MAC_OSX
+  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "OK", kOKCmd);
+  wid.push_back(b);
+  addOKWidget(b);
+  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
+  wid.push_back(b);
+  addCancelWidget(b);
+#else
+  b = new ButtonWidget(this, font, _w - 2 * (buttonWidth + 7), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Cancel", kCloseCmd);
+  wid.push_back(b);
+  addCancelWidget(b);
+  b = new ButtonWidget(this, font, _w - (buttonWidth + 10), _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "OK", kOKCmd);
+  wid.push_back(b);
+  addOKWidget(b);
+#endif
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ButtonWidget* Dialog::addButton(const GUI::Font& font, int x, int y,
-                                const string& label, int cmd, char hotkey)
+                                const string& label, int cmd)
 {
+  // FIXME - this is deprecated, and the UI code should be refactored
+  //         to remove all references to it
 #if 0
   const int w = 6 * font.getMaxCharWidth(),
             h = font.getFontHeight() + 6;
 
-  return new ButtonWidget(this, font, x, y, w, h, label, cmd, hotkey);
+  return new ButtonWidget(this, font, x, y, w, h, label, cmd);
 #else
-  return new ButtonWidget(this, font, x, y, kButtonWidth, 16, label, cmd, hotkey);
+  return new ButtonWidget(this, font, x, y, kButtonWidth, 16, label, cmd);
 #endif
 }
