@@ -8,23 +8,22 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: DialogContainer.cxx,v 1.31 2006/03/02 13:10:53 stephena Exp $
+// $Id: DialogContainer.cxx,v 1.44 2008/05/21 14:01:31 stephena Exp $
 //============================================================================
 
 #include "OSystem.hxx"
 #include "Dialog.hxx"
 #include "Stack.hxx"
 #include "EventHandler.hxx"
+#include "Joystick.hxx"
 #include "bspf.hxx"
 #include "DialogContainer.hxx"
 
-#define JOY_DEADZONE 3200
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DialogContainer::DialogContainer(OSystem* osystem)
   : myOSystem(osystem),
@@ -32,10 +31,6 @@ DialogContainer::DialogContainer(OSystem* osystem)
     myTime(0),
     myRefreshFlag(false)
 {
-  memset(&ourJoyMouse, 0, sizeof(JoyMouse));
-  ourJoyMouse.delay_time = 25;
-
-  ourEnableJoyMouseFlag = myOSystem->settings().getBool("joymouse");
   reset();
 }
 
@@ -58,50 +53,37 @@ void DialogContainer::updateTime(uInt32 time)
   // Check for pending continuous events and send them to the active dialog box
   Dialog* activeDialog = myDialogStack.top();
 
+  // Key still pressed
   if(myCurrentKeyDown.keycode != 0 && myKeyRepeatTime < myTime)
   {
     activeDialog->handleKeyDown(myCurrentKeyDown.ascii, myCurrentKeyDown.keycode,
                                 myCurrentKeyDown.flags);
-    myKeyRepeatTime = myTime + kKeyRepeatSustainDelay;
+    myKeyRepeatTime = myTime + kRepeatSustainDelay;
   }
 
+  // Mouse button still pressed
   if(myCurrentMouseDown.button != -1 && myClickRepeatTime < myTime)
   {
     activeDialog->handleMouseDown(myCurrentMouseDown.x - activeDialog->_x,
                                   myCurrentMouseDown.y - activeDialog->_y,
                                   myCurrentMouseDown.button, 1);
-    myClickRepeatTime = myTime + kClickRepeatSustainDelay;
+    myClickRepeatTime = myTime + kRepeatSustainDelay;
   }
 
-  if(ourEnableJoyMouseFlag && myCurrentAxisDown.stick != -1 &&
-     myAxisRepeatTime < myTime)
+  // Joystick button still pressed
+  if(myCurrentButtonDown.stick != -1 && myButtonRepeatTime < myTime)
   {
-    // The longer an axis event is enabled, the faster it should change
-    // We do this by decreasing the amount of time between consecutive axis events
-    // After a certain threshold, send 10 events at a time (this is necessary
-    // since at some point, we'd like to process more eventss than the
-    // current framerate allows)
-    myCurrentAxisDown.count++;
-    int interval = myCurrentAxisDown.count / 40 + 1;
-    myAxisRepeatTime = myTime + kAxisRepeatSustainDelay / interval;
-    if(interval > 3)
-    {
-      for(int i = 0; i < 10; ++i)
-        activeDialog->handleJoyAxis(myCurrentAxisDown.stick, myCurrentAxisDown.axis,
-                                    myCurrentAxisDown.value);
-      myAxisRepeatTime = myTime + kAxisRepeatSustainDelay / 3;
-    }
-    else
-    {
-      activeDialog->handleJoyAxis(myCurrentAxisDown.stick, myCurrentAxisDown.axis,
-                                  myCurrentAxisDown.value);
-      myAxisRepeatTime = myTime + kAxisRepeatSustainDelay / interval;
-    }
+    activeDialog->handleJoyDown(myCurrentButtonDown.stick, myCurrentButtonDown.button);
+    myButtonRepeatTime = myTime + kRepeatSustainDelay;
   }
 
-  // Update joy to mouse events
-  if(ourEnableJoyMouseFlag)
-    handleJoyMouse(time);
+  // Joystick axis still pressed
+  if(myCurrentAxisDown.stick != -1 && myAxisRepeatTime < myTime)
+  {
+    activeDialog->handleJoyAxis(myCurrentAxisDown.stick, myCurrentAxisDown.axis,
+                                myCurrentAxisDown.value);
+    myAxisRepeatTime = myTime + kRepeatSustainDelay;
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -128,6 +110,7 @@ void DialogContainer::addDialog(Dialog* d)
 {
   myDialogStack.push(d);
 
+  d->center();
   d->open();
   d->setDirty();  // Next update() will take care of drawing
 }
@@ -153,15 +136,12 @@ void DialogContainer::reStack()
     myDialogStack.pop();
   addDialog(myBaseDialog);
 
-  // Erase any previous messages
-  myOSystem->frameBuffer().hideMessage();
-
   // Reset all continuous events
   reset();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleKeyEvent(int unicode, int key, int mod, uInt8 state)
+void DialogContainer::handleKeyEvent(int ascii, int key, int mod, uInt8 state)
 {
   if(myDialogStack.empty())
     return;
@@ -170,16 +150,16 @@ void DialogContainer::handleKeyEvent(int unicode, int key, int mod, uInt8 state)
   Dialog* activeDialog = myDialogStack.top();
   if(state == 1)
   {
-    myCurrentKeyDown.ascii   = unicode;
+    myCurrentKeyDown.ascii   = ascii;
     myCurrentKeyDown.keycode = key;
     myCurrentKeyDown.flags   = mod;
-    myKeyRepeatTime = myTime + kKeyRepeatInitialDelay;
+    myKeyRepeatTime = myTime + kRepeatInitialDelay;
 
-    activeDialog->handleKeyDown(unicode, key, mod);
+    activeDialog->handleKeyDown(ascii, key, mod);
   }
   else
   {
-    activeDialog->handleKeyUp(unicode, key, mod);
+    activeDialog->handleKeyUp(ascii, key, mod);
 
     // Only stop firing events if it's the current key
     if (key == myCurrentKeyDown.keycode)
@@ -226,8 +206,8 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, int x, int y, uInt8
       }
 
       if(myLastClick.count && (myTime < myLastClick.time + kDoubleClickDelay)
-         && ABS(myLastClick.x - x) < 3
-         && ABS(myLastClick.y - y) < 3)
+         && BSPF_abs(myLastClick.x - x) < 3
+         && BSPF_abs(myLastClick.y - y) < 3)
       {
         myLastClick.count++;
       }
@@ -243,7 +223,7 @@ void DialogContainer::handleMouseButtonEvent(MouseButton b, int x, int y, uInt8
       myCurrentMouseDown.x = x;
       myCurrentMouseDown.y = y;
       myCurrentMouseDown.button = button;
-      myClickRepeatTime = myTime + kClickRepeatInitialDelay;
+      myClickRepeatTime = myTime + kRepeatInitialDelay;
 
       activeDialog->handleMouseDown(x - activeDialog->_x, y - activeDialog->_y,
                                     button, myLastClick.count);
@@ -277,85 +257,57 @@ void DialogContainer::handleJoyEvent(int stick, int button, uInt8 state)
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
 
-  if(activeDialog->wantsEvents())
+  if(state == 1)
   {
-    if(state == 1)
-      activeDialog->handleJoyDown(stick, button);
-    else
-      activeDialog->handleJoyUp(stick, button);
+    myCurrentButtonDown.stick  = stick;
+    myCurrentButtonDown.button = button;
+    myButtonRepeatTime = myTime + kRepeatInitialDelay;
+
+    activeDialog->handleJoyDown(stick, button);
+  }
+  else
+  {
+    // Only stop firing events if it's the current button
+    if(stick == myCurrentButtonDown.stick)
+      myCurrentButtonDown.stick = myCurrentButtonDown.button = -1;
+
+    activeDialog->handleJoyUp(stick, button);
   }
-  else if(ourEnableJoyMouseFlag)
-    myOSystem->eventHandler().createMouseButtonEvent(
-      ourJoyMouse.x, ourJoyMouse.y, state);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DialogContainer::handleJoyAxisEvent(int stick, int axis, int value)
 {
+// FIXME - analog axis events cause autofire to inadvertently come on and not go off
+
   if(myDialogStack.empty())
     return;
 
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
 
-  if(value > JOY_DEADZONE)
-    value -= JOY_DEADZONE;
-  else if(value < -JOY_DEADZONE )
-    value += JOY_DEADZONE;
+  int deadzone = Joystick::deadzone();
+  if(value > deadzone)
+    value -= deadzone;
+  else if(value < -deadzone )
+    value += deadzone;
   else
     value = 0;
 
-  if(activeDialog->wantsEvents())
+  // Only stop firing events if it's the current stick
+  if(myCurrentAxisDown.stick == stick && value == 0)
   {
-    // Only stop firing events if it's the current key
-    if(myCurrentAxisDown.stick == stick && value == 0)
-    {
-      myCurrentAxisDown.stick = myCurrentAxisDown.axis = -1;
-      myCurrentAxisDown.count = 0;
-    }
-    else
-    {
-      // Now account for repeated axis events (press and hold)
-      myCurrentAxisDown.stick = stick;
-      myCurrentAxisDown.axis  = axis;
-      myCurrentAxisDown.value = value;
-      myAxisRepeatTime = myTime + kAxisRepeatInitialDelay;
-    }
-    activeDialog->handleJoyAxis(stick, axis, value);
+    myCurrentAxisDown.stick = myCurrentAxisDown.axis = -1;
   }
   else
   {
-    if(axis % 2 == 0)  // x-direction
-    {
-      if(value != 0)
-      {
-        ourJoyMouse.x_vel = (value > 0) ? 1 : -1;
-        ourJoyMouse.x_down_count = 1;
-      }
-      else
-      {
-        ourJoyMouse.x_vel = 0;
-        ourJoyMouse.x_down_count = 0;
-      }
-    }
-    else   // y-direction
-    {
-      value = -value;
-
-      if(value != 0)
-      {
-        ourJoyMouse.y_vel = (-value > 0) ? 1 : -1;
-        ourJoyMouse.y_down_count = 1;
-      }
-      else
-      {
-        ourJoyMouse.y_vel = 0;
-        ourJoyMouse.y_down_count = 0;
-      }
-    }
-    myCurrentAxisDown.stick = myCurrentAxisDown.axis = -1;
-    myCurrentAxisDown.count = 0;
+    // Now account for repeated axis events (press and hold)
+    myCurrentAxisDown.stick = stick;
+    myCurrentAxisDown.axis  = axis;
+    myCurrentAxisDown.value = value;
+    myAxisRepeatTime = myTime + kRepeatInitialDelay;
   }
+  activeDialog->handleJoyAxis(stick, axis, value);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -367,126 +319,15 @@ void DialogContainer::handleJoyHatEvent(int stick, int hat, int value)
   // Send the event to the dialog box on the top of the stack
   Dialog* activeDialog = myDialogStack.top();
 
-  if(!(activeDialog->wantsEvents() &&
-       activeDialog->handleJoyHat(stick, hat, value)))
-  {
-    bool handled = true;
-    switch(value)
-    {
-      case kJHatCentered:
-        handleJoyAxisEvent(stick, 0, 0);
-        handleJoyAxisEvent(stick, 1, 0);       // axis 0 & 1, 0  ==> OFF
-        break;
-      case kJHatUp:
-        handleJoyAxisEvent(stick, 1, -32767);  // axis 1, -value ==> UP
-        break;
-      case kJHatLeft:
-        handleJoyAxisEvent(stick, 0, -32767);  // axis 0, -value ==> LEFT
-        break;
-      case kJHatDown:
-        handleJoyAxisEvent(stick, 1, 32767);   // axis 1, +value ==> DOWN
-        break;
-      case kJHatRight:
-        handleJoyAxisEvent(stick, 0, 32767);   // axis 0, +value ==> RIGHT
-        break;
-      default:
-        handled = false;
-    }
-    if(handled)
-      return;
-  }
+  // FIXME - add repeat processing, similar to axis/button events
+  activeDialog->handleJoyHat(stick, hat, value);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DialogContainer::handleJoyMouse(uInt32 time)
+void DialogContainer::handleResizeEvent()
 {
-  bool mouseAccel = true;//false;  // TODO - make this a commandline option
-  int oldX = ourJoyMouse.x, oldY = ourJoyMouse.y;
-
-  if(time >= ourJoyMouse.last_time + ourJoyMouse.delay_time)
-  {
-    ourJoyMouse.last_time = time;
-    if(ourJoyMouse.x_down_count == 1)
-    {
-      ourJoyMouse.x_down_time = time;
-      ourJoyMouse.x_down_count = 2;
-    }
-    if(ourJoyMouse.y_down_count == 1)
-    {
-      ourJoyMouse.y_down_time = time;
-      ourJoyMouse.y_down_count = 2;
-    }
-
-    if(ourJoyMouse.x_vel || ourJoyMouse.y_vel)
-    {
-      if(ourJoyMouse.x_down_count)
-      {
-        if(mouseAccel && time > ourJoyMouse.x_down_time + ourJoyMouse.delay_time * 12)
-        {
-          if(ourJoyMouse.x_vel > 0)
-            ourJoyMouse.x_vel++;
-          else
-            ourJoyMouse.x_vel--;
-        }
-        else if(time > ourJoyMouse.x_down_time + ourJoyMouse.delay_time * 8)
-        {
-          if(ourJoyMouse.x_vel > 0)
-            ourJoyMouse.x_vel = ourJoyMouse.amt;
-          else
-            ourJoyMouse.x_vel = -ourJoyMouse.amt;
-        }
-      }
-      if(ourJoyMouse.y_down_count)
-      {
-        if(mouseAccel && time > ourJoyMouse.y_down_time + ourJoyMouse.delay_time * 12)
-        {
-          if(ourJoyMouse.y_vel > 0)
-            ourJoyMouse.y_vel++;
-          else
-            ourJoyMouse.y_vel--;
-        }
-        else if(time > ourJoyMouse.y_down_time + ourJoyMouse.delay_time * 8)
-        {
-          if(ourJoyMouse.y_vel > 0)
-            ourJoyMouse.y_vel = ourJoyMouse.amt;
-          else
-            ourJoyMouse.y_vel = -ourJoyMouse.amt;
-        }
-      }
-
-      ourJoyMouse.x += ourJoyMouse.x_vel;
-      ourJoyMouse.y += ourJoyMouse.y_vel;
-
-      if(ourJoyMouse.x < 0)
-      {
-        ourJoyMouse.x = 0;
-        ourJoyMouse.x_vel = -1;
-        ourJoyMouse.x_down_count = 1;
-      }
-      else if(ourJoyMouse.x > ourJoyMouse.x_max)
-      {
-        ourJoyMouse.x = ourJoyMouse.x_max;
-        ourJoyMouse.x_vel = 1;
-        ourJoyMouse.x_down_count = 1;
-      }
-
-      if(ourJoyMouse.y < 0)
-      {
-        ourJoyMouse.y = 0;
-        ourJoyMouse.y_vel = -1;
-        ourJoyMouse.y_down_count = 1;
-      }
-      else if(ourJoyMouse.y > ourJoyMouse.y_max)
-      {
-        ourJoyMouse.y = ourJoyMouse.y_max;
-        ourJoyMouse.y_vel = 1;
-        ourJoyMouse.y_down_count = 1;
-      }
-
-      if(oldX != ourJoyMouse.x || oldY != ourJoyMouse.y)
-        myOSystem->eventHandler().createMouseMotionEvent(ourJoyMouse.x, ourJoyMouse.y);
-    }
-  }
+  // Send resize event to base dialog; it's up to the dialog to actually listen
+  myBaseDialog->handleCommand(NULL, kResizeCmd, 0, 0);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -498,21 +339,7 @@ void DialogContainer::reset()
   myLastClick.time = 0;
   myLastClick.count = 0;
 
+  myCurrentButtonDown.stick = myCurrentButtonDown.button = -1;
   myCurrentAxisDown.stick = myCurrentAxisDown.axis = -1;
-  myCurrentAxisDown.count = 0;
-
-  int oldX = ourJoyMouse.x, oldY = ourJoyMouse.y;
-  if(ourJoyMouse.x > ourJoyMouse.x_max)
-    ourJoyMouse.x = ourJoyMouse.x_max;
-  if(ourJoyMouse.y > ourJoyMouse.y_max)
-    ourJoyMouse.y = ourJoyMouse.y_max;
-
-  if(oldX != ourJoyMouse.x || oldY != ourJoyMouse.y)
-    myOSystem->eventHandler().createMouseMotionEvent(ourJoyMouse.x, ourJoyMouse.y);
+  myCurrentHatDown.stick = myCurrentHatDown.hat = -1;
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool DialogContainer::ourEnableJoyMouseFlag;
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-JoyMouse DialogContainer::ourJoyMouse;
