@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EditTextWidget.cxx,v 1.14 2006/02/22 17:38:04 stephena Exp $
+// $Id: EditTextWidget.cxx,v 1.19 2008/02/06 13:45:23 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -34,6 +34,7 @@ EditTextWidget::EditTextWidget(GuiObject* boss, const GUI::Font& font,
   _type = kEditTextWidget;
 
   _editable = true;
+  startEditMode();  // We're always in edit mode
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -82,7 +83,7 @@ void EditTextWidget::drawWidget(bool hilite)
   // Draw the text
   adjustOffset();
   fb.drawString(_font, _editString, _x + 2, _y + 2, getEditRect().width(),
-                _color, kTextAlignLeft, -_editScrollOffset, false);
+                _textcolor, kTextAlignLeft, -_editScrollOffset, false);
 
   // Draw the caret 
   drawCaret();
@@ -105,15 +106,18 @@ void EditTextWidget::lostFocusWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::startEditMode()
 {
+  EditableWidget::startEditMode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::endEditMode()
 {
+  EditableWidget::endEditMode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditTextWidget::abortEditMode()
 {
   setEditString(_backupString);
+  EditableWidget::abortEditMode();
 }
