@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: EventMappingWidget.cxx,v 1.14 2006/03/05 01:18:42 stephena Exp $
+// $Id: EventMappingWidget.cxx,v 1.23 2008/03/23 16:22:46 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -21,70 +21,73 @@
 
 #include <sstream>
 
-#include "OSystem.hxx"
-#include "Widget.hxx"
-#include "StringListWidget.hxx"
-#include "GuiUtils.hxx"
-#include "Event.hxx"
-#include "EventHandler.hxx"
-#include "EventMappingWidget.hxx"
-
 #include "bspf.hxx"
 
+#include "EventHandler.hxx"
+#include "Event.hxx"
+#include "OSystem.hxx"
+#include "StringListWidget.hxx"
+#include "Widget.hxx"
+
+#include "EventMappingWidget.hxx"
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventMappingWidget::EventMappingWidget(GuiObject* boss, const GUI::Font& font,
-                                       int x, int y, int w, int h)
+                                       int x, int y, int w, int h,
+                                       const StringList& actions, EventMode mode)
   : Widget(boss, font, x, y, w, h),
     CommandSender(boss),
+    myEventMode(mode),
     myActionSelected(-1),
     myRemapStatus(false),
     myFirstTime(true)
 {
-// FIXME
-  const int fontHeight = font.getFontHeight(),
-            lineHeight = font.getLineHeight();
+  const int fontHeight   = font.getFontHeight(),
+            lineHeight   = font.getLineHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 10,
+            buttonHeight = font.getLineHeight() + 4;
   int xpos = 5, ypos = 5;
 
   myActionsList = new StringListWidget(boss, font, xpos, ypos,
-                                       _w - 70, _h - 3*lineHeight);
+                                       _w - buttonWidth - 20, _h - 3*lineHeight);
   myActionsList->setTarget(this);
   myActionsList->setNumberingMode(kListNumberingOff);
   myActionsList->setEditable(false);
-  myActionsList->setFlags(WIDGET_NODRAW_FOCUS);
+  myActionsList->setList(actions);
   addFocusWidget(myActionsList);
 
   // Add remap, erase, cancel and default buttons
   xpos += myActionsList->getWidth() + 15;  ypos += 5;
-  myMapButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  myMapButton = new ButtonWidget(boss, font, xpos, ypos,
+                                 buttonWidth, buttonHeight,
                                  "Map", kStartMapCmd);
   myMapButton->setTarget(this);
-  ypos += 20;
-  myEraseButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  addFocusWidget(myMapButton);
+  ypos += lineHeight + 10;
+  myEraseButton = new ButtonWidget(boss, font, xpos, ypos,
+                                   buttonWidth, buttonHeight,
                                    "Erase", kEraseCmd);
   myEraseButton->setTarget(this);
-  ypos += 20;
-  myCancelMapButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  addFocusWidget(myEraseButton);
+  ypos += lineHeight + 10;
+  myCancelMapButton = new ButtonWidget(boss, font, xpos, ypos,
+                                       buttonWidth, buttonHeight,
                                        "Cancel", kStopMapCmd);
   myCancelMapButton->setTarget(this);
   myCancelMapButton->clearFlags(WIDGET_ENABLED);
-  ypos += 30;
-  myDefaultsButton = new ButtonWidget(boss, font, xpos, ypos, 50, 16,
+  addFocusWidget(myCancelMapButton);
+  ypos += lineHeight + 30;
+  myDefaultsButton = new ButtonWidget(boss, font, xpos, ypos,
+                                      buttonWidth, buttonHeight,
                                       "Defaults", kDefaultsCmd);
   myDefaultsButton->setTarget(this);
+  addFocusWidget(myDefaultsButton);
 
   // Show message for currently selected event
   xpos = 10;  ypos = 5 + myActionsList->getHeight() + 3;
   myKeyMapping  = new StaticTextWidget(boss, font, xpos, ypos, _w - 20, fontHeight,
                                        "Action: ", kTextAlignLeft);
   myKeyMapping->setFlags(WIDGET_CLEARBG);
-
-  // Get actions names
-  StringList l;
-
-  for(int i = 0; i < kActionListSize; ++i)
-    l.push_back(EventHandler::ourActionList[i].action);
-
-  myActionsList->setList(l);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -95,7 +98,6 @@ EventMappingWidget::~EventMappingWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventMappingWidget::loadConfig()
 {
-//cerr << "EventMappingWidget::loadConfig()\n";
   if(myFirstTime)
   {
     myActionsList->setSelected(0);
@@ -131,14 +133,14 @@ void EventMappingWidget::startRemapping()
   // And show a message indicating which key is being remapped
   ostringstream buf;
   buf << "Select action for '"
-      << EventHandler::ourActionList[ myActionSelected ].action
+      << instance()->eventHandler().actionAtIndex(myActionSelected, myEventMode)
       << "' event";	 	
-  myKeyMapping->setColor(kTextColorEm);
+  myKeyMapping->setTextColor(kTextColorEm);
   myKeyMapping->setLabel(buf.str());
 
-  // Make sure that this widget receives all events,
-  // and they aren't handled anywhere else
-  myActionsList->setFlags(WIDGET_WANTS_EVENTS);
+  // Make sure that this widget receives all raw data, before any
+  // pre-processing occurs
+  myActionsList->setFlags(WIDGET_WANTS_RAWDATA);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -147,8 +149,9 @@ void EventMappingWidget::eraseRemapping()
   if(myActionSelected < 0)
     return;
 
-  Event::Type event = EventHandler::ourActionList[ myActionSelected ].event;
-  instance()->eventHandler().eraseMapping(event);
+  Event::Type event =
+    instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
+  instance()->eventHandler().eraseMapping(event, myEventMode);
 
   drawKeyMapping();
 }
@@ -175,7 +178,7 @@ void EventMappingWidget::stopRemapping()
   }
 
   // Widget is now free to process events normally
-  myActionsList->clearFlags(WIDGET_WANTS_EVENTS);
+  myActionsList->clearFlags(WIDGET_WANTS_RAWDATA);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -184,8 +187,9 @@ void EventMappingWidget::drawKeyMapping()
   if(myActionSelected >= 0)
   {
     ostringstream buf;
-    buf << "Action: " << EventHandler::ourActionList[ myActionSelected ].key;
-    myKeyMapping->setColor(kTextColor);
+    buf << "Action: "
+        << instance()->eventHandler().keyAtIndex(myActionSelected, myEventMode);
+    myKeyMapping->setTextColor(kTextColor);
     myKeyMapping->setLabel(buf.str());
   }
 }
@@ -196,8 +200,9 @@ bool EventMappingWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   // Remap keys in remap mode
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event = EventHandler::ourActionList[ myActionSelected ].event;
-    if(instance()->eventHandler().addKeyMapping(event, keycode))
+    Event::Type event =
+      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance()->eventHandler().addKeyMapping(event, myEventMode, keycode))
       stopRemapping();
   }
   return true;
@@ -209,8 +214,9 @@ void EventMappingWidget::handleJoyDown(int stick, int button)
   // Remap joystick buttons in remap mode
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event = EventHandler::ourActionList[ myActionSelected ].event;
-    if(instance()->eventHandler().addJoyMapping(event, stick, button))
+    Event::Type event =
+      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance()->eventHandler().addJoyMapping(event, myEventMode, stick, button))
       stopRemapping();
   }
 }
@@ -221,8 +227,10 @@ void EventMappingWidget::handleJoyAxis(int stick, int axis, int value)
   // Remap joystick axes in remap mode
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event = EventHandler::ourActionList[ myActionSelected ].event;
-    if(instance()->eventHandler().addJoyAxisMapping(event, stick, axis, value))
+    Event::Type event =
+      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance()->eventHandler().addJoyAxisMapping(event, myEventMode,
+                                                    stick, axis, value))
       stopRemapping();
   }
 }
@@ -235,8 +243,10 @@ bool EventMappingWidget::handleJoyHat(int stick, int hat, int value)
   // Remap joystick hats in remap mode
   if(myRemapStatus && myActionSelected >= 0)
   {
-    Event::Type event = EventHandler::ourActionList[ myActionSelected ].event;
-    if(instance()->eventHandler().addJoyHatMapping(event, stick, hat, value))
+    Event::Type event =
+      instance()->eventHandler().eventAtIndex(myActionSelected, myEventMode);
+    if(instance()->eventHandler().addJoyHatMapping(event, myEventMode,
+                                                   stick, hat, value))
     {
       stopRemapping();
       result = true;
@@ -286,7 +296,7 @@ void EventMappingWidget::handleCommand(CommandSender* sender, int cmd,
       break;
 
     case kDefaultsCmd:
-      instance()->eventHandler().setDefaultMapping();
+      instance()->eventHandler().setDefaultMapping(myEventMode);
       drawKeyMapping();
       break;
   }
