@@ -8,25 +8,28 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: GameInfoDialog.cxx,v 1.24 2006/03/05 01:18:42 stephena Exp $
+// $Id: GameInfoDialog.cxx,v 1.56 2008/05/16 12:17:23 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "GuiUtils.hxx"
-#include "OSystem.hxx"
-#include "Props.hxx"
-#include "Widget.hxx"
+#include "Console.hxx"
 #include "Dialog.hxx"
 #include "EditTextWidget.hxx"
+#include "Launcher.hxx"
+#include "OSystem.hxx"
 #include "PopUpWidget.hxx"
+#include "Props.hxx"
+#include "PropsSet.hxx"
 #include "TabWidget.hxx"
+#include "Widget.hxx"
+
 #include "GameInfoDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -34,11 +37,14 @@ GameInfoDialog::GameInfoDialog(
       OSystem* osystem, DialogContainer* parent, const GUI::Font& font,
       GuiObject* boss, int x, int y, int w, int h)
   : Dialog(osystem, parent, x, y, w, h),
-    CommandSender(boss)
+    CommandSender(boss),
+    myPropertiesLoaded(false),
+    myDefaultsSelected(false)
 {
-  const int fontHeight = font.getFontHeight(),
-            lineHeight = font.getLineHeight();
-
+  const int lineHeight   = font.getLineHeight(),
+            fontHeight   = font.getFontHeight(),
+            buttonWidth  = font.getStringWidth("Defaults") + 20,
+            buttonHeight = font.getLineHeight() + 4;
   const int vBorder = 4;
   int xpos, ypos, lwidth, fwidth, pwidth, tabID;
   unsigned int i;
@@ -46,7 +52,9 @@ GameInfoDialog::GameInfoDialog(
 
   // The tab widget
   xpos = 2; ypos = vBorder;
-  myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos, _h - 24 - 2*ypos);
+  myTab = new TabWidget(this, font, xpos, ypos, _w - 2*xpos, _h - 24 - 2*ypos - 15);
+  addTabWidget(myTab);
+  addFocusWidget(myTab);
 
   // 1) Cartridge properties
   wid.clear();
@@ -109,10 +117,10 @@ GameInfoDialog::GameInfoDialog(
   ypos += lineHeight + 3;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Type:", kTextAlignLeft);
-  pwidth = font.getStringWidth("CV (Commavid extra ram)");
+  pwidth = font.getStringWidth("SB (128-256k SUPERbanking)");
   myType = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                            pwidth, lineHeight, "", 0, 0);
-  for(i = 0; i < 21; ++i)
+  for(i = 0; i < kNumCartTypes; ++i)
     myType->appendEntry(ourCartridgeList[i][0], i+1);
   wid.push_back(myType);
 
@@ -135,7 +143,7 @@ GameInfoDialog::GameInfoDialog(
   myLeftDiff->appendEntry("A", 2);
   wid.push_back(myLeftDiff);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Right Difficulty:", kTextAlignLeft);
   myRightDiff = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
@@ -144,7 +152,7 @@ GameInfoDialog::GameInfoDialog(
   myRightDiff->appendEntry("A", 2);
   wid.push_back(myRightDiff);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "TV Type:", kTextAlignLeft);
   myTVType = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
@@ -153,15 +161,6 @@ GameInfoDialog::GameInfoDialog(
   myTVType->appendEntry("B & W", 2);
   wid.push_back(myTVType);
 
-  ypos += lineHeight + 3;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Swap ports:", kTextAlignLeft);
-  mySwapPorts = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                pwidth, lineHeight, "", 0, 0);
-  mySwapPorts->appendEntry("Yes", 1);
-  mySwapPorts->appendEntry("No", 2);
-  wid.push_back(mySwapPorts);
-
   // Add items for tab 1
   addToFocusList(wid, tabID);
 
@@ -171,24 +170,55 @@ GameInfoDialog::GameInfoDialog(
   tabID = myTab->addTab("Controller");
 
   xpos = 10; ypos = vBorder;
-  lwidth = font.getStringWidth("Right Controller: ");
-  pwidth = font.getStringWidth("Booster-Grip");
+  lwidth = font.getStringWidth("P0 Controller: ");
+  pwidth = font.getStringWidth("CX-22 Trakball");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Left Controller:", kTextAlignLeft);
-  myLeftController = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                     pwidth, lineHeight, "", 0, 0);
-  for(i = 0; i < 5; ++i)
-    myLeftController->appendEntry(ourControllerList[i][0], i+1);
-  wid.push_back(myLeftController);
+                       "P0 Controller:", kTextAlignLeft);
+  myP0Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
+                                   pwidth, lineHeight, "", 0, 0);
+  for(i = 0; i < kNumControllerTypes; ++i)
+    myP0Controller->appendEntry(ourControllerList[i][0], i+1);
+  wid.push_back(myP0Controller);
 
-  ypos += lineHeight + 3;
+  xpos += lwidth+myP0Controller->getWidth() + 4;
+  new StaticTextWidget(myTab, font, xpos, ypos+1, font.getStringWidth("in "),
+                       fontHeight, "in ", kTextAlignLeft);
+  xpos += font.getStringWidth("in ");
+  myLeftPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                               "", 0, kLeftCChanged);
+  myLeftPort->appendEntry("left port", 1);
+  myLeftPort->appendEntry("right port", 2);
+  wid.push_back(myLeftPort);
+
+  xpos = 10;  ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Right Controller:", kTextAlignLeft);
-  myRightController = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                                      pwidth, lineHeight, "", 0, 0);
-  for(i = 0; i < 5; ++i)
-    myRightController->appendEntry(ourControllerList[i][0], i+1);
-  wid.push_back(myRightController);
+                       "P1 Controller:", kTextAlignLeft);
+  myP1Controller = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
+                                   pwidth, lineHeight, "", 0, 0);
+  for(i = 0; i < kNumControllerTypes; ++i)
+    myP1Controller->appendEntry(ourControllerList[i][0], i+1);
+  wid.push_back(myP1Controller);
+
+  xpos += lwidth+myP1Controller->getWidth() + 4;
+  new StaticTextWidget(myTab, font, xpos, ypos+1, font.getStringWidth("in "),
+                       fontHeight, "in ", kTextAlignLeft);
+  xpos += font.getStringWidth("in ");
+  myRightPort = new PopUpWidget(myTab, font, xpos, ypos, pwidth, lineHeight,
+                                "", 0, kRightCChanged);
+  myRightPort->appendEntry("left port", 1);
+  myRightPort->appendEntry("right port", 2);
+  wid.push_back(myRightPort);
+
+  xpos = 10;  ypos += lineHeight + 5;
+  pwidth = font.getStringWidth("Yes");
+  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
+                       "Swap Paddles:", kTextAlignLeft);
+  mySwapPaddles = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
+                                  pwidth, lineHeight, "", 0, 0);
+  mySwapPaddles->appendEntry("Yes", 1);
+  mySwapPaddles->appendEntry("No", 2);
+  wid.push_back(mySwapPaddles);
+
 
   // Add items for tab 2
   addToFocusList(wid, tabID);
@@ -200,54 +230,59 @@ GameInfoDialog::GameInfoDialog(
 
   xpos = 10; ypos = vBorder;
   lwidth = font.getStringWidth("Use Phosphor: ");
-  pwidth = font.getStringWidth("NTSC");
+  pwidth = font.getStringWidth("Auto-detect");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Format:", kTextAlignLeft);
   myFormat = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
                              pwidth, lineHeight, "", 0, 0);
-  myFormat->appendEntry("NTSC", 1);
-  myFormat->appendEntry("PAL", 2);
+  myFormat->appendEntry("Auto-detect", 1);
+  myFormat->appendEntry("NTSC", 2);
+  myFormat->appendEntry("PAL", 3);
+  myFormat->appendEntry("SECAM", 4);
+  myFormat->appendEntry("NTSC50", 5);
+  myFormat->appendEntry("PAL60", 6);
+  myFormat->appendEntry("SECAM60", 7);
+
   wid.push_back(myFormat);
 
-  ypos += lineHeight + 3;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "XStart:", kTextAlignLeft);
-  myXStart = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                                25, fontHeight, "");
-  wid.push_back(myXStart);
-
-  ypos += lineHeight + 3;
-  new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Width:", kTextAlignLeft);
-  myWidth = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
-                               25, fontHeight, "");
-  wid.push_back(myWidth);
-
-  ypos += lineHeight + 3;
+  ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "YStart:", kTextAlignLeft);
   myYStart = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
                                 25, fontHeight, "");
   wid.push_back(myYStart);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Height:", kTextAlignLeft);
   myHeight = new EditTextWidget(myTab, font, xpos+lwidth, ypos,
                                 25, fontHeight, "");
   wid.push_back(myHeight);
 
-  ypos += lineHeight + 3;
+  ypos += lineHeight + 5;
   pwidth = font.getStringWidth("Yes");
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Use Phosphor:", kTextAlignLeft);
   myPhosphor = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
-                               pwidth, lineHeight, "", 0, 0);
+                               pwidth, lineHeight, "", 0, kPhosphorChanged);
   myPhosphor->appendEntry("Yes", 1);
   myPhosphor->appendEntry("No", 2);
   wid.push_back(myPhosphor);
 
-  ypos += lineHeight + 3;
+  myPPBlend = new SliderWidget(myTab, font, xpos + lwidth + myPhosphor->getWidth() + 10,
+                               ypos, 30, lineHeight, "Blend: ",
+                               font.getStringWidth("Blend: "),
+                               kPPBlendChanged);
+  myPPBlend->setMinValue(1); myPPBlend->setMaxValue(100);
+  wid.push_back(myPPBlend);
+
+  myPPBlendLabel = new StaticTextWidget(myTab, font,
+                                        xpos + lwidth + myPhosphor->getWidth() + 10 + \
+                                        myPPBlend->getWidth() + 4, ypos+1,
+                                        15, fontHeight, "", kTextAlignLeft);
+  myPPBlendLabel->setFlags(WIDGET_CLEARBG);
+
+  ypos += lineHeight + 5;
   new StaticTextWidget(myTab, font, xpos, ypos+1, lwidth, fontHeight,
                        "Use HMBlanks:", kTextAlignLeft);
   myHmoveBlanks = new PopUpWidget(myTab, font, xpos+lwidth, ypos,
@@ -264,17 +299,19 @@ GameInfoDialog::GameInfoDialog(
   myTab->setActiveTab(0);
 
   // Add message concerning usage
-  new StaticTextWidget(this, font, 10, _h - 20, 120, fontHeight,
-                       "(*) Requires a ROM reload", kTextAlignLeft);
+  lwidth = font.getStringWidth("(*) Changes to properties require a ROM reload");
+  new StaticTextWidget(this, font, 10, _h - 38, lwidth, fontHeight,
+                       "(*) Changes to properties require a ROM reload",
+                       kTextAlignLeft);
 
   // Add Defaults, OK and Cancel buttons
-#ifndef MAC_OSX
-  addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "OK", kOKCmd, 0);
-  addButton(font, _w - (kButtonWidth + 10), _h - 24, "Cancel", kCloseCmd, 0);
-#else
-  addButton(font, _w - 2 * (kButtonWidth + 7), _h - 24, "Cancel", kCloseCmd, 0);
-  addButton(font, _w - (kButtonWidth + 10), _h - 24, "OK", kOKCmd, 0);
-#endif
+  wid.clear();
+  ButtonWidget* b;
+  b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
+                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
+  wid.push_back(b);
+  addOKCancelBGroup(wid, font);
+  addBGroupToFocusList(wid);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -285,29 +322,56 @@ GameInfoDialog::~GameInfoDialog()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GameInfoDialog::loadConfig()
 {
+  myPropertiesLoaded = false;
+  myDefaultsSelected = false;
+
+  if(&myOSystem->console())
+  {
+    myGameProperties = myOSystem->console().properties();
+    myPropertiesLoaded = true;
+    loadView();
+  }
+  else if(&myOSystem->launcher())
+  {
+    const string& md5 = myOSystem->launcher().selectedRomMD5();
+    if(md5 != "")
+    {
+      instance()->propSet().getMD5(md5, myGameProperties);
+      myPropertiesLoaded = true;
+      loadView();
+    }
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void GameInfoDialog::loadView()
+{
+  if(!myPropertiesLoaded)
+    return;
+
   string s;
   int i;
 
   // Cartridge properties
-  s = myGameProperties->get(Cartridge_Name);
+  s = myGameProperties.get(Cartridge_Name);
   myName->setEditString(s);
 
-  s = myGameProperties->get(Cartridge_MD5);
+  s = myGameProperties.get(Cartridge_MD5);
   myMD5->setLabel(s);
 
-  s = myGameProperties->get(Cartridge_Manufacturer);
+  s = myGameProperties.get(Cartridge_Manufacturer);
   myManufacturer->setEditString(s);
 
-  s = myGameProperties->get(Cartridge_ModelNo);
+  s = myGameProperties.get(Cartridge_ModelNo);
   myModelNo->setEditString(s);
 
-  s = myGameProperties->get(Cartridge_Rarity);
+  s = myGameProperties.get(Cartridge_Rarity);
   myRarity->setEditString(s);
 
-  s = myGameProperties->get(Cartridge_Note);
+  s = myGameProperties.get(Cartridge_Note);
   myNote->setEditString(s);
 
-  s = myGameProperties->get(Cartridge_Sound);
+  s = myGameProperties.get(Cartridge_Sound);
   if(s == "MONO")
     mySound->setSelectedTag(1);
   else if(s == "STEREO")
@@ -315,17 +379,17 @@ void GameInfoDialog::loadConfig()
   else
     mySound->setSelectedTag(0);
 
-  s = myGameProperties->get(Cartridge_Type);
-  for(i = 0; i < 21; ++i)
+  s = myGameProperties.get(Cartridge_Type);
+  for(i = 0; i < kNumCartTypes; ++i)
   {
     if(s == ourCartridgeList[i][1])
       break;
   }
-  i = (i == 21) ? 0: i + 1;
+  i = (i == kNumCartTypes) ? 0: i + 1;
   myType->setSelectedTag(i);
 
   // Console properties
-  s = myGameProperties->get(Console_LeftDifficulty);
+  s = myGameProperties.get(Console_LeftDifficulty);
   if(s == "B")
     myLeftDiff->setSelectedTag(1);
   else if(s == "A")
@@ -333,7 +397,7 @@ void GameInfoDialog::loadConfig()
   else
     myLeftDiff->setSelectedTag(0);
 
-  s = myGameProperties->get(Console_RightDifficulty);
+  s = myGameProperties.get(Console_RightDifficulty);
   if(s == "B")
     myRightDiff->setSelectedTag(1);
   else if(s == "A")
@@ -341,7 +405,7 @@ void GameInfoDialog::loadConfig()
   else
     myRightDiff->setSelectedTag(0);
 
-  s = myGameProperties->get(Console_TelevisionType);
+  s = myGameProperties.get(Console_TelevisionType);
   if(s == "COLOR")
     myTVType->setSelectedTag(1);
   else if(s == "BLACKANDWHITE")
@@ -349,63 +413,81 @@ void GameInfoDialog::loadConfig()
   else
     myTVType->setSelectedTag(0);
 
-  s = myGameProperties->get(Console_SwapPorts);
-  if(s == "YES")
-    mySwapPorts->setSelectedTag(1);
-  else if(s == "NO")
-    mySwapPorts->setSelectedTag(2);
-  else
-    mySwapPorts->setSelectedTag(0);
+  s = myGameProperties.get(Console_SwapPorts);
+  myLeftPort->setSelectedTag(s == "NO" ? 1 : 2);
+  myRightPort->setSelectedTag(s == "NO" ? 2 : 1);
 
   // Controller properties
-  s = myGameProperties->get(Controller_Left);
-  for(i = 0; i < 5; ++i)
+  s = myGameProperties.get(Controller_Left);
+  for(i = 0; i < kNumControllerTypes; ++i)
   {
     if(s == ourControllerList[i][1])
       break;
   }
-  i = (i == 5) ? 0: i + 1;
-  myLeftController->setSelectedTag(i);
+  i = (i == kNumControllerTypes) ? 0: i + 1;
+  myP0Controller->setSelectedTag(i);
 
-  s = myGameProperties->get(Controller_Right);
-  for(i = 0; i < 5; ++i)
+  s = myGameProperties.get(Controller_Right);
+  for(i = 0; i < kNumControllerTypes; ++i)
   {
     if(s == ourControllerList[i][1])
       break;
   }
-  i = (i == 5) ? 0: i + 1;
-  myRightController->setSelectedTag(i);
+  i = (i == kNumControllerTypes) ? 0: i + 1;
+  myP1Controller->setSelectedTag(i);
+
+  s = myGameProperties.get(Controller_SwapPaddles);
+  if(s == "YES")
+    mySwapPaddles->setSelectedTag(1);
+  else if(s == "NO")
+    mySwapPaddles->setSelectedTag(2);
+  else
+    mySwapPaddles->setSelectedTag(0);
 
   // Display properties
-  s = myGameProperties->get(Display_Format);
-  if(s == "NTSC")
+  s = myGameProperties.get(Display_Format);
+  if(s == "AUTO-DETECT")
     myFormat->setSelectedTag(1);
-  else if(s == "PAL")
+  else if(s == "NTSC")
     myFormat->setSelectedTag(2);
+  else if(s == "PAL")
+    myFormat->setSelectedTag(3);
+  else if(s == "SECAM")
+    myFormat->setSelectedTag(4);
+  else if(s == "NTSC50")
+    myFormat->setSelectedTag(5);
+  else if(s == "PAL60")
+    myFormat->setSelectedTag(6);
+  else if(s == "SECAM60")
+    myFormat->setSelectedTag(7);
   else
     myFormat->setSelectedTag(0);
 
-  s = myGameProperties->get(Display_XStart);
-  myXStart->setEditString(s);
-
-  s = myGameProperties->get(Display_Width);
-  myWidth->setEditString(s);
-
-  s = myGameProperties->get(Display_YStart);
+  s = myGameProperties.get(Display_YStart);
   myYStart->setEditString(s);
 
-  s = myGameProperties->get(Display_Height);
+  s = myGameProperties.get(Display_Height);
   myHeight->setEditString(s);
 
-  s = myGameProperties->get(Display_Phosphor);
+  myPPBlend->setEnabled(false);
+  myPPBlendLabel->setEnabled(false);
+  s = myGameProperties.get(Display_Phosphor);
   if(s == "YES")
+  {
     myPhosphor->setSelectedTag(1);
+    myPPBlend->setEnabled(true);
+    myPPBlendLabel->setEnabled(true);
+  }
   else if(s == "NO")
     myPhosphor->setSelectedTag(2);
   else
     myPhosphor->setSelectedTag(0);
 
-  s = myGameProperties->get(Emulation_HmoveBlanks);
+  s = myGameProperties.get(Display_PPBlend);
+  myPPBlend->setValue(atoi(s.c_str()));
+  myPPBlendLabel->setLabel(s);
+
+  s = myGameProperties.get(Emulation_HmoveBlanks);
   if(s == "YES")
     myHmoveBlanks->setSelectedTag(1);
   else if(s == "NO")
@@ -419,35 +501,38 @@ void GameInfoDialog::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GameInfoDialog::saveConfig()
 {
+  if(!myPropertiesLoaded)
+    return;
+
   string s;
   int i, tag;
 
   // Cartridge properties
   s = myName->getEditString();
-  myGameProperties->set(Cartridge_Name, s);
+  myGameProperties.set(Cartridge_Name, s);
 
   s = myManufacturer->getEditString();
-  myGameProperties->set(Cartridge_Manufacturer, s);
+  myGameProperties.set(Cartridge_Manufacturer, s);
 
   s = myModelNo->getEditString();
-  myGameProperties->set(Cartridge_ModelNo, s);
+  myGameProperties.set(Cartridge_ModelNo, s);
 
   s = myRarity->getEditString();
-  myGameProperties->set(Cartridge_Rarity, s);
+  myGameProperties.set(Cartridge_Rarity, s);
 
   s = myNote->getEditString();
-  myGameProperties->set(Cartridge_Note, s);
+  myGameProperties.set(Cartridge_Note, s);
 
   tag = mySound->getSelectedTag();
   s = (tag == 1) ? "Mono" : "Stereo";
-  myGameProperties->set(Cartridge_Sound, s);
+  myGameProperties.set(Cartridge_Sound, s);
 
   tag = myType->getSelectedTag();
-  for(i = 0; i < 21; ++i)
+  for(i = 0; i < kNumCartTypes; ++i)
   {
     if(i == tag-1)
     {
-      myGameProperties->set(Cartridge_Type, ourCartridgeList[i][1]);
+      myGameProperties.set(Cartridge_Type, ourCartridgeList[i][1]);
       break;
     }
   }
@@ -455,79 +540,130 @@ void GameInfoDialog::saveConfig()
   // Console properties
   tag = myLeftDiff->getSelectedTag();
   s = (tag == 1) ? "B" : "A";
-  myGameProperties->set(Console_LeftDifficulty, s);
+  myGameProperties.set(Console_LeftDifficulty, s);
 
   tag = myRightDiff->getSelectedTag();
   s = (tag == 1) ? "B" : "A";
-  myGameProperties->set(Console_RightDifficulty, s);
+  myGameProperties.set(Console_RightDifficulty, s);
 
   tag = myTVType->getSelectedTag();
   s = (tag == 1) ? "Color" : "BlackAndWhite";
-  myGameProperties->set(Console_TelevisionType, s);
-
-  tag = mySwapPorts->getSelectedTag();
-  s = (tag == 1) ? "Yes" : "No";
-  myGameProperties->set(Console_SwapPorts, s);
+  myGameProperties.set(Console_TelevisionType, s);
 
   // Controller properties
-  tag = myLeftController->getSelectedTag();
-  for(i = 0; i < 5; ++i)
+  tag = myP0Controller->getSelectedTag();
+  for(i = 0; i < kNumControllerTypes; ++i)
   {
     if(i == tag-1)
     {
-      myGameProperties->set(Controller_Left, ourControllerList[i][0]);
+      myGameProperties.set(Controller_Left, ourControllerList[i][1]);
       break;
     }
   }
 
-  tag = myRightController->getSelectedTag();
-  for(i = 0; i < 5; ++i)
+  tag = myP1Controller->getSelectedTag();
+  for(i = 0; i < kNumControllerTypes; ++i)
   {
     if(i == tag-1)
     {
-      myGameProperties->set(Controller_Right, ourControllerList[i][0]);
+      myGameProperties.set(Controller_Right, ourControllerList[i][1]);
       break;
     }
   }
 
+  tag = myLeftPort->getSelectedTag();
+  s = (tag == 1) ? "No" : "Yes";
+  myGameProperties.set(Console_SwapPorts, s);
+
+  tag = mySwapPaddles->getSelectedTag();
+  s = (tag == 1) ? "Yes" : "No";
+  myGameProperties.set(Controller_SwapPaddles, s);
+
   // Display properties
   tag = myFormat->getSelectedTag();
-  s = (tag == 1) ? "NTSC" : "PAL";
-  myGameProperties->set(Display_Format, s);
-
-  s = myXStart->getEditString();
-  myGameProperties->set(Display_XStart, s);
-
-  s = myWidth->getEditString();
-  myGameProperties->set(Display_Width, s);
+  s = (tag == 7) ? "SECAM60" : (tag == 6) ? "PAL60" : (tag == 5) ? "NTSC50" :
+      (tag == 4) ? "SECAM"   : (tag == 3) ? "PAL"   : (tag == 2) ? "NTSC" :
+      "AUTO-DETECT";
+  myGameProperties.set(Display_Format, s);
 
   s = myYStart->getEditString();
-  myGameProperties->set(Display_YStart, s);
+  myGameProperties.set(Display_YStart, s);
 
   s = myHeight->getEditString();
-  myGameProperties->set(Display_Height, s);
+  myGameProperties.set(Display_Height, s);
 
   tag = myPhosphor->getSelectedTag();
   s = (tag == 1) ? "Yes" : "No";
-  myGameProperties->set(Display_Phosphor, s);
+  myGameProperties.set(Display_Phosphor, s);
+
+  s = myPPBlendLabel->getLabel();
+  myGameProperties.set(Display_PPBlend, s);
 
   tag = myHmoveBlanks->getSelectedTag();
   s = (tag == 1) ? "Yes" : "No";
-  myGameProperties->set(Emulation_HmoveBlanks, s);
+  myGameProperties.set(Emulation_HmoveBlanks, s);
+
+  // Determine whether to add or remove an entry from the properties set
+  if(myDefaultsSelected)
+    instance()->propSet().removeMD5(myGameProperties.get(Cartridge_MD5));
+  else
+    instance()->propSet().insert(myGameProperties, true);
+
+  // In any event, inform the Console and save the properties
+  if(&myOSystem->console())
+    instance()->console().setProperties(myGameProperties);
+  instance()->propSet().save(myOSystem->propertiesFile());
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void GameInfoDialog::setDefaults()
+{
+  // Load the default properties
+  string md5 = myGameProperties.get(Cartridge_MD5);
+  instance()->propSet().getMD5(md5, myGameProperties, true);
+
+  // Reload the current dialog
+  loadView();
+  myDefaultsSelected = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
-                                          int data, int id)
+                                   int data, int id)
 {
   switch (cmd)
   {
     case kOKCmd:
       saveConfig();
-      instance()->eventHandler().saveProperties();
       close();
       break;
 
+    case kDefaultsCmd:
+      setDefaults();
+      break;
+
+    case kLeftCChanged:
+      myRightPort->setSelectedTag(
+        myLeftPort->getSelectedTag() == 2 ? 1 : 2);
+      break;
+
+    case kRightCChanged:
+      myLeftPort->setSelectedTag(
+        myRightPort->getSelectedTag() == 2 ? 1 : 2);
+      break;
+
+    case kPhosphorChanged:
+    {
+      bool status = myPhosphor->getSelectedTag() == 1 ? true : false;
+      myPPBlend->setEnabled(status);
+      myPPBlendLabel->setEnabled(status);
+      break;
+    }
+
+    case kPPBlendChanged:
+      myPPBlendLabel->setValue(myPPBlend->getValue());
+      break;
+
     default:
       Dialog::handleCommand(sender, cmd, data, 0);
       break;
@@ -535,35 +671,44 @@ void GameInfoDialog::handleCommand(CommandSender* sender, int cmd,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* GameInfoDialog::ourControllerList[5][2] = {
-  { "Booster-Grip", "BOOSTER-GRIP" },
-  { "Driving",      "DRIVING"      },
-  { "Keyboard",     "KEYBOARD"     },
-  { "Paddles",      "PADDLES"      },
-  { "Joystick",     "JOYSTICK"     }
+const char* GameInfoDialog::ourControllerList[kNumControllerTypes][2] = {
+  { "Joystick",       "JOYSTICK"    },
+  { "Paddles",        "PADDLES"     },
+  { "BoosterGrip",    "BOOSTERGRIP" },
+  { "Driving",        "DRIVING"     },
+  { "Keyboard",       "KEYBOARD"    },
+  { "CX-22 Trakball", "TRACKBALL22" },
+  { "CX-80 Mouse",    "TRACKBALL80" },
+  { "AmigaMouse",     "AMIGAMOUSE"  },
+  { "AtariVox",       "ATARIVOX"    },
+  { "SaveKey",        "SAVEKEY"     }
 };
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* GameInfoDialog::ourCartridgeList[21][2] = {
-  { "Auto-detect",     "AUTO-DETECT"   },
-  { "2K (2K Atari)",            "2K"   },
-  { "3E (32K Tigervision)",     "3E"   },
-  { "3F (512K Tigervision)",    "3F"   },
-  { "4K (4K Atari)",            "4K"   },
-  { "AR (Supercharger)",        "AR"   },
-  { "CV (Commavid extra ram)",  "CV"   },
-  { "DPC (Pitfall II)",         "DPC"  },
-  { "E0 (8K Parker Bros)",      "E0"   },
-  { "E7 (16K M-network)",       "E7"   },
-  { "F4 (32K Atari)",           "F4"   },
-  { "F4SC (32K Atari + ram)",   "F4SC" },
-  { "F6 (16K Atari)",           "F6"   },
-  { "F6SC (16K Atari + ram)",   "F6SC" },
-  { "F8 (8K Atari)",            "F8"   },
-  { "F8SC (8K Atari + ram)",    "F8SC" },
-  { "FASC (CBS RAM Plus)",      "FASC" },
-  { "FE (8K Decathlon)",        "FE"   },
-  { "MB (Dynacom Megaboy)",     "MB"   },
-  { "MC (C. Wilkson Megacart)", "MC"   },
-  { "UA (8K UA Ltd.)",          "UA"   }
+const char* GameInfoDialog::ourCartridgeList[kNumCartTypes][2] = {
+  { "Auto-detect",       "AUTO-DETECT"   },
+  { "0840 (8K ECONObanking)",     "0840" },
+  { "2K (2K Atari)",              "2K"   },
+  { "3E (32K Tigervision)",       "3E"   },
+  { "3F (512K Tigervision)",      "3F"   },
+  { "4A50 (64K 4A50 + ram)",      "4A50" },
+  { "4K (4K Atari)",              "4K"   },
+  { "AR (Supercharger)",          "AR"   },
+  { "CV (Commavid extra ram)",    "CV"   },
+  { "DPC (Pitfall II)",           "DPC"  },
+  { "E0 (8K Parker Bros)",        "E0"   },
+  { "E7 (16K M-network)",         "E7"   },
+  { "F4 (32K Atari)",             "F4"   },
+  { "F4SC (32K Atari + ram)",     "F4SC" },
+  { "F6 (16K Atari)",             "F6"   },
+  { "F6SC (16K Atari + ram)",     "F6SC" },
+  { "F8 (8K Atari)",              "F8"   },
+  { "F8SC (8K Atari + ram)",      "F8SC" },
+  { "FASC (CBS RAM Plus)",        "FASC" },
+  { "FE (8K Decathlon)",          "FE"   },
+  { "MB (Dynacom Megaboy)",       "MB"   },
+  { "MC (C. Wilkson Megacart)",   "MC"   },
+  { "SB (128-256k SUPERbanking)", "SB"   },
+  { "UA (8K UA Ltd.)",            "UA"   },
+  { "X07 (64K AtariAge)",         "X07"  }
 };
