@@ -8,30 +8,26 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: InputTextDialog.cxx,v 1.12 2006/03/23 16:16:33 stephena Exp $
+// $Id: InputTextDialog.cxx,v 1.20 2008/02/06 13:45:23 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "Widget.hxx"
-#include "EditTextWidget.hxx"
-#include "Dialog.hxx"
-#include "GuiObject.hxx"
-#include "GuiUtils.hxx"
-#include "InputTextDialog.hxx"
-
 #include "bspf.hxx"
 
-enum {
-  kAcceptCmd = 'ACPT'
-};
+#include "Dialog.hxx"
+#include "EditTextWidget.hxx"
+#include "GuiObject.hxx"
+#include "OSystem.hxx"
+#include "Widget.hxx"
+
+#include "InputTextDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& font,
@@ -43,9 +39,10 @@ InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& font,
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
             lineHeight = font.getLineHeight(),
-            bwidth  = font.getStringWidth(" Cancel "),
-            bheight = font.getLineHeight() + 2;
+            bwidth  = font.getStringWidth("  Cancel  "),
+            bheight = font.getLineHeight() + 4;
   unsigned int xpos, ypos, i, lwidth = 0, maxIdx = 0;
+  WidgetArray wid;
 
   // Calculate real dimensions
   _w = fontWidth * 25;
@@ -63,7 +60,6 @@ InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& font,
   lwidth = font.getStringWidth(labels[maxIdx]);
 
   // Create editboxes for all labels
-  WidgetArray wid;
   ypos = lineHeight;
   for(i = 0; i < labels.size(); ++i)
   {
@@ -80,24 +76,34 @@ InputTextDialog::InputTextDialog(GuiObject* boss, const GUI::Font& font,
     myInput.push_back(w);
     ypos += lineHeight + 5;
   }
-  addToFocusList(wid);
 
   xpos = 10;
   myTitle = new StaticTextWidget(this, font, xpos, ypos, _w - 2*xpos, fontHeight,
                                  "", kTextAlignCenter);
-  myTitle->setColor(kTextColorEm);
+  myTitle->setTextColor(kTextColorEm);
 
+  ButtonWidget* b;
 #ifndef MAC_OSX
-  new ButtonWidget(this, font, _w - 2 * (bwidth + 10), _h - bheight - 10,
-                   bwidth, bheight, "OK", kAcceptCmd, 0);
-  new ButtonWidget(this, font, _w - (bwidth + 10), _h - bheight - 10,
-                   bwidth, bheight, "Cancel", kCloseCmd, 0);
+  b = new ButtonWidget(this, font, _w - 2 * (bwidth + 10), _h - bheight - 10,
+                   bwidth, bheight, "OK", kAcceptCmd);
+  wid.push_back(b);
+  addOKWidget(b);
+  b = new ButtonWidget(this, font, _w - (bwidth + 10), _h - bheight - 10,
+                   bwidth, bheight, "Cancel", kCloseCmd);
+  wid.push_back(b);
+  addCancelWidget(b);
 #else
-  new ButtonWidget(this, font, _w - 2 * (bwidth + 10), _h - bheight - 10,
-                   bwidth, bheight, "Cancel", kCloseCmd, 0);
-  new ButtonWidget(this, font, _w - (bwidth + 10), _h - bheight - 10,
-                   bwidth, bheight, "OK", kAcceptCmd, 0);
+  b = new ButtonWidget(this, font, _w - 2 * (bwidth + 10), _h - bheight - 10,
+                   bwidth, bheight, "Cancel", kCloseCmd);
+  wid.push_back(b);
+  addCancelWidget(b);
+  b = new ButtonWidget(this, font, _w - (bwidth + 10), _h - bheight - 10,
+                   bwidth, bheight, "OK", kAcceptCmd);
+  wid.push_back(b);
+  addOKWidget(b);
 #endif
+
+  addToFocusList(wid);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
