@@ -8,25 +8,45 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Launcher.cxx,v 1.7 2005/06/25 17:26:32 stephena Exp $
+// $Id: Launcher.cxx,v 1.22 2008/03/12 22:04:51 stephena Exp $
 //============================================================================
 
+#include <sstream>
+
+class Properties;
+
+#include "LauncherDialog.hxx"
 #include "Version.hxx"
 #include "OSystem.hxx"
+#include "Settings.hxx"
 #include "FrameBuffer.hxx"
-#include "LauncherDialog.hxx"
 #include "bspf.hxx"
 #include "Launcher.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Launcher::Launcher(OSystem* osystem)
-    : DialogContainer(osystem)
+  : DialogContainer(osystem),
+    myWidth(400),
+    myHeight(300)
 {
+  int w, h;
+  myOSystem->settings().getSize("launcherres", w, h);
+  myWidth = BSPF_max(w, 0);
+  myHeight = BSPF_max(h, 0);
+
+  // Error check the resolution
+  myWidth = BSPF_max(myWidth, 320u);
+  myWidth = BSPF_min(myWidth, osystem->desktopWidth());
+  myHeight = BSPF_max(myHeight, 240u);
+  myHeight = BSPF_min(myHeight, osystem->desktopHeight());
+  myOSystem->settings().setSize("launcherres", myWidth, myHeight);
+
+  myBaseDialog = new LauncherDialog(myOSystem, this, 0, 0, myWidth, myHeight);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -34,19 +54,15 @@ Launcher::~Launcher()
 {
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Launcher::initialize()
-{
-  // We only create one instance of this dialog, since each time we do so,
-  // the ROM listing is read from disk.  This can be very expensive.
-  if(myBaseDialog == NULL)
-    myBaseDialog = new LauncherDialog(myOSystem, this,
-                                      0, 0, kLauncherWidth, kLauncherHeight);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Launcher::initializeVideo()
 {
   string title = string("Stella ") + STELLA_VERSION;
-  myOSystem->frameBuffer().initialize(title, kLauncherWidth, kLauncherHeight, false);
+  myOSystem->frameBuffer().initialize(title, myWidth, myHeight);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string Launcher::selectedRomMD5()
+{
+  return ((LauncherDialog*)myBaseDialog)->selectedRomMD5();
 }
