@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ListWidget.cxx,v 1.41 2006/03/19 22:06:20 stephena Exp $
+// $Id: ListWidget.cxx,v 1.50 2008/02/06 13:45:24 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -46,6 +46,10 @@ ListWidget::ListWidget(GuiObject* boss, const GUI::Font& font,
 {
   _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
   _type = kListWidget;
+  _bgcolor = kWidColor;
+  _bgcolorhi = kWidColor;
+  _textcolor = kTextColor;
+  _textcolorhi = kTextColor;
 
   _cols = w / _fontWidth;
   _rows = h / _fontHeight;
@@ -67,7 +71,8 @@ ListWidget::~ListWidget()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ListWidget::setSelected(int item)
 {
-  assert(item >= -2 && item < (int)_list.size());
+  if(item < -1 || item >= (int)_list.size())
+    return;
 
   if(isEnabled())
   {
@@ -85,7 +90,8 @@ void ListWidget::setSelected(int item)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ListWidget::setHighlighted(int item)
 {
-  assert(item >= -1 && item < (int)_list.size());
+  if(item < -1 || item >= (int)_list.size())
+    return;
 
   if(isEnabled())
   {
@@ -137,6 +143,9 @@ void ListWidget::recalc()
 
   _scrollBar->_numEntries     = _list.size();
   _scrollBar->_entriesPerPage = _rows;
+
+  // Reset to normal data entry
+  abortEditMode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -258,7 +267,7 @@ bool ListWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   else
   {
     // not editmode
-    switch (ascii)
+    switch (keycode)
     {
       case ' ':  // space
         // Snap list back to currently highlighted line
@@ -269,48 +278,6 @@ bool ListWidget::handleKeyDown(int ascii, int keycode, int modifiers)
         }
         break;
 
-      case '\n':  // enter/return
-      case '\r':
-        if (_selectedItem >= 0)
-        {
-          // override continuous enter keydown
-          if (_editable && (_currentKeyDown != '\n' && _currentKeyDown != '\r'))
-            startEditMode();
-          else
-            sendCommand(kListItemActivatedCmd, _selectedItem, _id);
-        }
-        break;
-
-      case kCursorUp:
-        if (_selectedItem > 0)
-          _selectedItem--;
-        break;
-
-      case kCursorDown:
-        if (_selectedItem < (int)_list.size() - 1)
-          _selectedItem++;
-        break;
-
-      case kCursorPgUp:
-        _selectedItem -= _rows - 1;
-        if (_selectedItem < 0)
-          _selectedItem = 0;
-        break;
-
-      case kCursorPgDn:
-        _selectedItem += _rows - 1;
-        if (_selectedItem >= (int)_list.size() )
-          _selectedItem = _list.size() - 1;
-        break;
-
-      case kCursorHome:
-        _selectedItem = 0;
-        break;
-
-      case kCursorEnd:
-        _selectedItem = _list.size() - 1;
-        break;
-
       default:
         handled = false;
     }
@@ -336,10 +303,79 @@ bool ListWidget::handleKeyUp(int ascii, int keycode, int modifiers)
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool ListWidget::handleEvent(Event::Type e)
+{
+  if(!isEnabled() || _editMode)
+    return false;
+
+  bool handled = true;
+  int oldSelectedItem = _selectedItem;
+
+  switch(e)
+  {
+    case Event::UISelect:
+      if (_selectedItem >= 0)
+      {
+        if (_editable)
+          startEditMode();
+        else
+          sendCommand(kListItemActivatedCmd, _selectedItem, _id);
+      }
+      break;
+
+    case Event::UIUp:
+      if (_selectedItem > 0)
+        _selectedItem--;
+      break;
+
+    case Event::UIDown:
+      if (_selectedItem < (int)_list.size() - 1)
+        _selectedItem++;
+      break;
+
+    case Event::UIPgUp:
+      _selectedItem -= _rows - 1;
+      if (_selectedItem < 0)
+        _selectedItem = 0;
+      break;
+
+    case Event::UIPgDown:
+      _selectedItem += _rows - 1;
+      if (_selectedItem >= (int)_list.size() )
+        _selectedItem = _list.size() - 1;
+      break;
+
+    case Event::UIHome:
+      _selectedItem = 0;
+      break;
+
+    case Event::UIEnd:
+      _selectedItem = _list.size() - 1;
+      break;
+
+    default:
+      handled = false;
+  }
+
+  if (_selectedItem != oldSelectedItem)
+  {
+    _scrollBar->draw();
+    scrollToSelected();
+
+    sendCommand(kListSelectionChangedCmd, _selectedItem, _id);
+  }
+
+  return handled;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ListWidget::lostFocusWidget()
 {
   _editMode = false;
+
+  // Reset to normal data entry
+  abortEditMode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -408,6 +444,9 @@ void ListWidget::startEditMode()
   {
     _editMode = true;
     setEditString(_list[_selectedItem]);
+
+    // Widget gets raw data while editing
+    EditableWidget::startEditMode();
   }
 }
 
@@ -417,16 +456,21 @@ void ListWidget::endEditMode()
   if (!_editMode)
     return;
 
-  // send a message that editing finished with a return/enter key press
+  // Send a message that editing finished with a return/enter key press
   _editMode = false;
   _list[_selectedItem] = _editString;
   sendCommand(kListItemDataChangedCmd, _selectedItem, _id);
+
+  // Reset to normal data entry
+  EditableWidget::endEditMode();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ListWidget::abortEditMode()
 {
-  // undo any changes made
-  assert(_selectedItem >= 0);
+  // Undo any changes made
   _editMode = false;
+
+  // Reset to normal data entry
+  EditableWidget::abortEditMode();
 }
