@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Menu.cxx,v 1.12 2005/12/18 18:37:03 stephena Exp $
+// $Id: Menu.cxx,v 1.17 2008/02/06 13:45:24 stephena Exp $
 //============================================================================
 
 #include "Dialog.hxx"
@@ -25,24 +25,12 @@ class Properties;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Menu::Menu(OSystem* osystem)
-    : DialogContainer(osystem)
+  : DialogContainer(osystem)
 {
+  myBaseDialog = new OptionsDialog(myOSystem, this, 0, false);  // in game mode
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Menu::~Menu()
 {
 }
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Menu::initialize()
-{
-  delete myBaseDialog;
-  myBaseDialog = new OptionsDialog(myOSystem, this);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Menu::setGameProfile(Properties& props)
-{
-  ((OptionsDialog*)myBaseDialog)->setGameProfile(props);
-}
