@@ -8,25 +8,26 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: PopUpWidget.cxx,v 1.24 2006/02/22 17:38:04 stephena Exp $
+// $Id: PopUpWidget.cxx,v 1.39 2008/02/06 13:45:24 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
 #include "bspf.hxx"
-#include "OSystem.hxx"
-#include "FrameBuffer.hxx"
-#include "Stack.hxx"
-#include "Dialog.hxx"
+
 #include "DialogContainer.hxx"
-#include "GuiUtils.hxx"
+#include "Dialog.hxx"
+#include "FrameBuffer.hxx"
+#include "OSystem.hxx"
+#include "Stack.hxx"
 #include "StringListWidget.hxx"
+
 #include "PopUpWidget.hxx"
 
 #define UP_DOWN_BOX_HEIGHT	10
@@ -68,7 +69,6 @@ void PopUpDialog::drawDialog()
   {
     FrameBuffer& fb = instance()->frameBuffer();
 
-//cerr << "PopUpDialog::drawDialog()\n";
     // Draw the menu border
     fb.hLine(_x, _y, _x + _w - 1, kColor);
     fb.hLine(_x, _y + _h - 1, _x + _w - 1, kShadowColor);
@@ -87,7 +87,7 @@ void PopUpDialog::drawDialog()
     // The last entry may be empty. Fill it with black.
     if(_twoColumns && (count & 1))
       fb.fillRect(_x + 1 + _w / 2, _y + 1 + _popUpBoss->_fontHeight * (_entriesPerColumn - 1),
-                  _w / 2 - 1, _popUpBoss->_fontHeight, kBGColor);
+                  _w / 2 - 1, _popUpBoss->_fontHeight, kWidColor);
 
     _dirty = false;
     fb.addDirtyRect(_x, _y, _w, _h);
@@ -135,34 +135,71 @@ void PopUpDialog::handleKeyDown(int ascii, int keycode, int modifiers)
   if(isMouseDown())
     return;
 
-  switch (ascii)
+  Event::Type e = instance()->eventHandler().eventForKey(keycode, kMenuMode);
+  handleEvent(e);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpDialog::handleJoyDown(int stick, int button)
+{
+  Event::Type e =
+    instance()->eventHandler().eventForJoyButton(stick, button, kMenuMode);
+  handleEvent(e);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpDialog::handleJoyAxis(int stick, int axis, int value)
+{
+  if(value != 0)  // we don't care about 'axis up' events
   {
-    case '\n':      // enter/return
-    case '\r':
+    Event::Type e =
+      instance()->eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode);
+    handleEvent(e);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool PopUpDialog::handleJoyHat(int stick, int hat, int value)
+{
+  Event::Type e =
+    instance()->eventHandler().eventForJoyHat(stick, hat, value, kMenuMode);
+  handleEvent(e);
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpDialog::handleEvent(Event::Type e)
+{
+  switch(e)
+  {
+    case Event::UISelect:
       sendSelection();
       break;
-    case 27:        // escape
-      cancelSelection();
-      break;
-    case 256+17:    // up arrow
+    case Event::UIUp:
+    case Event::UILeft:
       moveUp();
       break;
-    case 256+18:    // down arrow
+    case Event::UIDown:
+    case Event::UIRight:
       moveDown();
       break;
-    case 256+22:    // home
+    case Event::UIHome:
       setSelection(0);
       break;
-    case 256+23:    // end
+    case Event::UIEnd:
       setSelection(_popUpBoss->_entries.size()-1);
       break;
+    case Event::UICancel:
+      cancelSelection();
+      break;
+    default:
+      break;
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpDialog::drawMenuEntry(int entry, bool hilite)
 {
-//cerr << "PopUpDialog::drawMenuEntry\n";
   FrameBuffer& fb = instance()->frameBuffer();
 
   // Draw one entry of the popup menu, including selection
@@ -196,7 +233,7 @@ void PopUpDialog::drawMenuEntry(int entry, bool hilite)
   }
 
   string& name = _popUpBoss->_entries[entry].name;
-  fb.fillRect(x, y, w, _popUpBoss->_fontHeight, hilite ? kTextColorHi : kBGColor);
+  fb.fillRect(x, y, w, _popUpBoss->_fontHeight, hilite ? kTextColorHi : kWidColor);
 
   if(name.size() == 0)
   {
@@ -206,7 +243,7 @@ void PopUpDialog::drawMenuEntry(int entry, bool hilite)
   }
   else
     fb.drawString(_popUpBoss->font(), name, x + 1, y + 2, w - 2,
-                  hilite ? kBGColor : kTextColor);
+                  hilite ? _popUpBoss->_textcolorhi : _popUpBoss->_textcolor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -215,6 +252,9 @@ void PopUpDialog::recalc()
   // Perform clipping / switch to scrolling mode if we don't fit on the screen
   const int height = instance()->frameBuffer().baseHeight();
 
+  _x = _popUpBoss->getAbsX() + _popUpBoss->_labelWidth;
+  _y = _popUpBoss->getAbsY() + _popUpBoss->getHeight();
+
   _h = _popUpBoss->_entries.size() * _popUpBoss->_fontHeight + 2;
 
   // HACK: For now, we do not do scrolling. Instead, we draw the dialog
@@ -389,9 +429,12 @@ PopUpWidget::PopUpWidget(GuiObject* boss, const GUI::Font& font,
     _labelWidth(labelWidth),
     _cmd(cmd)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
-           WIDGET_NODRAW_FOCUS;
+  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
   _type = kPopUpWidget;
+  _bgcolor = kDlgColor;
+  _bgcolorhi = kWidColor;
+  _textcolor = kTextColor;
+  _textcolorhi = kTextColor;
 
   _selectedItem = -1;
 
@@ -424,6 +467,22 @@ void PopUpWidget::handleMouseDown(int x, int y, int button, int clickCount)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool PopUpWidget::handleEvent(Event::Type e)
+{
+  if(!isEnabled())
+    return false;
+
+  switch(e)
+  {
+    case Event::UISelect:
+      handleMouseDown(0, 0, 1, 0);
+      return true;
+    default:
+      return false;
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpWidget::appendEntry(const string& entry, int tag)
 {
@@ -431,11 +490,6 @@ void PopUpWidget::appendEntry(const string& entry, int tag)
   e.name = entry;
   e.tag = tag;
   _entries.push_back(e);
-
-  // Each time an entry is added, the popup dialog gets larger
-  // This isn't as efficient as it could be, since it's called
-  // each time an entry is added (which can be dozens of times).
-  myPopUpDialog->recalc();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -460,6 +514,19 @@ void PopUpWidget::setSelected(int item)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::setSelectedName(const string& name)
+{
+  for(unsigned int item = 0; item < _entries.size(); ++item)
+  {
+    if(_entries[item].name == name)
+    {
+      setSelected(item);
+      return;
+    }
+  }
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpWidget::setSelectedTag(int tag)
 {
@@ -473,6 +540,12 @@ void PopUpWidget::setSelectedTag(int tag)
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PopUpWidget::setSelectedMax()
+{
+  setSelected(_entries.size() - 1);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PopUpWidget::drawWidget(bool hilite)
 {
@@ -485,7 +558,7 @@ void PopUpWidget::drawWidget(bool hilite)
   // Draw the label, if any
   if (_labelWidth > 0)
     fb.drawString(_font, _label, _x, _y + myTextY, _labelWidth,
-                  isEnabled() ? kTextColor : kColor, kTextAlignRight);
+                  isEnabled() ? _textcolor : kColor, kTextAlignRight);
 
   // Draw a thin frame around us.
   fb.hLine(x, _y, x + w - 1, kColor);
@@ -493,6 +566,9 @@ void PopUpWidget::drawWidget(bool hilite)
   fb.vLine(x, _y, _y+_h-1, kColor);
   fb.vLine(x + w - 1, _y, _y +_h - 1, kShadowColor);
 
+  // Fill the background
+  fb.fillRect(x + 1, _y + 1, w - 2, _h - 2, kWidColor);
+
   // Draw an arrow pointing down at the right end to signal this is a dropdown/popup
   fb.drawBitmap(up_down_arrows, x+w - 10, _y + myArrowsY,
                 !isEnabled() ? kColor : hilite ? kTextColorHi : kTextColor);
