@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ProgressDialog.cxx,v 1.6 2006/02/22 17:38:04 stephena Exp $
+// $Id: ProgressDialog.cxx,v 1.12 2008/03/14 23:52:17 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -24,19 +24,18 @@
 #include "Dialog.hxx"
 #include "DialogContainer.hxx"
 #include "ProgressDialog.hxx"
-#include "GuiUtils.hxx"
 
 #include "bspf.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
                                const string& message)
-    : Dialog(boss->instance(), boss->parent(), 0, 0, 16, 16),
-      myMessage(NULL),
-      mySlider(NULL),
-      myStart(0),
-      myFinish(0),
-      myStep(0)
+  : Dialog(boss->instance(), boss->parent(), 0, 0, 16, 16),
+    myMessage(NULL),
+    mySlider(NULL),
+    myStart(0),
+    myFinish(0),
+    myStep(0)
 {
   const int fontWidth  = font.getMaxCharWidth(),
             fontHeight = font.getFontHeight(),
@@ -53,14 +52,12 @@ ProgressDialog::ProgressDialog(GuiObject* boss, const GUI::Font& font,
   xpos = fontWidth; ypos = lineHeight;
   myMessage = new StaticTextWidget(this, font, xpos, ypos, lwidth, fontHeight,
                                    message, kTextAlignCenter);
-  myMessage->setColor(kTextColorEm);
+  myMessage->setTextColor(kTextColorEm);
 
   xpos = fontWidth; ypos += 2 * lineHeight;
   mySlider = new SliderWidget(this, font, xpos, ypos, lwidth, lineHeight, "", 0, 0);
-  mySlider->setMinValue(100);
-  mySlider->setMaxValue(200);
-  mySlider->setValue(100);  // Prevents the slider from initially drawing
-                            // across the entire screen for a split-second
+  mySlider->setMinValue(1);
+  mySlider->setMaxValue(100);
 
   parent()->addDialog(this);
   instance()->frameBuffer().update();
@@ -88,20 +85,19 @@ void ProgressDialog::setRange(int start, int finish, int step)
 {
   myStart = start;
   myFinish = finish;
-  myStep = step;
-  myCurrentStep = 100;
+  myStep = (int)((step / 100.0) * (myFinish - myStart + 1));
+
+  mySlider->setMinValue(myStart);
+  mySlider->setMaxValue(myFinish);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ProgressDialog::setProgress(int progress)
 {
   // Only increase the progress bar if we have arrived at a new step
-  // IE, we only increase in intervals specified by setRange()
-  int p = (int) (((double)progress / myFinish) * 100 + 100);
-  if(p >= myCurrentStep)
+  if(progress - mySlider->getValue() > myStep)
   {
-    myCurrentStep += myStep;
-    mySlider->setValue(p);
+    mySlider->setValue(progress);
     instance()->frameBuffer().update();
   }
 }
