@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: ScrollBarWidget.cxx,v 1.14 2006/03/25 00:34:17 stephena Exp $
+// $Id: ScrollBarWidget.cxx,v 1.22 2008/03/02 20:48:51 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -27,16 +27,12 @@
 
 /*
  * TODO:
- * - Auto-repeat: if user clicks & holds on one of the arrows, then after a
- *   brief delay, it should start to contiously scroll
  * - Allow for a horizontal scrollbar, too?
  * - If there are less items than fit on one pages, no scrolling can be done
  *   and we thus should not highlight the arrows/slider.
- * - Allow the mouse wheel to scroll more than one line at a time
  */
 
 #define UP_DOWN_BOX_HEIGHT	10
-#define WHEEL_LINES 2
 
 // Up arrow
 static unsigned int up_arrow[8] = {
@@ -69,6 +65,8 @@ ScrollBarWidget::ScrollBarWidget(GuiObject* boss, const GUI::Font& font,
 {
   _flags = WIDGET_ENABLED | WIDGET_TRACK_MOUSE | WIDGET_CLEARBG;
   _type = kScrollBarWidget;
+  _bgcolor = kWidColor;
+  _bgcolorhi = kWidColor;
 
   _part = kNoPart;
   _sliderHeight = 0;
@@ -138,9 +136,9 @@ void ScrollBarWidget::handleMouseWheel(int x, int y, int direction)
     return;
 
   if(direction < 0)
-    _currentPos -= WHEEL_LINES;
+    _currentPos -= _WHEEL_LINES;
   else
-    _currentPos += WHEEL_LINES;
+    _currentPos += _WHEEL_LINES;
 
   // Make sure that _currentPos is still inside the bounds
   checkBounds(old_pos);
@@ -262,24 +260,26 @@ void ScrollBarWidget::drawWidget(bool hilite)
   fb.frameRect(_x, _y, _w, UP_DOWN_BOX_HEIGHT, kColor);
   fb.drawBitmap(up_arrow, _x, _y,
                 isSinglePage ? kColor :
-                (hilite && _part == kUpArrowPart) ? kTextColorHi : kTextColor);
+                (hilite && _part == kUpArrowPart) ? kScrollColorHi : kScrollColor);
 
   // Down arrow
   fb.frameRect(_x, bottomY - UP_DOWN_BOX_HEIGHT, _w, UP_DOWN_BOX_HEIGHT, kColor);
   fb.drawBitmap(down_arrow, _x, bottomY - UP_DOWN_BOX_HEIGHT,
                 isSinglePage ? kColor :
-                (hilite && _part == kDownArrowPart) ? kTextColorHi : kTextColor);
+                (hilite && _part == kDownArrowPart) ? kScrollColorHi : kScrollColor);
 
   // Slider
   if(!isSinglePage)
   {
     fb.fillRect(_x, _y + _sliderPos, _w, _sliderHeight,
-               (hilite && _part == kSliderPart) ? kTextColorHi : kTextColor);
+               (hilite && _part == kSliderPart) ? kScrollColorHi : kScrollColor);
     fb.frameRect(_x, _y + _sliderPos, _w, _sliderHeight, kColor);
     int y = _y + _sliderPos + _sliderHeight / 2;
-    int color = (hilite && _part == kSliderPart) ? kColor : kShadowColor;
-    fb.hLine(_x + 2, y - 2, _x + _w - 3, color);
-    fb.hLine(_x + 2, y,     _x + _w - 3, color);
-    fb.hLine(_x + 2, y + 2, _x + _w - 3, color);
+    fb.hLine(_x + 2, y - 2, _x + _w - 3, kWidColor);
+    fb.hLine(_x + 2, y,     _x + _w - 3, kWidColor);
+    fb.hLine(_x + 2, y + 2, _x + _w - 3, kWidColor);
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int ScrollBarWidget::_WHEEL_LINES = 4;
