@@ -8,28 +8,29 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: Widget.cxx,v 1.43 2006/03/25 00:34:17 stephena Exp $
+// $Id: Widget.cxx,v 1.57 2008/02/06 13:45:24 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
 //============================================================================
 
-#include "OSystem.hxx"
-#include "FrameBuffer.hxx"
-#include "Font.hxx"
-#include "Dialog.hxx"
-#include "DialogContainer.hxx"
-#include "Command.hxx"
-#include "GuiObject.hxx"
 #include "bspf.hxx"
-#include "GuiUtils.hxx"
-#include "Widget.hxx"
+
+#include "Command.hxx"
+#include "DialogContainer.hxx"
+#include "Dialog.hxx"
 #include "EditableWidget.hxx"
+#include "Font.hxx"
+#include "FrameBuffer.hxx"
+#include "GuiObject.hxx"
+#include "OSystem.hxx"
+
+#include "Widget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Widget::Widget(GuiObject* boss, const GUI::Font& font,
@@ -41,7 +42,10 @@ Widget::Widget(GuiObject* boss, const GUI::Font& font,
     _id(-1),
     _flags(0),
     _hasFocus(false),
-    _color(kTextColor)
+    _bgcolor(kWidColor),
+    _bgcolorhi(kWidColor),
+    _textcolor(kTextColor),
+    _textcolorhi(kTextColorHi)
 {
   // Insert into the widget list of the boss
   _next = _boss->_firstWidget;
@@ -73,6 +77,7 @@ void Widget::draw()
   if(!isVisible() || !_boss->isVisible())
     return;
 
+  bool hasBorder = _flags & WIDGET_BORDER;
   int oldX = _x, oldY = _y, oldW = _w, oldH = _h;
 
   // Account for our relative position in the dialog
@@ -81,15 +86,18 @@ void Widget::draw()
 
   // Clear background (unless alpha blending is enabled)
   if(_flags & WIDGET_CLEARBG)
-    fb.fillRect(_x, _y, _w, _h, kBGColor);
+  {
+    int x = _x, y = _y, w = _w, h = _h;
+    if(hasBorder)
+    {
+      x++; y++; w-=2; h-=2;
+    }
+    fb.fillRect(x, y, w, h, (_flags & WIDGET_HILITED) ? _bgcolorhi : _bgcolor);
+  }
 
   // Draw border
-  if(_flags & WIDGET_BORDER) {
-    int colorA = kColor;
-    int colorB = kShadowColor;
-    if((_flags & WIDGET_INV_BORDER) == WIDGET_INV_BORDER)
-      SWAP(colorA, colorB);
-    fb.box(_x, _y, _w, _h, colorA, colorB);
+  if(hasBorder) {
+    fb.box(_x, _y, _w, _h, kColor, kShadowColor);
     _x += 4;
     _y += 4;
     _w -= 8;
@@ -100,7 +108,7 @@ void Widget::draw()
   drawWidget((_flags & WIDGET_HILITED) ? true : false);
 
   // Restore x/y
-  if (_flags & WIDGET_BORDER) {
+  if (hasBorder) {
     _x -= 4;
     _y -= 4;
     _w += 8;
@@ -239,8 +247,7 @@ Widget* Widget::setFocusForChain(GuiObject* boss, WidgetArray& arr,
     if(tmp->_hasFocus)
     {
       tmp->lostFocus();
-      if(!(tmp->_flags & WIDGET_NODRAW_FOCUS))
-        fb.frameRect(x, y, w, h, kBGColor);
+      fb.frameRect(x, y, w, h, kDlgColor);
 
       tmp->setDirty(); tmp->draw();
       fb.addDirtyRect(x, y, w, h);
@@ -279,8 +286,7 @@ Widget* Widget::setFocusForChain(GuiObject* boss, WidgetArray& arr,
       w = rect.width(), h = rect.height();
 
   tmp->receivedFocus();
-  if(!(tmp->_flags & WIDGET_NODRAW_FOCUS))
-    fb.frameRect(x, y, w, h, kTextColorEm, kDashLine);
+  fb.frameRect(x, y, w, h, kWidFrameColor, kDashLine);
 
   tmp->setDirty(); tmp->draw();
   fb.addDirtyRect(x, y, w, h);
@@ -302,11 +308,16 @@ void Widget::setDirtyInChain(Widget* start)
 StaticTextWidget::StaticTextWidget(GuiObject *boss, const GUI::Font& font,
                                    int x, int y, int w, int h,
                                    const string& text, TextAlignment align)
-    : Widget(boss, font, x, y, w, h),
-      _align(align)
+  : Widget(boss, font, x, y, w, h),
+    _align(align)
 {
   _flags = WIDGET_ENABLED | WIDGET_CLEARBG;
   _type = kStaticTextWidget;
+  _bgcolor = kDlgColor;
+  _bgcolorhi = kDlgColor;
+  _textcolor = kTextColor;
+  _textcolorhi = kTextColor;
+
   _label = text;
 }
 
@@ -332,21 +343,25 @@ void StaticTextWidget::drawWidget(bool hilite)
 {
   FrameBuffer& fb = _boss->instance()->frameBuffer();
   fb.drawString(_font, _label, _x, _y, _w,
-                isEnabled() ? _color : kColor, _align);
+                isEnabled() ? _textcolor : kColor, _align);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ButtonWidget::ButtonWidget(GuiObject *boss, const GUI::Font& font,
                            int x, int y, int w, int h,
-                           const string& label, int cmd, uInt8 hotkey)
+                           const string& label, int cmd)
   : StaticTextWidget(boss, font, x, y, w, h, label, kTextAlignCenter),
     CommandSender(boss),
-    _cmd(cmd),
-    _editable(false),
-    _hotkey(hotkey)
+    _cmd(cmd)
 {
   _flags = WIDGET_ENABLED | WIDGET_BORDER | WIDGET_CLEARBG;
   _type = kButtonWidget;
+  _bgcolor = kBtnColor;
+  _bgcolorhi = kBtnColorHi;
+  _textcolor = kBtnTextColor;
+  _textcolorhi = kBtnTextColorHi;
+
+  _editable = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -364,14 +379,14 @@ void ButtonWidget::handleMouseLeft(int button)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ButtonWidget::handleKeyDown(int ascii, int keycode, int modifiers)
+bool ButtonWidget::handleEvent(Event::Type e)
 {
-  // (De)activate with space or return
-  switch(ascii)
+  if(!isEnabled())
+    return false;
+
+  switch(e)
   {
-    case '\n':  // enter/return
-    case '\r':
-    case ' ' :  // space
+    case Event::UISelect:
       // Simulate mouse event
       handleMouseUp(0, 0, 1, 0);
       return true;
@@ -380,14 +395,6 @@ bool ButtonWidget::handleKeyDown(int ascii, int keycode, int modifiers)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ButtonWidget::handleJoyDown(int stick, int button)
-{
-  // Any button activates the button, but only while in joymouse mode
-  if(DialogContainer::joymouse())
-    handleMouseUp(0, 0, 1, 0);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ButtonWidget::handleMouseUp(int x, int y, int button, int clickCount)
 {
@@ -398,33 +405,17 @@ void ButtonWidget::handleMouseUp(int x, int y, int button, int clickCount)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ButtonWidget::wantsFocus()
-{
-  return _editable;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ButtonWidget::setEditable(bool editable)
-{
-  _editable = editable;
-  if(_editable)
-    setFlags(WIDGET_RETAIN_FOCUS);
-  else
-    clearFlags(WIDGET_RETAIN_FOCUS);
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ButtonWidget::drawWidget(bool hilite)
 {
   FrameBuffer& fb = _boss->instance()->frameBuffer();
   fb.drawString(_font, _label, _x, _y + (_h - _fontHeight)/2 + 1, _w,
-                !isEnabled() ? kColor : hilite ? kTextColorHi : _color, _align);
+                !isEnabled() ? kColor : hilite ? _textcolorhi : _textcolor, _align);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 /* 8x8 checkbox bitmap */
-static unsigned int checked_img[8] =
+static unsigned int checked_img_x[8] =
 {
 	0x00000000,
 	0x01000010,
@@ -436,13 +427,24 @@ static unsigned int checked_img[8] =
 	0x00000000,
 };
 
+static unsigned int checked_img_o[8] =
+{
+	0x00011000,
+	0x00111100,
+	0x01111110,
+	0x11111111,
+	0x11111111,
+	0x01111110,
+	0x00111100,
+	0x00011000,
+};
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CheckboxWidget::CheckboxWidget(GuiObject *boss, const GUI::Font& font,
                                int x, int y, const string& label,
                                int cmd)
-  : ButtonWidget(boss, font, x, y, 16, 16, label, cmd, 0),
+  : ButtonWidget(boss, font, x, y, 16, 16, label, cmd),
     _state(false),
-    _editable(true),
     _holdFocus(true),
     _fillRect(false),
     _drawBox(true),
@@ -450,8 +452,11 @@ CheckboxWidget::CheckboxWidget(GuiObject *boss, const GUI::Font& font,
     _boxY(0),
     _textY(0)
 {
-  _flags = WIDGET_ENABLED | WIDGET_RETAIN_FOCUS;
+  _flags = WIDGET_ENABLED;
   _type = kCheckboxWidget;
+  _bgcolor = _bgcolorhi = kWidColor;
+
+  _editable = true;
 
   if(label == "")
     _w = 14;
@@ -480,36 +485,10 @@ void CheckboxWidget::handleMouseUp(int x, int y, int button, int clickCount)
   }
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CheckboxWidget::handleKeyDown(int ascii, int keycode, int modifiers)
-{
-  // (De)activate with space or return
-  switch(ascii)
-  {
-    case '\n':  // enter/return
-    case '\r':
-    case ' ' :  // space
-      // Simulate mouse event
-      handleMouseUp(0, 0, 1, 0);
-      return true;
-    default:
-      return false;
-  }
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool CheckboxWidget::wantsFocus()
-{
-  if(!_holdFocus)
-    return false;
-  else
-    return _editable;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CheckboxWidget::setEditable(bool editable)
 {
-  _holdFocus = _editable = editable;
+  _editable = editable;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -518,7 +497,6 @@ void CheckboxWidget::setState(bool state)
   if(_state != state)
   {
     _state = state;
-    _flags ^= WIDGET_INV_BORDER;
     setDirty(); draw();
   }
 }
@@ -532,31 +510,32 @@ void CheckboxWidget::drawWidget(bool hilite)
   if(_drawBox)
     fb.box(_x, _y + _boxY, 14, 14, kColor, kShadowColor);
 
-  // If checked, draw cross inside the box
-  if(_state)
+  // Do we draw a square or cross?
+  fb.fillRect(_x + 2, _y + _boxY + 2, 10, 10, _bgcolor);
+  if(isEnabled())
   {
-    if(_fillRect)
-      fb.fillRect(_x + 2, _y + _boxY + 2, 10, 10,
-                  isEnabled() ? _color : kColor);
-    else
-      fb.drawBitmap(checked_img, _x + 3, _y + _boxY + 3,
-                    isEnabled() ? _color : kColor);
+    if(_state)
+    {
+      unsigned int* img = _fillRect ? checked_img_o : checked_img_x;
+	  int color = _fillRect ? kWidFrameColor : kCheckColor;
+	  fb.drawBitmap(img, _x + 3, _y + _boxY + 3, color);
+    }
   }
   else
-    fb.fillRect(_x + 2, _y + _boxY + 2, 10, 10, kBGColor);
+    fb.fillRect(_x + 2, _y + _boxY + 2, 10, 10, kColor);
 
   // Finally draw the label
   fb.drawString(_font, _label, _x + 20, _y + _textY, _w,
-                isEnabled() ? _color : kColor);
+                isEnabled() ? kTextColor : kColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SliderWidget::SliderWidget(GuiObject *boss, const GUI::Font& font,
                            int x, int y, int w, int h,
-                           const string& label, int labelWidth, int cmd, uInt8 hotkey)
-  : ButtonWidget(boss, font, x, y, w, h, label, cmd, hotkey),
+                           const string& label, int labelWidth, int cmd)
+  : ButtonWidget(boss, font, x, y, w, h, label, cmd),
     _value(0),
-    _oldValue(0),
+    _stepValue(1),
     _valueMin(0),
     _valueMax(100),
     _isDragging(false),
@@ -564,6 +543,8 @@ SliderWidget::SliderWidget(GuiObject *boss, const GUI::Font& font,
 {
   _flags = WIDGET_ENABLED | WIDGET_TRACK_MOUSE | WIDGET_CLEARBG;
   _type = kSliderWidget;
+  _bgcolor = kDlgColor;
+  _bgcolorhi = kDlgColor;
 
   if(!_label.empty() && _labelWidth == 0)
     _labelWidth = _font->getStringWidth(_label);
@@ -574,8 +555,33 @@ SliderWidget::SliderWidget(GuiObject *boss, const GUI::Font& font,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SliderWidget::setValue(int value)
 {
-  _value = value;
-  setDirty(); draw();
+  if(value < _valueMin)      value = _valueMin;
+  else if(value > _valueMax) value = _valueMax;
+
+  if(value != _value)
+  {
+    _value = value; 
+    setDirty(); draw();
+    sendCommand(_cmd, _value, _id);
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SliderWidget::setMinValue(int value)
+{
+  _valueMin = value;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SliderWidget::setMaxValue(int value)
+{
+  _valueMax = value;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void SliderWidget::setStepValue(int value)
+{
+  _stepValue = value;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -584,21 +590,7 @@ void SliderWidget::handleMouseMoved(int x, int y, int button)
   // TODO: when the mouse is dragged outside the widget, the slider should
   // snap back to the old value.
   if(isEnabled() && _isDragging && x >= (int)_labelWidth)
-  {
-    int newValue = posToValue(x - _labelWidth);
-
-    if(newValue < _valueMin)
-      newValue = _valueMin;
-    else if (newValue > _valueMax)
-      newValue = _valueMax;
-
-    if(newValue != _value)
-    {
-      _value = newValue; 
-      setDirty(); draw();
-      sendCommand(_cmd, _value, _id);
-    }
-  }
+    setValue(posToValue(x - _labelWidth));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -620,6 +612,41 @@ void SliderWidget::handleMouseUp(int x, int y, int button, int clickCount)
   _isDragging = false;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool SliderWidget::handleEvent(Event::Type e)
+{
+  if(!isEnabled())
+    return false;
+
+  switch(e)
+  {
+    case Event::UILeft:
+    case Event::UIDown:
+    case Event::UIPgDown:
+      setValue(_value - _stepValue);
+      break;
+  
+    case Event::UIRight:
+    case Event::UIUp:
+    case Event::UIPgUp:
+      setValue(_value + _stepValue);
+      break;
+
+    case Event::UIHome:
+      setValue(_valueMin);
+      break;
+
+    case Event::UIEnd:
+      setValue(_valueMax);
+      break;
+
+    default:
+      return false;
+      break;
+  }
+  return true;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void SliderWidget::drawWidget(bool hilite)
 {
@@ -628,25 +655,35 @@ void SliderWidget::drawWidget(bool hilite)
   // Draw the label, if any
   if(_labelWidth > 0)
     fb.drawString(_font, _label, _x, _y + 2, _labelWidth,
-                  isEnabled() ? _color : kColor, kTextAlignRight);
+                  isEnabled() ? kTextColor : kColor, kTextAlignRight);
 
   // Draw the box
   fb.box(_x + _labelWidth, _y, _w - _labelWidth, _h, kColor, kShadowColor);
 
+  // Fill the box
+  fb.fillRect(_x + _labelWidth + 2, _y + 2, _w - _labelWidth - 4, _h - 4,
+              !isEnabled() ? kColor : kWidColor);
+
   // Draw the 'bar'
   fb.fillRect(_x + _labelWidth + 2, _y + 2, valueToPos(_value), _h - 4,
               !isEnabled() ? kColor :
-              hilite ? kTextColorHi : _color);
+              hilite ? kSliderColorHi : kSliderColor);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int SliderWidget::valueToPos(int value)
 {
+  if(value < _valueMin)      value = _valueMin;
+  else if(value > _valueMax) value = _valueMax;
+
   return ((_w - _labelWidth - 4) * (value - _valueMin) / (_valueMax - _valueMin));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int SliderWidget::posToValue(int pos)
 {
-  return (pos) * (_valueMax - _valueMin) / (_w - _labelWidth - 4) + _valueMin;
+  int value = (pos) * (_valueMax - _valueMin) / (_w - _labelWidth - 4) + _valueMin;
+
+  // Scale the position to the correct interval (according to step value)
+  return value - (value % _stepValue);
 }
