@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsUNIX.cxx,v 1.17 2006/03/06 02:26:16 stephena Exp $
+// $Id: SettingsUNIX.cxx,v 1.24 2008/02/06 13:45:24 stephena Exp $
 //============================================================================
 
 #include "bspf.hxx"
@@ -24,9 +24,9 @@
 SettingsUNIX::SettingsUNIX(OSystem* osystem)
   : Settings(osystem)
 {
-  // This argument is only valid for Linux/UNIX, and will eventually be removed
-  setInternal("accurate", "false");
   setInternal("gl_lib", "libGL.so");
+  // Most Linux GL implementations don't support this yet
+  setInternal("gl_vsync", "false");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
