@@ -8,12 +8,12 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNodeWin32.cxx,v 1.7 2005/09/11 15:44:51 stephena Exp $
+// $Id: FSNodeWin32.cxx,v 1.14 2008/03/14 23:52:17 stephena Exp $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -282,16 +282,8 @@ bool AbstractFilesystemNode::makeDir(const string& path)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string AbstractFilesystemNode::modTime(const string& path)
+bool AbstractFilesystemNode::renameFile(const string& oldfile,
+                                        const string& newfile)
 {
-  WIN32_FILE_ATTRIBUTE_DATA attr;
-  BOOL b = GetFileAttributesEx(path.c_str(), GetFileExInfoStandard, &attr);
-
-  if(b == 0)
-    return "";
-
-  ostringstream buf;
-  buf << attr.ftLastWriteTime.dwHighDateTime << attr.ftLastWriteTime.dwLowDateTime;
-
-  return buf.str();
+  return MoveFile(oldfile.c_str(), newfile.c_str()) != 0;
 }
