@@ -8,18 +8,14 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2008 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: SettingsWin32.cxx,v 1.22 2006/03/06 21:41:58 stephena Exp $
+// $Id: SettingsWin32.cxx,v 1.29 2008/05/04 17:16:39 stephena Exp $
 //============================================================================
 
-#include <sstream>
-#include <fstream>
-#include <direct.h>
-
 #include "bspf.hxx"
 #include "Settings.hxx"
 #include "SettingsWin32.hxx"
@@ -28,10 +24,12 @@
 SettingsWin32::SettingsWin32(OSystem* osystem)
   : Settings(osystem)
 {
-  setInternal("fragsize", "2048");     // Anything less than this usually causes sound skipping
-  setInternal("video", "hard");        // Use software mode with hardware surface
-  setInternal("dirtyrects", "false");  // Most Windows systems work better without this
   setInternal("gl_lib", "opengl32.dll");
+  // Anything less than this usually causes sound skipping
+  setInternal("fragsize", "2048");
+  // Most Windows systems work better without this
+  setInternal("dirtyrects", "false");
+  setInternal("romdir", "c:\\");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
