@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
 // Windows CE Port by Kostas Nakos
+// $Id: FrameBufferWinCE.cpp,v 1.14 2007/02/12 11:16:59 knakos Exp $
 //============================================================================
 
 #include <windows.h>
@@ -24,11 +25,13 @@
 
 #define OPTPIXAVERAGE(pix1,pix2) ( ((((pix1 & optgreenmaskN) + (pix2 & optgreenmaskN)) >> 1) & optgreenmaskN) | ((((pix1 & optgreenmask) + (pix2 & optgreenmask)) >> 1) & optgreenmask) )
 
-FrameBufferWinCE::FrameBufferWinCE(OSystem *osystem)
-: FrameBuffer(osystem), myDstScreen(NULL), SubsystemInited(false), displacement(0),
-issmartphone(true), islandscape(false), displaymode(0), legacygapi(true), devres(SM_LOW)
+FrameBufferWinCE::FrameBufferWinCE(OSystem *osystem): FrameBuffer(osystem), 
+	myDstScreen(NULL), SubsystemInited(false), displacement(0), issquare(false),
+	issmartphone(true), islandscape(false), legacygapi(true), devres(SM_LOW), screenlocked(false)
 {
 	gxdp.cxWidth = gxdp.cyHeight = gxdp.cbxPitch = gxdp.cbyPitch = gxdp.cBPP = gxdp.ffFormat = 0;
+	displaymode = myOSystem->settings().getInt("wince_orientation");
+	if (displaymode > 2) displaymode = 0;
 }
 
 FrameBufferWinCE::~FrameBufferWinCE()
@@ -80,46 +83,79 @@ void FrameBufferWinCE::GetDeviceProperties(void)
 
 	if (gxdp.cxWidth == 176 && gxdp.cyHeight == 220)
 		devres = SM_LOW;
-	/*else if (gxdp.cxWidth == 480 && gxdp.cyHeight == 640)
-		devres = VGA;*/
+	else if (gxdp.cxWidth == 480 && gxdp.cyHeight == 640)
+		devres = VGA;
 	else
+	{
 		devres = QVGA;
+		// as a special case, qvga landscape devices are represented by the combination of:
+		// devres=QVGA && islandscape=true && displaymode=0
+		if (gxdp.cxWidth > gxdp.cyHeight)
+			islandscape = true;
+		// square QVGA (240x240) devices are portrait, landscape (as above) and square
+		else if (gxdp.cxWidth == gxdp.cyHeight)
+			issquare = islandscape = true;
+
+	}
 }
 
-void FrameBufferWinCE::setPalette(const uInt32* palette)
+void FrameBufferWinCE::setTIAPalette(const uInt32* palette)
 {
-	//setup palette
 	GetDeviceProperties();
 	for (uInt16 i=0; i<256; i++)
 	{
 		uInt8 r = (uInt8) ((palette[i] & 0xFF0000) >> 16);
 		uInt8 g = (uInt8) ((palette[i] & 0x00FF00) >> 8);
-		uInt8 b = (uInt8) (palette[i] & 0x0000FF);
+		uInt8 b = (uInt8)  (palette[i] & 0x0000FF);
 		if(gxdp.ffFormat & kfDirect565)
 			pal[i] = (uInt16) ( ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | ((b & 0xF8) >> 3) );
 		else if(gxdp.ffFormat & kfDirect555)
 			pal[i] = (uInt16) ( ((r & 0xF8) << 7) | ((g & 0xF8) << 3) | ((b & 0xF8) >> 3) );
 		else
 			return;
+		paldouble[i] = pal[i] | (pal[i] << 16);
+	}
+	SubsystemInited = false;
+}
+
+void FrameBufferWinCE::setUIPalette(const uInt32* palette)
+{
+	GetDeviceProperties();
+	for (int i=0; i<kNumColors - 256; i++)
+	{
+		uInt8 r = (uInt8) ((palette[i] & 0xFF0000) >> 16);
+		uInt8 g = (uInt8) ((palette[i] & 0x00FF00) >> 8);
+		uInt8 b = (uInt8)  (palette[i] & 0x0000FF);
+		if(gxdp.ffFormat & kfDirect565)
+			pal[i+256] = (uInt16) ( ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | ((b & 0xF8) >> 3) );
+		else if(gxdp.ffFormat & kfDirect555)
+			pal[i+256] = (uInt16) ( ((r & 0xF8) << 7) | ((g & 0xF8) << 3) | ((b & 0xF8) >> 3) );
+		else
+			return;
+		paldouble[i+256] = pal[i+256] | (pal[i+256] << 16);
 	}
 	SubsystemInited = false;
 }
 
 bool FrameBufferWinCE::initSubsystem()
 {
+	static bool firsttime = true;
+
 	GetDeviceProperties();
-	for (int i=0; i<kNumColors - 256; i++)
+
+	// we need to modify our basedim rect in order to center the ingame menu properly
+	// this is, more or less, a hack
+	if (IsSmartphoneLowRes())
 	{
-		uInt8 r = (ourGUIColors[i][0] & 0xFF);
-		uInt8 g = (ourGUIColors[i][1] & 0xFF);
-		uInt8 b = (ourGUIColors[i][2] & 0xFF);
-		if(gxdp.ffFormat & kfDirect565)
-			guipal[i] = (uInt16) ( ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | ((b & 0xF8) >> 3) );
-		else if(gxdp.ffFormat & kfDirect555)
-			guipal[i] = (uInt16) ( ((r & 0xF8) << 7) | ((g & 0xF8) << 3) | ((b & 0xF8) >> 3) );
-		else
-			return false;
+		myBaseDim.w = 220;
+		myBaseDim.h = 176;
 	}
+	else if (issquare)
+	{
+		myBaseDim.w = 240;
+		myBaseDim.h = 240;
+	}
+
 	// screen extents
 	if(gxdp.ffFormat & kfDirect565)
 	{
@@ -137,18 +173,18 @@ bool FrameBufferWinCE::initSubsystem()
 	scrpixelstep = gxdp.cbxPitch;
 	scrlinestep = gxdp.cbyPitch;
 
-//	if (scrwidth == 176 && scrheight == 220)
-//		issmartphone = true;
-//	else
-//		issmartphone = false;
-
 	setmode(displaymode);
+	if (!firsttime)
+		wipescreen(true);		// hold the 'initializing' screen on startup
+	firsttime = false;
 	SubsystemInited = false;
 	return true;
 }
 
 void FrameBufferWinCE::setmode(uInt8 mode)
 {
+	bool qvga_l = (displaymode == 0) && islandscape;
+	
 	displaymode = mode % 3;
 	switch (displaymode)
 	{
@@ -156,40 +192,48 @@ void FrameBufferWinCE::setmode(uInt8 mode)
 		case 0:
 			pixelstep = gxdp.cbxPitch;
 			linestep = gxdp.cbyPitch;
-			islandscape = false;
 			break;
 
 		// landscape
 		case 1:
 			pixelstep = - gxdp.cbyPitch;
 			linestep = gxdp.cbxPitch;
-			islandscape = true;
 			break;
 
 		// inverted landscape
 		case 2:
 			pixelstep = gxdp.cbyPitch;
 			linestep = - gxdp.cbxPitch;
-			islandscape = true;
 			break;
 	}
+	islandscape = displaymode || qvga_l;
 
+	pixelstepdouble = pixelstep << 1;
+	linestepdouble = linestep << 1;
 	pixelsteptimes5 = pixelstep * 5;
 	pixelsteptimes6 = pixelstep * 6;
+	pixelsteptimes8 = pixelstep * 8;
+	pixelsteptimes12 = pixelstep * 12;
+	pixelsteptimes16 = pixelstep * 16;
 	SubsystemInited = false;
 }
 
 uInt8 FrameBufferWinCE::rotatedisplay(void)
 {
-	displaymode = (displaymode + 1) % 3;
+	if (!(displaymode == 0 && islandscape))
+	{
+		islandscape = false;
+		displaymode = (displaymode + 1) % 3;
+	}
 	setmode(displaymode);
 	wipescreen();
+	myOSystem->settings().setInt("wince_orientation", displaymode);
 	return displaymode;
 }
 
 void FrameBufferWinCE::lateinit(void)
 {
-	int w,h;
+	int w, h;
 
 	myWidth = myOSystem->console().mediaSource().width();
 	myHeight = myOSystem->console().mediaSource().height();
@@ -200,18 +244,23 @@ void FrameBufferWinCE::lateinit(void)
 			w = myWidth;
 		else
 			w = (int) ((float) myWidth * 11.0f / 8.0f + 0.5f);
-	else //if (devres == QVGA)
-		if (!islandscape)
+	else if (devres == QVGA)
+		if (!islandscape && !issquare)
 			w = (int) ((float) myWidth * 3.0f / 2.0f + 0.5f);
 		else
 			w = myWidth * 2;
-	/*else
+	else
 	{
-		// VGA
+		if (!islandscape)
+			w = (int) ((float) myWidth * 3.0f + 0.5f);
+		else
+			w = myWidth * 4;
 	}
-	*/
+
 	if (devres == SM_LOW && islandscape)
 		h = (int) ((float) myHeight * 4.0f / 5.0f + 0.5f);
+	else if (devres == VGA)
+		h = myHeight * 2;
 	else
 		h = myHeight;
 
@@ -259,6 +308,12 @@ void FrameBufferWinCE::lateinit(void)
 
 	}
 
+	if (devres == VGA)
+	{
+		minydim >>= 1;
+		displacement &= ~3;		// ensure longword alignment
+	}
+
 	SubsystemInited = true;
 }
 
@@ -267,6 +322,9 @@ void FrameBufferWinCE::preFrameUpdate()
 	static HDC hdc;
 	static RawFrameBufferInfo rfbi;
 
+	if (screenlocked)
+		return;
+
 	if (legacygapi)
 		myDstScreen = (uInt8 *) GXBeginDraw();
 	else
@@ -276,13 +334,16 @@ void FrameBufferWinCE::preFrameUpdate()
 		ReleaseDC(NULL, hdc);
 		myDstScreen = (uInt8 *) rfbi.pFramePointer;
 	}
+
+	screenlocked = true;
 }
 
 void FrameBufferWinCE::drawMediaSource()
 {
 	static uInt8 *sc, *sp, *sc_n, *sp_n;
-	static uInt8 *d, *pl, *p;
+	static uInt8 *d, *pl, *nl;
 	static uInt16 pix1, pix2, pix3, x, y;
+	uInt32 pix1d, pix2d, pix3d;
 
 	if (!SubsystemInited)
 		lateinit();
@@ -297,9 +358,8 @@ void FrameBufferWinCE::drawMediaSource()
 
 	if (theRedrawTIAIndicator)
 	{
-		p = sp;
-		for (uInt16 y=0; y<myHeight; y++)
-			for (uInt16 x=0; x<myWidth; x += 4, *p = *p + 1, p += 4);
+		memset(sp, 0, myWidth*myHeight-1);
+		memset(myDstScreen, 0, scrwidth*scrheight*2);
 		theRedrawTIAIndicator = false;
 	}
 	
@@ -445,7 +505,7 @@ void FrameBufferWinCE::drawMediaSource()
 			pl = d;
 		}
 	}
-	else if (devres == QVGA && !islandscape)
+	else if (devres == QVGA && (!islandscape || issquare))
 	{
 		// 3/2
 		for (y=0; y<minydim; y++)
@@ -498,7 +558,7 @@ void FrameBufferWinCE::drawMediaSource()
 				else
 				{
 					sc += 4;
-					d += (pixelstep << 3);
+					d += pixelsteptimes8;
 				}
 				sp += 4;
 			}
@@ -506,233 +566,403 @@ void FrameBufferWinCE::drawMediaSource()
 			pl = d;
 		}
 	}
+	else if (devres == VGA && !islandscape)
+	{
+		for (y=0; y<minydim; y++)
+		{
+			// 2/1
+			for (x=0; x<myWidthdiv4; x++)
+			{
+				// 3/1
+				if ( *((uInt32 *) sc) != *((uInt32 *) sp) )
+				{
+					nl = d + linestep;
+					pix1d = paldouble[*sc++]; pix2d = paldouble[*sc++];
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					pix3d = ((uInt16) pix1d) | (((uInt16) pix2d) << 16);
+					*((uInt32 *)d) = pix3d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix3d; nl += pixelstepdouble;
+					*((uInt32 *)d) = pix2d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix2d; nl += pixelstepdouble;
+					pix1d = paldouble[*sc++]; pix2d = paldouble[*sc++];
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					pix3d = ((uInt16) pix1d) | (((uInt16) pix2d) << 16);
+					*((uInt32 *)d) = pix3d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix3d; nl += pixelstepdouble;
+					*((uInt32 *)d) = pix2d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix2d;
+				}
+				else
+				{
+					sc += 4;
+					d += pixelsteptimes12;
+				}
+				sp += 4;
+			}
+			d = pl + linestepdouble;
+			pl = d;
+		}
+	}
+	else if (devres == VGA && islandscape)
+	{
+		for (y=0; y<minydim; y++)
+		{
+			// 2/1
+			for (x=0; x<myWidthdiv4; x++)
+			{
+				// 4/1
+				if ( *((uInt32 *) sc) != *((uInt32 *) sp) )
+				{
+					nl = d + pixelstep;
+					pix1d = paldouble[*sc++];
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					pix1d = paldouble[*sc++];
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					pix1d = paldouble[*sc++];
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					pix1d = paldouble[*sc++];
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d; nl += pixelstepdouble;
+					*((uInt32 *)d) = pix1d; d += pixelstepdouble;
+					*((uInt32 *)nl) = pix1d;
+				}
+				else
+				{
+					sc += 4;
+					d += pixelsteptimes16;
+				}
+				sp += 4;
+			}
+			d = pl + linestepdouble;
+			pl = d;
+		}
+	}
 
 }
 
-void FrameBufferWinCE::wipescreen(void)
+void FrameBufferWinCE::wipescreen(bool atinit)
 {
-	if (!SubsystemInited)
-		lateinit();
+	if (!atinit)
+	{
+		if (!SubsystemInited)
+			lateinit();
 
-	uInt8 *s = myOSystem->console().mediaSource().currentFrameBuffer();
-	memset(s, 0, myWidth*myHeight-1);
-	s = myOSystem->console().mediaSource().previousFrameBuffer();
-	memset(s, 0, myWidth*myHeight-1);
+		uInt8 *s = myOSystem->console().mediaSource().currentFrameBuffer();
+		memset(s, 0, myWidth*myHeight-1);
+		s = myOSystem->console().mediaSource().previousFrameBuffer();
+		memset(s, 0, myWidth*myHeight-1);
+	}
 
 	preFrameUpdate();
-	//uInt8 *d;
-	//d=myDstScreen;
-	//for (int i=0; i < scrwidth*scrheight; i++, *((uInt16 *)d) = 0, d += scrpixelstep);
 	memset(myDstScreen, 0, scrwidth*scrheight*2);
 	postFrameUpdate();
 }
 
 void FrameBufferWinCE::postFrameUpdate()
 {
-	if (legacygapi) GXEndDraw();
+	if (!screenlocked)	return;
+	if (legacygapi)		GXEndDraw();
 }
 
-void FrameBufferWinCE::drawChar(const GUI::Font* font, uInt8 c, uInt32 x, uInt32 y, OverlayColor color)
+void FrameBufferWinCE::drawChar(const GUI::Font* myfont, uInt8 c, uInt32 x, uInt32 y, int color)
 {
-  GUI::Font* myfont = (GUI::Font*)font;
-  const FontDesc& desc = myfont->desc();
+	const FontDesc& desc = myfont->desc();
 
-  if (!myDstScreen) return;
+	if (!myDstScreen) return;
 
-  if(c < desc.firstchar || c >= desc.firstchar + desc.size)
-  {
-    if (c == ' ')
-      return;
-    c = desc.defaultchar;
-  }
-
-  Int32 w = myfont->getCharWidth(c);
-  const Int32 h = myfont->getFontHeight();
-  c -= desc.firstchar;
-  const uInt16* tmp = desc.bits + (desc.offset ? desc.offset[c] : (c * h));
-
-  if (x<0 || y<0 || (x>>1)+w>scrwidth || y+h>scrheight) return;
-
-  uInt8 *d;
-  uInt32 stride;
-  if (devres == SM_LOW)
-  {
-	  d = myDstScreen + y * scrlinestep + ((x+1) >> 1) * scrpixelstep;
-	  stride = (scrwidth - w) * scrpixelstep;
-  }
-  else
-  {
-	  if (displaymode != 2)
-		d = myDstScreen + (scrheight-x-1) * scrlinestep + (y-1) * scrpixelstep;
-	  else
-		d = myDstScreen + x * scrlinestep + (scrwidth-y-1) * scrpixelstep;
-  }
-
-  uInt16 col = guipal[((int) color) - 256];
-
-  for(int y2 = 0; y2 < h; y2++)
-  {
-    const uInt16 buffer = *tmp++;
-	if (devres == SM_LOW)
+	if (c < desc.firstchar || c >= desc.firstchar + desc.size)
 	{
-		uInt16 mask = 0xC000;
-		for(int x2 = 0; x2 < w; x2++, mask >>= 2)
-		{
-		  if (buffer & mask)
-			  *((uInt16 *)d) = col;
-		  d += scrpixelstep;
-		}
-		d += stride;
+		if (c == ' ')
+			return;
+		c = desc.defaultchar;
+	}
+
+	Int32 w = myfont->getCharWidth(c);
+	const Int32 h = myfont->getFontHeight();
+	c -= desc.firstchar;
+	const uInt16* tmp = desc.bits + (desc.offset ? desc.offset[c] : (c * h));
+
+	if ((Int32)x<0 || (Int32)y<0 || (x>>1)+w>scrwidth || y+h>scrheight) return;
+
+	uInt8 *d;
+	if (devres != VGA)
+	{
+		if (!displaymode && islandscape)
+			d = myDstScreen + y * scrlinestep + x * scrpixelstep;
+		else if (displaymode != 2)
+			d = myDstScreen + (scrheight-x) * scrlinestep + y * scrpixelstep;
+		else
+			d = myDstScreen + x * scrlinestep + (scrwidth-y) * scrpixelstep;
 	}
 	else
 	{
-		uInt16 mask = 0x8000;
-		uInt8 *tmp = d;
-		for(int x2 = 0; x2 < w; x2++, mask >>= 1)
-		{
-		  if (buffer & mask)
-			  *((uInt16 *)d) = col;
-		  if (displaymode != 2)
-			d -= scrlinestep;
-		  else
-			d += scrlinestep;
-		}
 		if (displaymode != 2)
-			d = tmp + scrpixelstep;
+			d = myDstScreen + ((scrheight>>1)-x-1) * (scrlinestep<<1) + y * (scrpixelstep << 1);
 		else
-			d = tmp - scrpixelstep;
+			d = myDstScreen + x * (scrlinestep<<1) + ((scrwidth>>1)-y) * (scrpixelstep<<1);
 	}
-  }
-}
 
-void FrameBufferWinCE::scanline(uInt32 row, uInt8* data)
-{
-	return;
-}
+	uInt16 col = pal[color];
+	uInt32 cold = paldouble[color];
 
-void FrameBufferWinCE::setAspectRatio()
-{
-	return;
-}
-
-bool FrameBufferWinCE::createScreen()
-{
-	return true;
-}
-
-void FrameBufferWinCE::toggleFilter()
-{
-	return;
-}
-
-uInt32 FrameBufferWinCE::mapRGB(Uint8 r, Uint8 g, Uint8 b)
-{
-	return 0xFFFFFFFF;
-}
-
-void FrameBufferWinCE::hLine(uInt32 x, uInt32 y, uInt32 x2, OverlayColor color)
-{
-	if (devres == SM_LOW)
+	for (int y2 = 0; y2 < h; y2++)
 	{
-		int kx = x >> 1; int ky = y; int kx2 = x2>> 1;
-		if (kx<0) kx=0; if (ky<0) ky=0; if (ky>scrheight-1) return; if (kx2>scrwidth-1) kx2=scrwidth-1;
-		PlothLine(kx, ky, kx2, color);
+		const uInt16 buffer = *tmp++;
+		if (devres != VGA)
+		{
+			uInt16 mask = 0x8000;
+			uInt8 *tmp = d;
+			for (int x2 = 0; x2 < w; x2++, mask >>= 1)
+			{
+				if (buffer & mask)
+					*((uInt16 *)d) = col;
+				if (!displaymode && islandscape)
+					d += scrpixelstep;
+				else if (displaymode != 2)
+					d -= scrlinestep;
+				else
+					d += scrlinestep;
+			}
+			if (!displaymode && islandscape)
+				d = tmp + scrlinestep;
+			else if (displaymode != 2)
+				d = tmp + scrpixelstep;
+			else
+				d = tmp - scrpixelstep;
+		}
+		else
+		{
+			uInt16 mask = 0x8000;
+			uInt8 *tmp = d;
+			for (int x2 = 0; x2 < w; x2++, mask >>= 1)
+			{
+				if (buffer & mask)
+				{
+					*((uInt32 *)d) = cold;
+					*((uInt32 *)(d+scrlinestep)) = cold;
+				}
+				if (displaymode != 2)
+					d -= (scrlinestep<<1);
+				else
+					d += (scrlinestep<<1);
+			}
+			if (displaymode != 2)
+				d = tmp + (scrpixelstep<<1);
+			else
+				d = tmp - (scrpixelstep<<1);
+		}
 	}
+}
+
+void FrameBufferWinCE::hLine(uInt32 x, uInt32 y, uInt32 x2, int color)
+{
+	if (devres != VGA)
+		if (!displaymode && islandscape)
+			PlothLine(x, y, x2, color);
+		else if (displaymode != 2)
+			PlotvLine(y, scrheight-x, scrheight-x2, color);
+		else
+			PlotvLine(scrwidth-y, x, x2, color);
 	else
 		if (displaymode != 2)
-			PlotvLine(y, scrheight-x, scrheight-x2-1, color);
+		{
+			PlotvLine((y<<1), (((scrheight>>1)-x)<<1)-1, (((scrheight>>1)-x2)<<1)-1, color);
+			PlotvLine((y<<1)+1, (((scrheight>>1)-x)<<1)-1, (((scrheight>>1)-x2)<<1)-1, color);
+		}
 		else
-			PlotvLine(scrwidth-y-1, x, x2+1, color);
+		{
+			PlotvLine(((scrwidth>>1)-y)<<1, x<<1, x2<<1, color);
+			PlotvLine((((scrwidth>>1)-y)<<1)+1, x<<1, x2<<1, color);
+		}
 }
 
-void FrameBufferWinCE::PlothLine(uInt32 x, uInt32 y, uInt32 x2, OverlayColor color)
+void FrameBufferWinCE::PlothLine(uInt32 x, uInt32 y, uInt32 x2, int color)
 {
 	if (!myDstScreen) return;
 	if (x>x2) { x2 ^= x; x ^= x2; x2 ^= x;} //lazy swap
 	uInt8 *d = myDstScreen + y * scrlinestep + x * scrpixelstep;
-	uInt16 col = guipal[((int) color) - 256];
-	for (;x < x2; x++, *((uInt16 *)d) = col, d += scrpixelstep);
+	uInt16 col = pal[color];
+	for (;x <= x2; x++, *((uInt16 *)d) = col, d += scrpixelstep);
 }
 
-void FrameBufferWinCE::vLine(uInt32 x, uInt32 y, uInt32 y2, OverlayColor color)
+void FrameBufferWinCE::vLine(uInt32 x, uInt32 y, uInt32 y2, int color)
 {
-	if (devres == SM_LOW)
-	{
-		int kx = x >> 1; int ky = y; int ky2 = y2;
-		if (kx<0) kx=0; if (ky<0) ky=0; if (kx>scrwidth-1) return; if (ky>scrheight-1) ky=scrheight-1; if (ky2>scrheight-1) ky2=scrheight-1;
-		PlotvLine(kx, ky, ky2, color);
-	}
-	else
-		if (displaymode != 2)
-			PlothLine(y, scrheight-x-1, y2, color);
+	if (devres != VGA)
+		if (!displaymode && islandscape)
+			PlotvLine(x, y, y2, color);
+		else if (displaymode != 2)
+			PlothLine(y, scrheight-x, y2, color);
 		else
 			PlothLine(scrwidth-y, x, scrwidth-y2, color);
-
+	else
+		if (displaymode != 2)
+		{
+			PlothLine(y<<1, (((scrheight>>1)-x)<<1)-2, y2<<1, color);
+			PlothLine(y<<1, (((scrheight>>1)-x)<<1)-1, y2<<1, color);
+		}
+		else
+		{
+			PlothLine(((scrwidth>>1)-y)<<1, x<<1, ((scrwidth>>1)-y2)<<1, color);
+			PlothLine(((scrwidth>>1)-y)<<1, (x<<1)+1, ((scrwidth>>1)-y2)<<1, color);
+		}
 }
 
-void FrameBufferWinCE::PlotvLine(uInt32 x, uInt32 y, uInt32 y2, OverlayColor color)
+void FrameBufferWinCE::PlotvLine(uInt32 x, uInt32 y, uInt32 y2, int color)
 {
 	if (y>y2) { y2 ^= y; y ^= y2; y2 ^= y;} //lazy swap
 	if (!myDstScreen) return;
 	uInt8 *d = myDstScreen + y * scrlinestep + x * scrpixelstep;
-	uInt16 col = guipal[((int) color) - 256];
-	for (;y < y2; y++, *((uInt16 *)d) = col, d += scrlinestep);
+	uInt16 col = pal[color];
+	for (;y <= y2; y++, *((uInt16 *)d) = col, d += scrlinestep);
 }
 
-void FrameBufferWinCE::fillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, OverlayColor color)
+void FrameBufferWinCE::fillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, int color)
 {
-	if (devres == SM_LOW)
-	{
-		int kx = x >> 1; int ky = y; int kw = (w >> 1); int kh = h;
-		if (ky>scrheight-1) return; if (kx>scrwidth-1) return;
-		if (kx<0) kx=0; if (ky<0) ky=0;if (kw<0) kw=0; if (kh<0) kh=0;
-		if (kx+kw>scrwidth-1) kw=scrwidth-kx-1; if (ky+kh>scrheight-1) kh=scrheight-ky-1;
-		PlotfillRect(kx, ky, kw, kh, color);
-	}
-	else
+	if (w==0 || h==0) return;
+	if (devres != VGA)
 	{
+		if (!displaymode && islandscape)
+		{
+			PlotfillRect(x, y, w, h, color);
+			return;
+		}
 		if (x>scrheight) return; if (y>scrwidth) return;
 		if (x+w>scrheight) w=scrheight-x; if (y+h>scrwidth) h=scrwidth-y;
 		if (displaymode != 2)
-			PlotfillRect(y, scrheight-x-w, h-1, w-1, color);
+			PlotfillRect(y, scrheight-x-w+1, h, w, color);
 		else
-			PlotfillRect(scrwidth-y-h, x, h-1, w-1, color);
+			PlotfillRect(scrwidth-y-h+1, x, h, w, color);
+	}
+	else
+	{
+		if ((int)x>(scrheight>>1)) return; if ((int)y>(scrwidth>>1)) return;
+		if ((int)(x+w)>(scrheight>>1)) w=(scrheight>>1)-x; if ((int)(y+h)>(scrwidth>>1)) h=(scrwidth>>1)-y;
+		if (displaymode != 2)
+			PlotfillRect(y<<1, (((scrheight>>1)-x-w+1)<<1)-2, h<<1, w<<1, color);
+		else
+			PlotfillRect(((scrwidth>>1)-y-h+1)<<1, x<<1, h<<1, w<<1, color);
 	}
 
 }
 
-void FrameBufferWinCE::PlotfillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, OverlayColor color)
+void FrameBufferWinCE::PlotfillRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h, int color)
 {
 	if (!myDstScreen) return;
 	uInt8 *d = myDstScreen + y * scrlinestep + x * scrpixelstep;
-	uInt16 col = guipal[((int) color) - 256];
-	uInt32 stride = (scrwidth - w - 1) * scrpixelstep;
-	for (h++; h != 0; h--, d += stride)
-		for (int w2=w; w2>=0; w2--, *((uInt16 *)d) = col, d += scrpixelstep);
+	uInt16 col = pal[color];
+	uInt32 stride = (scrwidth - w) * scrpixelstep;
+	for (;h != 0; h--, d += stride)
+		for (int w2=w; w2>0; w2--, *((uInt16 *)d) = col, d += scrpixelstep);
 }
 
-void FrameBufferWinCE::drawBitmap(uInt32* bitmap, Int32 x, Int32 y, OverlayColor color, Int32 h)
+void FrameBufferWinCE::drawBitmap(uInt32* bitmap, Int32 x, Int32 y, int color, Int32 h)
 {
-	return;
+	uInt8 *d;
+	uInt16 col;
+	uInt32 cold;
+
+	if (!myDstScreen) return;
+
+	if (devres != VGA)
+		if (!displaymode && islandscape)
+			d = myDstScreen + y * scrlinestep + x * scrpixelstep;
+		else if (displaymode != 2)
+			d = myDstScreen + (scrheight-x) * scrlinestep + y * scrpixelstep;
+		else
+			d = myDstScreen + x * scrlinestep + (scrwidth-y) * scrpixelstep;
+	else
+		if (displaymode != 2)
+			d = myDstScreen + ((scrheight>>1)-x-1) * (scrlinestep<<1) + y * (scrpixelstep<<1);
+		else
+			d = myDstScreen + x * (scrlinestep<<1) + ((scrwidth>>1)-y) * (scrpixelstep<<1);
+
+	col = pal[color];
+	cold = paldouble[color];
+	for (int i = 0; i < h; i++)
+	{
+		uInt32 mask = 0xF0000000;
+		uInt8 *tmp = d;
+		
+		for (int j = 0; j < 8; j++, mask >>= 4)
+		{
+			if(bitmap[i] & mask)
+			{
+				if (devres != VGA)
+					*((uInt16 *)d) = col;
+				else
+				{
+					*((uInt32 *)d) = cold;
+					*((uInt32 *)(d+scrlinestep)) = cold;
+				}
+			}
+			if (devres != VGA)
+			{
+				if (!displaymode && islandscape)
+					d += scrpixelstep;
+				else if (displaymode != 2)
+					d -= scrlinestep;
+				else
+					d += scrlinestep;
+			}
+			else
+			{
+				if (displaymode != 2)
+					d -= (scrlinestep<<1);
+				else
+					d += (scrlinestep<<1);
+			}
+		}
+
+		if (devres != VGA)
+		{
+			if (!displaymode && islandscape)
+				d = tmp + scrlinestep;
+			else if (displaymode != 2)
+				d = tmp + scrpixelstep;
+			else
+				d = tmp - scrpixelstep;
+		}
+		else
+		{
+			if (displaymode != 2)
+				d = tmp + (scrpixelstep<<1);
+			else
+				d = tmp - (scrpixelstep<<1);
+		}
+	}
 }
 
 void FrameBufferWinCE::translateCoords(Int32* x, Int32* y)
 {
-	if (!issmartphone)
+	if ((displaymode == 1) || (displaymode==0 && !islandscape && myOSystem->eventHandler().state() != EventHandler::S_EMULATE))
 	{
-		if ((displaymode == 1) || (displaymode==0 && myOSystem->eventHandler().state() != 1))
-		{
-			Int32 x2 = *x;
-			*x = scrheight - *y;
-			*y = x2;
-		}
-		else if (displaymode == 2)
-		{
-			Int32 x2 = *x;
-			*x = *y;
-			*y = scrwidth - x2;
-		}
+		Int32 x2 = *x;
+		*x = scrheight - *y;
+		*y = x2;
 	}
+	else if (displaymode == 2)
+	{
+		Int32 x2 = *x;
+		*x = *y;
+		*y = scrwidth - x2;
+	}
+	if (devres == VGA)
+	{
+		*x >>= 1;
+		*y >>= 1;
+	}
+
 	return;
 }
 
@@ -740,21 +970,20 @@ void FrameBufferWinCE::addDirtyRect(uInt32 x, uInt32 y, uInt32 w, uInt32 h)
 {
 	static bool initflag = false;
 
-	if (myOSystem->eventHandler().state() == 3)
+	if (myOSystem->eventHandler().state() == EventHandler::S_MENU)
 		initflag = true;
 
-	if (myOSystem->eventHandler().state()==1 && initflag)
-	{
-		// TODO: optimize here
-		theRedrawTIAIndicator = true;
-	}
+	if (myOSystem->eventHandler().state() == EventHandler::S_EMULATE && initflag)
+		theRedrawTIAIndicator = true;		// TODO: optimize here
 
 	return;
 }
 
-uInt32 FrameBufferWinCE::lineDim()
+string FrameBufferWinCE::about()
 {
-	return 1;
-}
-
-
+	string id = "Video rendering: ";
+	id += (issmartphone ? "SM " : "PPC ");
+	id += (legacygapi ? "GAPI " : "Direct ");
+	id += (devres == SM_LOW ? "176x220\n" : (devres == QVGA ? "240x320\n" : "480x640\n"));
+	return id;
+}
\ No newline at end of file
