@@ -8,12 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2005 by Bradford W. Mott and the Stella team
+// Copyright (c) 1995-2007 by Bradford W. Mott and the Stella team
 //
 // See the file "license" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
 // Windows CE Port by Kostas Nakos
+// $Id: SettingsWinCE.cpp,v 1.8 2007/01/27 10:51:04 knakos Exp $
 //============================================================================
 
 #include <sstream>
@@ -26,8 +27,8 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 SettingsWinCE::SettingsWinCE(OSystem* osystem) : Settings(osystem) 
 {
-  setInternal("romdir", (string) getcwd() + '\\');
-  setInt("wce_smartphone_paddlespeed", 3000);
+  setInternal("romdir", (string) getcwd() + "\\Roms\\");
+  setInternal("wince_orientation", "0");
 }
 
 SettingsWinCE::~SettingsWinCE()
